//StarCatV, Star Catalog Visualization Tool
// (C) 1999  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


//	SCcmdw.h:	Header for SCcmdw class
//=======================================================================

#ifndef SCCMDW_H
#define SCCMDW_H

#include <v/vcmdwin.h>	// So we can use vCmdWindow
#include <v/vmenu.h>	// For the menu pane
#include <v/vutil.h>	// For V Utilities
#include <v/vcmdpane.h> // command pane
#include <v/vstatusp.h>	// For the status pane
#include <v/vtimer.h>	// Timer



#include<v/vfilesel.h>

#ifdef vDEBUG
#include <v/vdebug.h>

//#include <time.h>
#endif

#include "TimeLdlg.h"
#include "SCmdlgTime.h"

#include "SCcnv.h"	// SCCanvasPane
#include "SCdlg.h"	// SCDialog
#include "SCmdlg.h"	// SCModalDialog
#include "ProgBardlg.h"
//#include "Timeldlg.h"
//#######################
#include "StarCatalog.h"

//#include "SCmdlg.h"	// SCModalDialog

//#######################

    class SCCmdWindow;

    class SCAuxTimer : public vTimer
      {
      public:		//---------------------------------------- public
	SCAuxTimer(SCCmdWindow* cw) { cmdw = cw; }
	~SCAuxTimer() {}
	virtual void TimerTick();
      private:		//--------------------------------------- private
	SCCmdWindow* cmdw;
      };


	//
    class SCCmdWindow : public vCmdWindow
      {
	friend int AppMain(int, char**);	// allow AppMain access

      public:		//---------------------------------------- public

//#######################
	//Classes
	StarCatalog	SC;
	CVMath VM;

	//methods
	void UpDateToolBar1(int n);
	void UpDateToolBar2(int n);
	void UpDateToolBarS(int n);
	void SetRADecfromAltAz();
	void SetJulian();
	void PlotView();
	void PrintPlotView(int Dest);
	void ZoomOut();
	void ZoomIn();
	void OnVFMessage();
	void ClearMagChecks();
	int LoadBSCat();
	int LoadHipCat();
	int LoadTycCat();
	void BuildBSCatCond();
	void BuildHipCatCond();
	void BuildTycCatCond();
	int LoadConLines();
	int LoadConNames();
	int LoadStarNames();
	int GetRecordData(int index);
	void SetTimeNow();
	void SetTimeDelta(double Days);
	void Setvm_TZLabel(double TZOffset);

	void OnOpenSet();
	void OnSaveSet();

	void CondRePlot();

	void OnTimer();
	void StopTimer();

	void GotoObject(int Object);
	void SetAutoBlob();

	void PrintSky();

	//printer stuff
	char PrintReport[400];


	//sys display vars
	STARDATA cZenith;
	STARDATA cViewPoint;

	//view conditions
	float	vc_RAh;
	float	vc_RAm;
	float	vc_Dec;
	float	vc_Rot;
	float	vc_VAng;
	float	vc_Mag;
	int		vc_Day;
	int		vc_Month;
	int		vc_Year;
	double	vc_Hour;
	double	vc_Alt;
	double	vc_Az;
	double	vc_JulianDate;
	double	vc_Minute;
	float	vc_LocLat;
	float	vc_LocLong;

	//view mode flags
	int vm_CSegMode;	//0=off,1==on
	int vm_RAGridMode;	//0=off, 1=on
	int vm_AltAzGridMode;//0=off, 1=on
	int vm_StarLabMode; //0=off,1=on
	int vm_LegendMode;		//0=off,1=on
	int vm_ViewMode;		//0=sky, 1=geo
	int vm_ViewCenterMode;	//0=off, 1 = on
	int vm_ConNameMode;
	int vm_StarNameMode;
	int vm_RecordMode;		//0=off,1=PRec,2=RawRec
	//int vm_AutoRecLoad;
	int vm_StarStyle;		//0=dot,1=,2=blob,3=med blob, 4= big blob
	int vm_AutoBlobMode;	//0=off,1=on
	float	vm_SymRefLev;
	float	vm_VCenDia;
	int vm_PlanetMode;//0=off,1=on
	int vm_PlanetLabel;
	int vm_Equinox;			//0=astromeric,1=J2000,2=D1950
	int vm_ColorMode;		//0=norm,1=whtonBlk,2=redonblk,3=black on white
	int vm_CentricMode;		//0=geocentric,1=topocentric
	int vm_Lock;			//0=unlocked
	int vm_LockedObject;	//Object locked onto.

	//int vm_TimeZone;		//Time Zone
	char vm_TZLabel[10];		//Time zone label
	char vm_LZT[30];		//zulu time string
	//int vm_DayLightTimeMode;//Daylight saveings time mode

	int vm_MousePos;		//0 = RAdec,1=Alt Az
	int vm_TimeZoneIndex;
	float vm_TimeZoneOffset;

	int vt_TimeLapseMode;	//0=off,1=0n
	int TimeLapseCnt;		//time lapse counter
	int vt_Intms;			//time interval, ms
	double vt_dJulian;		//time step julian

	//debug messages
	int DebugLevel;

	//Options
	int vo_ShowStatus;		//0=no status bar, 1=status bar;

	//int LastStarIndex;

	// pointer for ext access to plot function
	// -Add this pointer here.
	// -Add (Rootcmdw = this;) in constructor.
	SCCmdWindow* Rootcmdw;


//#######################


	SCCmdWindow(char*, int, int);
	virtual ~SCCmdWindow();
	virtual void WindowCommand(ItemVal id, ItemVal val, CmdType cType);
	virtual void KeyIn(vKey keysym, unsigned int shift);
	SCAuxTimer* GetSCAuxTimer() {return _auxTimer;}// Aux Timer

	 void OnMTest();

      protected:	//--------------------------------------- protected

      private:		//--------------------------------------- private

	// Standard elements
	vMenuPane* SCMenu;		// For the menu bar
	//SCCanvasPane* SCCanvas;		// For the canvas
	SCCanvasPane* VF;		// For the canvas
	vCommandPane* SCCmdPane;	// for the command pane
	vCommandPane* SCCmdPane2;
	vStatusPane* SCStatus;		// For the status bar
	SCAuxTimer* _auxTimer;	// Aux Timer

	ProgBarDialog* ProgBar;
	// Dialogs associated with CmdWindow

	SCDialog* StatDlg;
	SCModalDialog* LocDlg;

	CTimeDatedlg* TimeDlg;

	CTimeLdlg* TLDlg;

      };
#endif
