//StarCatV, Star Catalog Visualization Tool
// (C) 1999  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "VMath.h"
#include "StarCatalog.h"
#include <iostream.h>


StarCatalog::StarCatalog()
{
	
		//allocate mem block for star catalog data array
	//SCat = (float *)malloc((size_t)sizeof(float)(100000));
	SCat = (float *)malloc(10000*3*sizeof(float));
	SCatID = (int *)malloc(10000 * sizeof(int));
	SCatRecord = (int *)malloc(10000 * sizeof(int));
/*
	//block for relative file structure data file organization
	//data file names and locations
	//"..\\data\\stardata2.txt" find in parrellel dir
	//"data\\stardata2.txt" find in sub dir
	//strcpy(BSCondCatName ,"..\\StarCatsData\\BS_catalog_cond.dat");
	strcpy(BSCondCatName ,"..\\StarCatsData\\BS_catalog_cond.dat");	
	strcpy(BSCatName,"..\\BSCat\\BS_catalog.dat");
	strcpy(HipCondCatName,"..\\StarCatsData\\hip_main_cond.dat");
	strcpy(HipCatName,"..\\HipTycCat\\hip_main.dat");
	strcpy(TycCondCatName,"..\\StarCatsData\\tyc_main_cond.dat");
	strcpy(TycCatName,"..\\HipTycCat\\tyc_main.dat");
	strcpy(ConNamesFileName,"..\\StarCatsData\\ConNames.dat");
	strcpy(StarNamesFileName,"..\\StarCatsData\\StarNames.dat");
	strcpy(ConLinesFileName,"..\\StarCatsData\\ConLines.dat");
	strcpy(SettingsFileName,"..\\Sky1.sky");

*/

	//block for vers with all files in single dir
	strcpy(BSCondCatName ,"BS_catalog_cond.dat");	
	strcpy(BSCatName,"BS_catalog.dat");
	strcpy(HipCondCatName,"hip_main_cond.dat");
	strcpy(HipCatName,"hip_main.dat");
	strcpy(TycCondCatName,"tyc_main_cond.dat");
	strcpy(TycCatName,"tyc_main.dat");
	strcpy(ConNamesFileName,"ConNames.dat");
	strcpy(StarNamesFileName,"StarNames.dat");
	strcpy(ConLinesFileName,"ConLines.dat");
	strcpy(SettingsFileName,"Sky1.sky");


	ResolvePaths();

	//load default catalog
	//LoadBSCat();
	//LoadConLines();

}

StarCatalog::~StarCatalog()
{
	 //free mallocs
	free(SCat);
	free(SCatID);
	free(SCatRecord);
  

}

//////////////////////////////////////////////////////////
//GetConNPos(int n)
//
//
//////////////////////////////////////////////////////////
STARDATA StarCatalog::GetConNPos(int n)
{
	STARDATA ret;

	ret.RA = ConNameLoc[n][0];
	ret.Dec = ConNameLoc[n][1];
	ret.Mag = 0;

	return(ret);
}


//////////////////////////////////////////////////////////
//GetConName(int n)
//
//
//////////////////////////////////////////////////////////
char* StarCatalog::GetConName(char* res, int n) const
{
	int ind = n * 16;
	int len = ConNameLen[n];
	strncpy(res,&ConNameTab[ind],len);
	res[len] = '\0';

	return res ;
}
/////////////////////////////////////////////////
//LoadConNames()
//  Returns:0=nothing,-1 bad file, 
//          n = number con names
//
/////////////////////////////////////////////////
int StarCatalog::LoadConNames()
{
	int ret = 0;;

	FILE *pDataFile;		//data file pointer
	int i,len,ind;	
	float S1RA,S1Dec;	
	char instr[70];

	NumConName = 0;

	pDataFile = fopen(ConNamesFileName,"r");
	
		
	if(pDataFile == NULL)
	{
		ret = -1;//bad file error mess
	}
	else
	{
		while(fgets(instr,65,pDataFile) != NULL) 
		{
			NumConName += 1;
		
			//find len and adjust if too big..
			//note:must sub 1 to strip \n from
			//read when using Win...
			len = strlen(instr)-1;
			if(len > 16) len = 16;

			//fill len table element
			ConNameLen[NumConName] = len;
			
			ind = NumConName * 16;
			for(i=0;i<len;i++)
			{
				ConNameTab[ind] = instr[i];
				ind++;
			}
	
			//get position and fill Loc Table elements
			fgets(instr,65,pDataFile);
			sscanf(instr, "%f %f", 
			  &S1RA,&S1Dec );
			ConNameLoc[NumConName][0] = S1RA;
			ConNameLoc[NumConName][1] = S1Dec;
		
			i=1;
		}
		
		fclose(pDataFile);
		ret = NumConName;
	}
	
	return(ret);

}


//////////////////////////////////////////////////////////
//GetStarNPos(int n)
//
//
//////////////////////////////////////////////////////////
STARDATA StarCatalog::GetStarNPos(int n)
{
	STARDATA ret;

	ret.RA = StarNameLoc[n][0];
	ret.Dec = StarNameLoc[n][1];
	ret.Mag = 0;

	return(ret);
}
//////////////////////////////////////////////////////////
//GetStarName(int n)
//
//
//////////////////////////////////////////////////////////
char* StarCatalog::GetStarName(char* res,int n) const
{
	int ind = n * 16;
	int len = StarNameLen[n];
	strncpy(res,&StarNameTab[ind],len);
	res[len] = '\0';
	return res ;
}
/////////////////////////////////////////////////
//
//
//
/////////////////////////////////////////////////
int StarCatalog::LoadStarNames()
{

	int ret = 0;;

	FILE *pDataFile;		//data file pointer
	int i,len,ind;	
	float S1RA,S1Dec;	
	char instr[70];

	NumStarName = 0;

	pDataFile = fopen(StarNamesFileName,"r");
	
		
	if(pDataFile == NULL)
	{
		ret = -1;//bad file error mess
	}
	else
	{
		while(fgets(instr,65,pDataFile) != NULL) 
		{
			NumStarName += 1;
		
			//find len and adjust if too big..
			//note:must sub 1 to strip \n from
			//read when using Win...
			len = strlen(instr)-1;
			if(len > 16) len = 16;

			//fill len table element
			StarNameLen[NumStarName] = len;
			
			ind = NumStarName * 16;
			for(i=0;i<len;i++)
			{
				StarNameTab[ind] = instr[i];
				ind++;
			}
	
			//get position and fill Loc Table elements
			fgets(instr,65,pDataFile);
			sscanf(instr, "%f %f", 
			  &S1RA,&S1Dec );
			StarNameLoc[NumStarName][0] = S1RA;
			StarNameLoc[NumStarName][1] = S1Dec;
		
			i=1;
		}
		
		fclose(pDataFile);
		ret = NumStarName;
	}
	
	return(ret);

}
/////////////////////////////////////////////////
//LoadConLines()
//
//
/////////////////////////////////////////////////
int StarCatalog::LoadConLines()
{
	FILE *pDataFile;		//data file pointer
	int i,ret;	
	float S1RA,S2RA,S1Dec,S2Dec;	
	char instr[70];

	pDataFile = fopen(ConLinesFileName,"r");
	
	NumCSeg = 0;
	
	if(pDataFile == NULL)
	{
		//MessageBox("ConLines.dat Not Found!","File Error");
		ret = -1;//file not found error flag
	}
	else
	{
		while(fgets(instr,65,pDataFile) != NULL) 
		{
			//CString InStr;
			//InStr = instr;
			//if(InStr.Left(1) != "*")//skip comments
			if(instr[0] != 42)//skip comments(42 = "*")
			{
				sscanf(instr, "%f %f %f %f", 
				  &S1RA,&S1Dec,&S2RA,&S2Dec );
				NumCSeg += 1;
				ConLine[NumCSeg][0] = S1RA;
				ConLine[NumCSeg][1] = S1Dec;
				ConLine[NumCSeg][2] = S2RA;
				ConLine[NumCSeg][3] = S2Dec;
			i=1;
			}
		}	
	fclose(pDataFile);
	ret = NumCSeg;
	}
	
	return(ret);
}


/////////////////////////////////////////////////
//BuildBSCatCond()
//  Builds condensed catalog from BS catalog. Test
//  if file names are good first.
//
//  Uses: BSCatName
//        BSCondCatName
//
//  Returns: -1 if file error
//            0 if nothing
//            n, number of records
//
/////////////////////////////////////////////////
int StarCatalog::BuildBSCatCond(ProgBarDialog* pbd)
{
	int ret = 0;
	int FileStat = 0;

	FILE *pDataFile;		//data file pointer
	FILE *pOutFile;		//data file pointer

	//int i,j;
	int id,i;
	double RA,Dec;
	float RAs,Mag;
	int Decs,RAh,RAm,Decd,Decm;
	int NumRead,NumStar;
	//STARDATA CStar;
	//POLARVEC DVec;

	NumStar = 0;

	//Test files names
	pDataFile = fopen(BSCatName,"r");
	if(pDataFile == NULL)
	{
		ret = -1;
		FileStat = 0;
	}
	else
	{
		pOutFile = fopen(BSCondCatName,"w");
		if(pOutFile == NULL)
		{
			//bad file name
			ret = -1;
			FileStat = 0;
			fclose(pDataFile);//close data file before exit
		}
		else
		{
			FileStat = 1;//ok procedd
		}

	}

	char instr[200];

	//CString Cinstr,CStr,CSign;
	char CSign[4];
	NumRead = 0;

	//if files good, do it
	if(FileStat ==1)
	{
		int RunFlag = 1;
		while(fgets(instr,200,pDataFile) != NULL && RunFlag == 1)
		{
			NumRead +=1;

			//Cinstr = instr;
			strncpy(CSign,instr+83,1);
			//read and decode part of a record
			id = IntData(instr,4);
			RAh = IntData(instr+75,2);
			RAm = IntData(instr+77,2);
			RAs = FloatData(instr+79,4);
			Decd = IntData(instr+84,2);
			Decm = IntData(instr+84,2);
			Decs = IntData(instr+88,2);
			Mag = FloatData(instr+103,4);

			
			//convert to dec (RA in degrees around sphere)
			RA = (float)RAh * 15 + (float)RAm * .25 + RAs * (1/240.0);
			Dec = fabs((float)Decd) + (float)Decm / 60.0 + Decs / 3600.0;
			//Csign = sign;
			if(strcmp(CSign,"-") == 0) Dec = (-1.0) * Dec;
		i=1;
			//output a record
			fprintf(pOutFile,"%4d %5.2f %8.5f %7.4f     \n",
				id,Mag,RA,Dec); 
			
			//Test and update status
			if((float)NumRead/1000 == NumRead/1000)
			{
				int p;
				p =(int)(100* (float)NumRead/9000);
				pbd->PostProg(p);
				RunFlag = pbd->CheckRunFlag();
			}
		}

		fclose(pDataFile);
		fclose(pOutFile);
		ret = NumRead;
	}

	//char outstr[60];
	//sprintf( outstr, "NumRead:%d ",NumRead);

	return(ret);
}
/////////////////////////////////////////////////
//BuildHipCatCond()
//  see BuildBSCatCond()
//
/////////////////////////////////////////////////
int StarCatalog::BuildHipCatCond(ProgBarDialog* pbd)
{
	int ret = 0;
	int FileStat = 0;

	FILE *pDataFile;		//data file pointer
	FILE *pOutFile;		//data file pointer

	//int i,j;
	int id,i;
	double RA,Dec;
	float Mag;
	float RAh,RAm,RAs,Decd,Decm,Decs;
	int NumRead;

	NumStar = 0;

	//Test files names
	pDataFile = fopen(HipCatName,"r");
	if(pDataFile == NULL)
	{
		ret = -1;
		FileStat = 0;
	}
	else
	{
		pOutFile = fopen(HipCondCatName,"w");
		if(pOutFile == NULL)
		{
			//bad file name
			ret = -1;
			FileStat = 0;
			fclose(pDataFile);//close data file before exit
		}
		else
		{
			FileStat = 1;//ok procedd
		}

	}


	char instr[500];
	char dum[400],sign[2];
	//char Csign[4];
	NumRead = 0;

	//if files good, do it
	if(FileStat ==1)
	{
		int RunFlag = 1;
		while(fgets(instr,480,pDataFile) != NULL && RunFlag == 1)
		{
			NumRead +=1;

			//read and decode part of a record
			sscanf(instr,"%2c%d",dum,&id);
			sscanf(instr,"%17c%f",dum,&RAh);
			sscanf(instr,"%20c%f",dum,&RAm);
			sscanf(instr,"%23c%f",dum,&RAs);
			sscanf(instr,"%29c%1c%f",dum,sign,&Decd);
			sscanf(instr,"%33c%f",dum,&Decm);
			sscanf(instr,"%36c%f",dum,&Decs);
			sscanf(instr,"%41c%f",dum,&Mag);

			sign[1]='\0';
			
			//convert to dec (RA in degrees around sphere)
			RA = RAh * 15 + RAm * .25 + RAs * (1/240.0);
			Dec = fabs(Decd) + Decm / 60 + Decs / 3600;
			//Csign = sign;
			if(strcmp(sign,"-") == 0) Dec = (-1.0) * Dec;
			i=1;//debug Stop
			//output a record
			fprintf(pOutFile,"%6d %9.6f %9.6f %9.6f     \n",id,RA,Dec,Mag); 	
		

			//Test and update status
			if((float)NumRead/10000 == NumRead/10000)
			{
				int p;
				p =(int)(100* (float)NumRead/118000);
				pbd->PostProg(p);
				RunFlag = pbd->CheckRunFlag();
			}
		}

		fclose(pDataFile);
		fclose(pOutFile);
		ret = NumRead;
	}

	return(ret);
}
/////////////////////////////////////////////////
//BuildTycCatCond()
//  see BuildBSCatCond()
//
//
/////////////////////////////////////////////////
int StarCatalog::BuildTycCatCond(ProgBarDialog* pbd)
{
	int ret = 0;
	int FileStat = 0;

	FILE *pDataFile;		//data file pointer
	FILE *pOutFile;		//data file pointer

	//int i,j;
	int i;
	int id1,id2,id3;
//	double RA,Dec;
	float Mag;
	float RA,Dec;
	int NumRead;

	NumStar = 0;

	//Test files names
	pDataFile = fopen(TycCatName,"r");
	if(pDataFile == NULL)
	{
		ret = -1;
		FileStat = 0;
	}
	else
	{
		pOutFile = fopen(TycCondCatName,"w");
		if(pOutFile == NULL)
		{
			//bad file name
			ret = -1;
			FileStat = 0;
			fclose(pDataFile);//close data file before exit
		}
		else
		{
			FileStat = 1;//ok procedd
		}

	}


	char instr[500];
	char dum[400];//,sign[2];
	//char Csign[4];
	NumRead = 0;

	//if files good, do it
	if(FileStat ==1)
	{
		int RunFlag = 1;
		while(fgets(instr,480,pDataFile) != NULL && RunFlag == 1)
		{
			NumRead +=1;

			//read and decode part of a record
			sscanf(instr,"%2c%d",dum,&id1);
			sscanf(instr,"%6c%d",dum,&id2);
			sscanf(instr,"%12c%d",dum,&id3);

			sscanf(instr,"%51c%f",dum,&RA);
			sscanf(instr,"%64c%f",dum,&Dec);
			sscanf(instr,"%41c%f",dum,&Mag);
			
			i=id1 * 100000 + id2 * 10 + id3 ;
			//output a record
			fprintf(pOutFile,"%10d %12.8f %12.8f %5.2f \n"
				,i,RA,Dec,Mag); 	
		
			//Test and update status
			if((float)NumRead/10000 == NumRead/10000)
			{
				int p;
				p =(int)(100* (float)NumRead/1050000);
				pbd->PostProg(p);
				RunFlag = pbd->CheckRunFlag();
			}
		
		}

		fclose(pDataFile);
		fclose(pOutFile);
		ret = NumRead;
	}

	return(ret);
}
/////////////////////////////////////////////////
//LoadHipCat()
//
//
/////////////////////////////////////////////////
int StarCatalog::LoadHipCat(ProgBarDialog* pbd)
{
	FILE *pDataFile;		//data file pointer

	//char outstr[60];
	int id,ret;
	int FileStat;
	float RA,Dec,Mag;
	char instr[500];
	int NumRead;
	//STARDATA CStar;
	//POLARVEC DVec;



	//Allocate mem block
	free(SCat);
	free(SCatID);
	free(SCatRecord);

	SCat = (float *)malloc(120000*3*sizeof(float));
	SCatID = (int *)malloc(120000 * sizeof(int));
	SCatRecord = (int *)malloc(120000 * sizeof(int));
	MaxCatSize = 119900;

	NumStar = 0;

	pDataFile = fopen(HipCondCatName,"r");


	//test file name
	if(pDataFile == NULL)
	{
		FileStat = 0;
		ret = -1;
	}
	else
	{	
		FileStat = 1;
	}

	if(FileStat == 1)
	{
		NumStar = 0;
		NumRead = 0;
		int RunFlag = 1;

		while(fgets(instr,49,pDataFile) != NULL && RunFlag == 1)
		{
			NumRead +=1;
			sscanf(instr," %d %f %f %f",&id,&RA,&Dec,&Mag);						
			NumStar += 1;		
			PutSCat(NumStar,id,NumRead,RA,Dec,Mag);			
		
			//Test and update status
			if((float)NumRead/1000 == NumRead/1000)
			{
				int p;
				p =(int)(100* (float)NumRead/118000);
				pbd->PostProg(p);
				RunFlag = pbd->CheckRunFlag();
			}
		}
		fclose(pDataFile);

		//set hip catalog in use
		CatalogMode = 2;
	}

	return(NumRead);
}

/////////////////////////////////////////////////
//LoadTycCat()
//  see LoadBSCat
//
/////////////////////////////////////////////////
int StarCatalog::LoadTycCat(ProgBarDialog* pbd)
{
	FILE *pDataFile;		//data file pointer

	//int i,j;
	int id,ret;
	float RA,Dec,Mag;
	//float RAh,RAm,RAs,Decd,Decm,Decs;
	long NumRead;
	//char outstr[60];
	//STARDATA CStar;
	//POLARVEC DVec;
	int FileStat;


	//Allocate mem block
	free(SCat);
	free(SCatID);
	free(SCatRecord);

	SCat = (float *)malloc(1200000*3*sizeof(float));
	SCatID = (int *)malloc(1200000 * sizeof(int));
	SCatRecord = (int *)malloc(1200000 * sizeof(int));
	MaxCatSize = 1199900;

	pDataFile = fopen(TycCondCatName,"r");

	//test file name
	if(pDataFile == NULL)
	{
		ret = -14;
		FileStat = 0;
	}
	else
	{	
		FileStat = 1;
	}

	if(FileStat == 1)
	{

		char instr[500];

		NumStar = 0;
		NumRead = 0;
		int RunFlag = 1;

		while(fgets(instr,49,pDataFile) != NULL && RunFlag == 1)
		//while(fscanf(pDataFile,"%6d %9.6f %9.6f %9.6f\n",id,RA,Dec,Mag) != NULL )
		{
			NumRead +=1;
			sscanf(instr,"%d %f %f %f",&id,&RA,&Dec,&Mag);
			NumStar += 1;		
			PutSCat(NumStar,id,NumRead,RA,Dec,Mag);	
		
			//Test and update status
			if((float)NumRead/10000 == NumRead/10000)
			{
				int p;
				p =(int)(100* (float)NumRead/1050000);
				pbd->PostProg(p);
				RunFlag = pbd->CheckRunFlag();
			}
		}

		fclose(pDataFile);
		ret = NumRead;
		//set tyc catalog in use
		CatalogMode = 3;
	}

	return (ret);
}



/////////////////////////////////////////////////
//
//  see LoadBSCat
//
/////////////////////////////////////////////////
int StarCatalog::LoadBSCat(ProgBarDialog* pbd)
{
	FILE *pDataFile;		//data file pointer
	int FileStat; //1=openable, 0=not openable
	int id,NumRead;
	char instr[60];
	float RA,Dec,Mag;	
	int RunFlag = 0;
	//load star data
	pDataFile = fopen(BSCondCatName/*"BSCat.dat"*/,"r");

	//test file name
	if(pDataFile == NULL)
	{
		NumRead = -1;//error flag
		//vNoticeDialog note(this);
	    //note.Notice("New");
		
	}
	else
	{	
		FileStat = 1;
	}

	//read file and close it if good
	if(FileStat == 1) //then read file
	{

		//Allocate mem block
		free(SCat);
		free(SCatID);
		free(SCatRecord);

		SCat = (float *)malloc(10000*3*sizeof(float));
		SCatID = (int *)malloc(10000 * sizeof(int));
		SCatRecord = (int *)malloc(10000 * sizeof(int));
		MaxCatSize = 9900;

		//new style read	
		NumRead = 0;
		NumStar = 0;
		RunFlag = 1;

		while(fgets(instr,49,pDataFile) != NULL && RunFlag == 1 )
		{
			NumRead +=1;

			sscanf(instr," %d %f %f %f",&id,&Mag,&RA,&Dec);

			NumStar += 1;

			//aloc mem version
			PutSCat(NumStar,id,NumStar,RA,Dec,Mag);
		
			//Test and update status
			if((float)NumRead/1000 == NumRead/1000)
			{
				int p;
				p =(int)(100* (float)NumRead/9000);
				pbd->PostProg(p);
				RunFlag = pbd->CheckRunFlag();
			}
		
		}

		fclose(pDataFile);	

		//set BS catalog in use
		CatalogMode = 1;	//
	}
  
	return(NumRead);
}

//////////////////////////////////////////////////////////
//ReadBSRecord(int RecordNum)
//  Reads a record from Current catalog
//  Puts raw record in char RawRecord
//  Parsed record in PRecord
//
//  Returns: 1 if read ok
//           -1 if file not found
//           0 if bad data.
//
//  Uses: CatalogMode
//////////////////////////////////////////////////////////
int StarCatalog::ReadRecord(int RecordNum)
{
	int ret;

	if(CatalogMode == 1)
	{
		ret = ReadBSRecord(RecordNum);
	}

	else
	if(CatalogMode == 2)
	{
		ret = ReadHipRecord(RecordNum);
	}

	else
	if(CatalogMode == 3)
	{
		ret = ReadTycRecord(RecordNum);
	}

	else
	{
		ret = 0;
		strcpy(RawRecord,"CatalogMode Error");
		strcpy(PRecord ,".");
	}

	return(ret);
}

//////////////////////////////////////////////////////////
//ReadBSRecord(int RecordNum)
//  Reads a record from BS_catalog.dat
//  Puts raw record in char RawRecord
//  Parsed record in PRecord
//
//  Returns: 1 if read ok
//           -1 if file not found
//           0 if bad data.
//
//note: This catalog does not have consistant record length
//      therefor each record is read seq until desired
//      one is reached. Time not too bad.

//////////////////////////////////////////////////////////
int StarCatalog::ReadBSRecord(int RecordNum)
{
	int ret;
	FILE *pDataFile;		//data file pointer
	char instr[500];
	int Line;//,FileStat;


	pDataFile = fopen(BSCatName,"r");

	//test file name
	if(pDataFile == NULL)
	{
		ret = -1;
		strcpy(RawRecord,"BSCat File Not Found");
		strcpy(PRecord ,"File Not Found");
	}
	else//file found
	{		
		Line = 0;
		while(Line < RecordNum)
		{
			fgets(instr,200,pDataFile);
			Line +=1;
		}
		
		fclose(pDataFile);

		strcpy(RawRecord,instr);
		ret = 1;

		//PARSE THE RECORD
		ParseBSRecord();
		//strcpy(PRecord,"BS_catalog **NO PARSE YET**");
	}
	
	return(ret);
}

//////////////////////////////////////////////////////////
//ReadHipRecord
//
//  Returns: 1 if read ok
//           -1 if file not found
//           0 if bad data.
//////////////////////////////////////////////////////////
int StarCatalog::ReadHipRecord(int RecordNum)
{
	int ret;//,FileStat;
	FILE *pDataFile;		//data file pointer
	char instr[500];
	long RecordLength;
	RecordLength = 451;//449+2,determined by experiment

	long Pos = RecordLength * (long)(RecordNum - 1);

	pDataFile = fopen(HipCatName,"r");

	//test file name
	if(pDataFile == NULL)
	{
		ret = -1;
		strcpy(RawRecord,"HipCat File Not Found");
		strcpy(PRecord ,"File Not Found");	
	}

	else
	{
			fseek(pDataFile,Pos,SEEK_SET);
			fgets(instr,460,pDataFile);
				
			fclose(pDataFile);

			strcpy(RawRecord,instr);
			ret = 1;
			ParseHipRecord();			
	}
	
	return(ret);

}

//////////////////////////////////////////////////////////
//ReadTycRecord
//////////////////////////////////////////////////////////
int StarCatalog::ReadTycRecord(int RecordNum)
{
	int ret;//,FileStat;
	FILE *pDataFile;		//data file pointer
	char instr[500];
	long RecordLength;
	RecordLength = 351;  //350+1, verified by experiment.

	long Pos = RecordLength * (long)(RecordNum - 1);

	//
	pDataFile = fopen(TycCatName,"r");

	//test file name
	if(pDataFile == NULL)
	{
		ret = -1;
		strcpy(RawRecord,"TycCat File Not Found");
		strcpy(PRecord ,"File Not Found");
	}

	//read file and close it if good
	else
	{		
			fseek(pDataFile,Pos,SEEK_SET);
			fgets(instr,460,pDataFile);
			
			fclose(pDataFile);

			strcpy(RawRecord,instr);
			ParseTycRecord();
			ret = 1;	
	}

	return(ret);
}


//////////////////////////////////////////////////////////
//ParseBSRecord
//Uses: RawRecord
//Sets: ParsedRecord
//////////////////////////////////////////////////////////
void StarCatalog::ParseBSRecord()
{

	char s[500];
	char raw[500];

	s[0] = '\0';

	strcpy(raw,RawRecord);

	strcat(s,"Name=");
	strncat(s,raw+4,10);
	strcat(s,"\n");

	strcat(s,"RA=");
	strncat(s,raw+75,2);
	strcat(s,":");
	strncat(s,raw+76,2);
	strcat(s,":");
	strncat(s,raw+79,4);
	strcat(s,"\n");

	strcat(s," Dec=");
	strncat(s,raw+83,3);
	strcat(s,":");
	strncat(s,raw+86,2);
	strcat(s,":");
	strncat(s,raw+88,2);
	strcat(s,"\n");

	strcat(s,"VMag=");
	strncat(s,raw+102,5);
	strcat(s,"\n");

	//strcat(s,"HR=");
	//strncat(s,raw+10,4));
	//strcat(s,"\n");

	strcat(s,"HD=");
	strncat(s,raw+26,5);
	strcat(s,"\n");

	strcat(s,"SOA=");
	strncat(s,raw+31,6);
	strcat(s,"\n");

	strcat(s,"Mult=");
	strncat(s,raw+43,1);
	strcat(s,"\n");

	strcat(s,"Sep=");
	strncat(s,raw+184,6);
	strcat(s,"\n");

	strcat(s,"DMag=");
	strncat(s,raw+180,4);
	strcat(s,"\n");

	strcat(s,"MultCnt=");
	strncat(s,raw+194,2);
	strcat(s,"\n");

	strcat(s,"SpType=");
	strncat(s,raw+127,15);
	strcat(s,"\n");

	strcat(s,"pmRA=");
	strncat(s,raw+148,6);
	strcat(s,"\n");

	strcat(s,"pmDE=");
	strncat(s,raw+154,6);
	strcat(s,"\n");

	strcat(s,"Parallax=");
	strncat(s,raw+161,5);
	strcat(s,"\n");

	strcat(s,"NoteFlag=");
	strncat(s,raw+197,1);

	strcpy(PRecord,s);

}

//////////////////////////////////////////////////////////
//ParseHipRecord
//////////////////////////////////////////////////////////
void StarCatalog::ParseHipRecord()
{

	char s[500];
	char raw[500];

	s[0] = '\0';

	strcpy(raw,RawRecord);

	strcat(s,"HIP=");
	strncat(s,raw+8,6);
	strcat(s,"\n");

	strcat(s,"RAhms");
	strncat(s,raw+17,2);
	strcat(s,":");
	strncat(s,raw+20,2);
	strcat(s,":");
	strncat(s,raw+23,5);
	strcat(s,"\n");

	strcat(s,"DEhms=");
	strncat(s,raw+29,3);
	strcat(s,":");
	strncat(s,raw+33,2);
	strcat(s,":");
	strncat(s,raw+36,2);
	strcat(s,"\n");

	strcat(s,"Vmag=");
	strncat(s,raw+41,5);
	strcat(s,"\n");

	strcat(s,"VarFlag=");
	strncat(s,raw+47,1);
	strcat(s,"\n");

	strcat(s,"Period=");
	strncat(s,raw+313,7);

	strcat(s,"RAdeg=");
	strncat(s,raw+51,12);
	strcat(s,"\n");

	strcat(s,"DEdeg=");
	strncat(s,raw+64,12);
	strcat(s,"\n");

	strcat(s,"HD=");
	strncat(s,raw+390,6);
	strcat(s,"\n");


	strcat(s,"MultFlag=");
	strncat(s,raw+346,1);
	strcat(s,"\n");

	strcat(s,"nComp?=");
	strncat(s,raw+343,2);
	strcat(s,"\n");

	strcat(s,"rho=");
	strncat(s,raw+359,7);
	strcat(s,"\n");

	strcat(s,"dHp=");
	strncat(s,raw+373,5);
	strcat(s,"\n");

	strcat(s,"theta=");
	strncat(s,raw+355,3);
	strcat(s,"\n");

	strcat(s,"SpType=");
	strncat(s,raw+435,12);
	strcat(s,"\n");

	strcat(s,"DEdeg=");
	strncat(s,raw+64,12);
	strcat(s,"\n");

	strcat(s,"Plx=");
	strncat(s,raw+79,7);
	strcat(s,"\n");

	strcat(s,"pmRA=");
	strncat(s,raw+87,8);
	strcat(s,"\n");

	strcat(s,"pmDE=");
	strncat(s,raw+96,8);
	strcat(s,"\n");

	strcat(s,"Notes=");
	strncat(s,raw+388,1);


	strcpy(PRecord,s);
}

//////////////////////////////////////////////////////////
//ParseTycRecord
//////////////////////////////////////////////////////////
void StarCatalog::ParseTycRecord()
{
	char s[500];
	char raw[500];

	s[0] = '\0';

	strcpy(raw,RawRecord);

	strcat(s,"TYC=");
	strncat(s,raw+2,12);
	strcat(s,"\n");

	strcat(s,"RAhms=");
	strncat(s,raw+17,2);
	strcat(s,":");
	strncat(s,raw+20,2);
	strcat(s,":");
	strncat(s,raw+23,5);
	strcat(s,"\n");

	strcat(s,"DEhms=");
	strncat(s,raw+29,3);
	strcat(s,":");
	strncat(s,raw+33,2);
	strcat(s,":");
	strncat(s,raw+36,4);
	strcat(s,"\n");

	strcat(s,"Vmag=");
	strncat(s,raw+41,5);
	strcat(s,"\n");

	strcat(s,"MultFlag=");
	strncat(s,raw+295,1);
	strcat(s,"\n");

	strcat(s,"m_Hip=");
	strncat(s,raw+299,2);
	strcat(s,"\n");

	strcat(s,"RAdeg=");
	strncat(s,raw+51,12);
	strcat(s,"\n");

	strcat(s,"DEdeg");
	strncat(s,raw+64,12);
	strcat(s,"\n");

	strcat(s,"HD=");
	strncat(s,raw+309,6);
	strcat(s,"\n");

	strcat(s,"Plx=");
	strncat(s,raw+79,7);
	strcat(s,"\n");

	strcat(s,"pmRA=");
	strncat(s,raw+87,8);
	strcat(s,"\n");

	strcat(s,"pmDE=");
	strncat(s,raw+96,8);
	strcat(s,"\n");

	strcat(s,"Remark=");
	strncat(s,raw+349,1);


	strcpy(PRecord,s);
}

/////////////////////////////////////////////////////////////////
//PutSCat()
//  Puts star in dynamic mem block
/////////////////////////////////////////////////////////////////
void StarCatalog::PutSCat(int Index,int ID,int Record,
						  float RA,float Dec,float Mag)
{
	float * pData;
	int * pDataI;
	int dum;
	
	dum = Index * 3;
	pData = SCat + dum;
	*pData = RA;
	pData++;
	*pData = Dec;
	pData++;
	*pData = Mag;

	pDataI = SCatID + Index;
	*pDataI = ID;

	pDataI = SCatRecord + Index;
	*pDataI = Record;
}
/////////////////////////////////////////
//GetStar
//Gets star from star catalog from catalog index
//Gets star from alloc mem structure
//////////////////////////////////////////
STARDATA StarCatalog::GetStar(int index)
{
	STARDATA Star;

	float * pData;
	int dum;

	dum = index * 3;
	pData = SCat + dum;
	Star.RA = *pData;
	pData++;
	Star.Dec = *pData;
	pData++;
	Star.Mag = *pData;

	return(Star);
}

///////////////////////////////////////////////
//PutVSList(int x,int y, long id)
//  Adds viewframe x and y coords and star id to
//  VSList, which is a list of viewed starts plotted
//  in the view frame
///////////////////////////////////////////////
void StarCatalog::PutVSList(int x,int y, long id)
{
	if(NumVS < MAXVSL)
	{
		NumVS += 1;
		VSList[NumVS][0] = id;
		VSList[NumVS][1] = x;
		VSList[NumVS][2] = y;
	}
}

///////////////////////////////////////////////
//
//  
//  
//  
///////////////////////////////////////////////
void StarCatalog::GetVObj(int i, int *obj ,float *RA, float *Dec,
						  float *Mag,float *Dia, int *x, int *y)
{
	if(i < MAXVOBJ)
	{	
		*RA = VObjListPos[i][0];
		*Dec = VObjListPos[i][1];
		*Dia = VObjListPos[i][2];	//dia
		*Mag = VObjListPos[i][3];
															  
		*obj = VObjListData[i][0];
		*x = VObjListData[i][1];
		*y = VObjListData[i][2];

	}
}
///////////////////////////////////////////////
//
//  
//  
//  
///////////////////////////////////////////////
void StarCatalog::PutVObjList(int obj ,int x,int y,
							  float RA, float Dec,
							  float Mag,float Dia)
{
	if(NumVObj < MAXVOBJ)
	{
		NumVObj += 1;
	
		VObjListPos[NumVObj][0] = RA;
		VObjListPos[NumVObj][1] = Dec;
		VObjListPos[NumVObj][2] = Dia;	//dia
		VObjListPos[NumVObj][3] = Mag;
															  
		VObjListData[NumVObj][0] = obj; 
		VObjListData[NumVObj][1] =  x;
		VObjListData[NumVObj][2] =  y;

	}
}

///////////////////////////////////////////////
//ClearVSList()
//
///////////////////////////////////////////////
void StarCatalog::ClearVSList()
{
	NumVS = 0;
	NumVObj = 0;
}
///////////////////////////////////////////////
//GetVS_ID()
//  Searches the VSList for a star that was plotted
//  at the x, y input coords, returns the index into
//  the star table of the star
//  if found, otherwise returns zero.
//Also searches object list, if an object is found closest
//  returns negative object number + 10(10 sun,11 merc...)
//  (plus 10 is so object 0(sun) is not passed back as
//  0, no star found...
//  
///////////////////////////////////////////////
long StarCatalog::GetVS_ID(int x,int y)
{
	int i;
	long Ret;
	float MinDist,Dist;
	int SIndex;
	float RA,Dec,Mag,Dia;
	int xx,yy,obj;

	MinDist = 1000;	//init it large
	SIndex = 0;
	Ret = 0;

	for(i=1;i<NumVS+1;i++)
	{
		//find dist to star
		Dist = VM.CDist(x,y,VSList[i][1],VSList[i][2]);

		//is smallest so far?...
		if(Dist < MinDist)
		{
			MinDist = Dist;
			SIndex = VSList[i][0];
		}
	}

	//check for planet objects
	if(NumVObj >0)
	{
		for(i=1;i<=NumVObj;i++)
		{
			GetVObj(i ,&obj ,&RA, &Dec, &Mag, &Dia, &xx, &yy);
			Dist = VM.CDist(x,y,xx,yy);

			//is smallest so far?...
			if(Dist < MinDist)
			{
				MinDist = Dist;
				SIndex = -(obj + 10);
			}
		}
	}
	//if within 10 pix of clossest star
	if(MinDist < 10)
	{
		Ret = SIndex;
	}

	return(Ret);
}
////////////////////////////////////////////////
//
////////////////////////////////////////////////
int StarCatalog::GetStarID(int index)
{
	int ID;
	int *pData;

	pData = SCatID + index;
	ID = *pData;

	return(ID);
}

////////////////////////////////////////////////////
//ResolvePaths()
//  Resolves all file name paths to from relative
//  to absolute.  This must be done to avoid errors
//  after a SaveSet from main dialog which seems
//  to also redifine the defualt path and really
//  mess up the relative path references in the file
//  names
/////////////////////////////////////////////////////
void StarCatalog::ResolvePaths()
{
	//resolve path to abs ref
	char full[256];
	//CString FName;
	//FName = BSCondCatName;
#ifdef WIN32
	 if( _fullpath( full, BSCondCatName, 256) != NULL )
	 {
         strcpy(BSCondCatName,full);
	 }

	 if( _fullpath( full, BSCatName, 256) != NULL )
	 {
         strcpy(BSCatName,full);
	 }

	 if( _fullpath( full, HipCondCatName, 256) != NULL )
	 {
         strcpy(HipCondCatName,full);
	 }

	 if( _fullpath( full, HipCatName, 256) != NULL )
	 {
         strcpy(HipCatName,full);
	 }

	 if( _fullpath( full, TycCondCatName, 256) != NULL )
	 {
         strcpy(TycCondCatName,full);
	 }

	 if( _fullpath( full, TycCatName, 256) != NULL )
	 {
         strcpy(TycCatName,full);
	 }


	 if( _fullpath( full, ConLinesFileName, 256) != NULL )
	 {
         strcpy(ConLinesFileName,full);
	 }


	 if( _fullpath( full, ConNamesFileName, 256) != NULL )
	 {
         strcpy(ConNamesFileName,full);
	 }


	 if( _fullpath( full, StarNamesFileName, 256) != NULL )
	 {
         strcpy(StarNamesFileName,full);
	 }


	 if( _fullpath( full, SettingsFileName, 256) != NULL )
	 {
         strcpy(SettingsFileName,full);
	 }
#endif

//cliff's fix for linux operation:
#ifdef LINUX	
	char dataDir[256];
	char* ptr;
	if((ptr =getenv("STARCATDATA"))==NULL)sprintf(dataDir,"./");
	else strcpy(dataDir,ptr);

	 if( BSCondCatName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,BSCondCatName);
			strcpy(BSCondCatName,full);
#ifdef DEBUG_1
cout << "BSCondCatName = " << BSCondCatName << endl;
#endif

	 }
	 if( BSCatName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,BSCatName);
         strcpy(BSCatName,full);
#ifdef DEBUG_1
cout << "BSCatName = " << BSCatName << endl;
#endif
	 }

	 if( HipCondCatName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,HipCondCatName);
         strcpy(HipCondCatName,full);
#ifdef DEBUG_1
cout << "HipCondCatName = " << HipCondCatName << endl;
#endif
	 }

	 if( HipCatName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,HipCatName);
         strcpy(HipCatName,full);
#ifdef DEBUG_1
cout << "HipCatName = " << HipCatName << endl;
#endif
	 }

	 if( TycCondCatName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,TycCondCatName);
         strcpy(TycCondCatName,full);
#ifdef DEBUG_1
cout << "TycCondCatName = " << TycCondCatName << endl;
#endif
	 }

	 if( TycCatName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,TycCatName);
         strcpy(TycCatName,full);
#ifdef DEBUG_1
cout << "TycCatName = " << TycCatName << endl;
#endif
	 }


	 if( ConLinesFileName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,ConLinesFileName);
         strcpy(ConLinesFileName,full);
#ifdef DEBUG_1
cout << "ConLinesFileName = " << ConLinesFileName << endl;
#endif
	 }


	 if( ConNamesFileName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,ConNamesFileName);
         strcpy(ConNamesFileName,full);
#ifdef DEBUG_1
cout << "ConNamesFileName = " << ConNamesFileName << endl;
#endif
	 }


	 if(StarNamesFileName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,StarNamesFileName);
         strcpy(StarNamesFileName,full);
#ifdef DEBUG_1
cout << "StarNamesFileName = " << StarNamesFileName << endl;
#endif
	 }

	 if ( SettingsFileName[0] != '/' )
	 {
			strcpy(full,dataDir);
         strcat(full,SettingsFileName);
         strcpy(SettingsFileName,full);
#ifdef DEBUG_1
cout << "SettingsFileName = " << SettingsFileName << endl;
#endif
	 }
#endif


}

int StarCatalog::IntData(char* s,int size)
{
	char* data = new char[size+1]; // allocate memory
	strncpy(data,s,size);			 // copy the data
	data[size]='\0';					 // append null character
	int ans = atoi(data);			// get the value
	delete[] data;						// free the memory
	return ans;
}

float StarCatalog::FloatData(char* s,int size)
{
	char* data = new char[size+1]; // allocate memory
	strncpy(data,s,size);			 // copy the data
	data[size]='\0';					 // append null character
	float ans = (float)atof(data);			// get the value
	delete[] data;						// free the memory
	return ans;
}
