//StarCatV, Star Catalog Visualization Tool
// (C) 1999  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



//#include "VMath.h"
//#include "SCcmdw.h"
 
//#include <v/vcmdwin.h>	// So we can use vCmdWindow
//#include <v/vmenu.h>	// For the menu pane
//#include <v/vutil.h>	// For V Utilities
//#include <v/vcmdpane.h> // command pane
//#include <v/vstatusp.h>	// For the status pane

#include "ProgBardlg.h"

class StarCatalog  
{
	public:
	CVMath VM;

	StarCatalog();
	virtual ~StarCatalog();
	int LoadBSCat(ProgBarDialog* pbd);
	int LoadHipCat(ProgBarDialog* pbd);
	int LoadTycCat(ProgBarDialog* pbd);
	int ReadRecord(int RecordNum);
	int ReadBSRecord(int RecordNum);
	int ReadHipRecord(int RecordNum);
	int ReadTycRecord(int RecordNum);
	void ParseBSRecord();
	void ParseHipRecord();
	void ParseTycRecord();
	int BuildBSCatCond(ProgBarDialog* pbd);
	int BuildHipCatCond(ProgBarDialog* pbd);
	int BuildTycCatCond(ProgBarDialog* pbd);
	int LoadConLines();
	int LoadConNames();
	int LoadStarNames();
	STARDATA GetConNPos(int n);
	char* GetConName(char* res, int n) const;
	char* GetStarName(char* res, int n) const;
	STARDATA GetStarNPos(int n);
	STARDATA GetStar(int index);
	void PutSCat(int Index,int ID,int Record,
				float RA,float Dec,float Mag);
	void PutVSList(int x,int y, long id);
	void PutVObjList(int obj ,int x,int y,
					float RA, float Dec,
					float Mag, float Dia);
	void GetVObj(int i, int *obj ,float *RA, float *Dec,
				float *Mag,float *Dia,int *x, int *y);
	void ClearVSList();
	long GetVS_ID(int x,int y);
	int GetStarID(int index);
	void ResolvePaths();

		//catalog in use mode
	int CatalogMode;	//1=BS, 2=hip, 3=tyc

	//Modeflags and sysem vars
	int ViewSort;		//View sort mode 0=off. 1=on

	STARDATA VFCen;				//View Frame center, passed from Plot

	
	#define MAXCAT	10000		//max cat size
	int NumStar;				//number of stars in table
	
	float *SCat;
	int *SCatID;
	int *SCatRecord;
	int MaxCatSize;

	int NumCSeg;				//number of const lines
	
	float ConLine[600][4];		//ConLine[n][0] = RA1
								//ConLine[n][1] = Dec1
								//ConLine[n][2] = RA2
								//ConLine[n][3] = Dec2
	
		

	//StarName Table
	#define MAX_SN	100
	#define MAX_SN_TAB	2000
	int NumStarName;	
	int StarNameLen[MAX_SN];	//[n][0] index into Tab
									//[n][1] length
	float StarNameLoc[MAX_SN][2];	//[n][x] 0=RA,1=Dec
	char StarNameTab[MAX_SN_TAB];


	//ConName Table
	#define MAX_CN	100
	#define MAX_CN_TAB	2000
	int NumConName;					//Number Con Names
	int ConNameLen[MAX_CN];	//[n][0] index into Tab
									//[n][1] length
	float ConNameLoc[MAX_CN][2];	//0=RA,1=Dec
	char ConNameTab[MAX_CN_TAB];



	#define MAXVSL	3000			//max size viewed star list
	int NumVS;					//number in VSList
	long VSList[MAXVSL+5][3];	//Viewed Star List
								//VSList[n][] = number
								//VSList[n][0] = StarID
								//VSList[n][1] = Frame coords x position
								//VSList[n][2] = Frmae coord y position
	
	
	#define MAXVOBJ	20
	int NumVObj;
	float VObjListPos[21][4];	//VObjListPos[n][] =	
								//VObjListPos[n][0] = RA
								//VObjListPos[n][1] = Dec
								//VObjListPos[n][2] = Dia
								//VObjListPos[n][3] = Mag

	int VObjListData[21][3];    //VObjListData[n][] = Mag
								//VObjListData[n][0] = Object id 
								//VObjListData[n][1] = frame coords x
								//VObjListData[n][2] = frame coords y

	//data file names
	char BSCondCatName[80];
	char BSCatName[80];
	char HipCondCatName[80];
	char HipCatName[80];
	char TycCondCatName[80];
	char TycCatName[80];
	char ConLinesFileName[80];
	char ConNamesFileName[80];
	char StarNamesFileName[80];
	char SettingsFileName[80];

	//Catalog Record Data
	char RawRecord[500];
	char PRecord[500];
private:
	int IntData(char* ptr, int size);
	float FloatData(char* ptr, int size);
};
