//StarCatV, Star Catalog Visualization Tool
// (C) 1999  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// VMath.cpp: implementation of the CVMath class.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
//#include "StarView.h"
#include "VMath.h"

//mfc crap:
//#ifdef _DEBUG
//#undef THIS_FILE
//static char THIS_FILE[]=__FILE__;
//#define new DEBUG_NEW
//#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVMath::CVMath()
{

}

CVMath::~CVMath()
{

}

////////////////////////////////////////////////////////////
//DistVec
//uses vector algerbra  to find dist and ang between two stars
//returns this in a POLARVEC type
//Set up to handle arbitrary legnth vectors, should work without
//the vect mag calculations for unit sphere
//  uses:CrossProd()
//       StoRect()
//       DotProd()
////Debug Note:Gives error in altaz plot when
//  viewcen RA==314.238 and S1 RA=134.238
//Note(2/4/2000) if stars are very close, sometimes returns
//  bad values. Hacked to prevent...  This routine may be the 
//  source of the RADec gridline trashing that other hacks
//  reduce...
/////////////////////////////////////////////////////////////
POLARVEC CVMath::DistVec(const STARDATA &Ref1,const STARDATA &Ref2)
{
	POLARVEC DVec;
	RECTVEC NPole,P1,P2,CP1,CP2;
	RECTVEC VS1,VS2;
	double dRA,RA1,RA2,DP,MP1,MP2;
	double Ang;

	double DtoR = ( 2 * 3.141559) / 360;
	double RtoD = 1 / DtoR;

	//Hack to prevent error on both stars close....
	if(    fabs(Ref1.RA - Ref2.RA) < .0001
		&& fabs(Ref1.Dec - Ref2.Dec) < .0001)
	{
		DVec.Ang = 0;
		DVec.Rad = 0;
		return(DVec);
	}

	//get pos data in radians
	RA1 = DtoR * Ref1.RA;
	RA2 = DtoR * Ref2.RA;

	//figure dRA and
	//set dRA in range of 180>=dRA>180.
	dRA = RA2 - RA1;
	if (fabs(dRA) >= 360 * DtoR)
		dRA = 0;
	else if (dRA > 180 * DtoR)
		dRA = dRA - 360 * DtoR;
	else if (dRA <= -180 * DtoR)
		dRA = dRA + 360 * DtoR;

	//glug
	//if(dRA = - 180) dRA = -179.999;
	//if(dRA =  180) dRA = 179.999;

	//init stuff
	NPole.x = 0;
	NPole.y = 0;
	NPole.z = 1;
	P1 = StoRect(Ref1.RA,Ref1.Dec);
	P2 = StoRect(Ref2.RA,Ref2.Dec);
	MP1 = sqrt(P1.x * P1.x + P1.y * P1.y + P1.z * P1.z);
	MP2 = sqrt(P2.x * P2.x + P2.y * P2.y + P2.z * P2.z);

	//figure distance (cord)
	DP = DotProd(P1,P2);
	Ang = acos(DP/(MP1*MP2));
	DVec.Rad = Ang * RtoD;
		
	//handle dRA=0 spec case after the fact
	if(dRA == 0)
	{
		if (Ref1.Dec < Ref2.Dec)
			DVec.Ang = 0;
		else
			DVec.Ang = 180;
	}
	else	//figure compose angle to
	{
		VS1 = CrossProd(NPole,P1);
		CP1 = CrossProd(P1,VS1);
		VS2 = CrossProd(P2,P1);
		CP2 = CrossProd(P1,VS2);
		DP = DotProd(CP1,CP2);
		MP1 = sqrt(CP1.x * CP1.x + CP1.y * CP1.y + CP1.z * CP1.z);
		MP2 = sqrt(CP2.x * CP2.x + CP2.y * CP2.y + CP2.z * CP2.z);
		Ang = acos(DP/(MP1*MP2));
		DVec.Ang = Ang * RtoD;
	}

	if(dRA < 0)
		DVec.Ang = 360 - DVec.Ang;

	//handle dRA=0 spec case after the fact
	if(dRA == 0)
	{
		if (Ref1.Dec < Ref2.Dec)
			DVec.Ang = 0;
		else
			DVec.Ang = 180;
	}

	return(DVec);
}

////////////////////////////////////////////////////////////
//VectorCross Product
/////////////////////////////////////////////////////////////
RECTVEC CVMath::CrossProd(const RECTVEC &a,const RECTVEC &b)
{
	RECTVEC R;
		R.x = a.y * b.z - a.z * b.y;		
		R.y = -(a.x * b.z - a.z * b.x);
		R.z = a.x * b.y - a.y * b.x;
	return(R);
}

////////////////////////////////////////////////////////////
//RecttoS
//
/////////////////////////////////////////////////////////////
void CVMath::RecttoS(const double &xx, const double &yy,
					 const double &zz,
		             double *RA, double *Dec )
{	
	double Phi,Temp,Theta,RAB,DecB,Mag,x,y,z;

	double DtoR = ( 2 * 3.14159265) / 360;
	double RtoD = 1 / DtoR;

	//normalize
	Mag = sqrt(xx * xx + yy * yy + zz * zz);
	x = xx / Mag;
	y = yy / Mag;
	z = zz / Mag;


	//Note: sin returns -90 to 90, cos returns 0 to 180.
	Phi = acos(z );
	
	//This to fix probs at north pole
	Temp = x/sin(Phi);
	if(Temp > 1)
		Temp = 1;
	if (Temp < -1)
		Temp = -1;

	Theta = acos(Temp);
	RAB =Theta * RtoD;
	if(y >= 0)
		*RA = RAB;
	else
		*RA = 360 - RAB;

	DecB = Phi * RtoD;
	*Dec = 90 - DecB;
}

////////////////////////////////////////////////////////////
//StoRect
//  Spherical to Rect conversion, results in unit vector.
//  in puts in RA, Dec, outputs RECTVEX.x.y.z
//  RECTVEC conv is z = Dec=90, x =(RA=0Dec=0),...y = (RA=90Dec=0)
//  Uses shpere to rect eqs, corrects RA to phi convention,
//  so should probably be named CtoRect since it converts celest
//  coords to rect coord.
/////////////////////////////////////////////////////////////
RECTVEC CVMath::StoRect(const double &RA,const double &Dec )
{
	RECTVEC R;
	double theta,phi;
	double DtoR = ( 2 * 3.14159265) / 360;
	double RtoD = 1 / DtoR;
	theta = DtoR * RA;
	phi = DtoR * (90 - fabs(Dec));

	R.x = cos(theta) * sin(phi);
	R.y = sin(theta) * sin(phi);
	R.z = cos(phi);

	if (Dec < 0)
		R.z = -R.z;

	return(R);
}


////////////////////////////////////////////////////////////
//AltAztoRect()
//AltAz to Rect conversion
//inputs Alt Az, outputs RECTXYZ.
//RECTVEC convention is;x = north, y= west, z=zenith
/////////////////////////////////////////////////////////////
RECTVEC CVMath::AltAztoRect(const double &Alt,const double &Az )
{

	RECTVEC R;
	double theta,phi;
	double DtoR = ( 2 * 3.14159265) / 360;
	double RtoD = 1 / DtoR;
	theta = DtoR *(360.0 -  Az);
	phi = DtoR * (90.0 - fabs(Alt));

	R.x = cos(theta) * sin(phi);
	R.y = sin(theta) * sin(phi);
	R.z = cos(phi);

	if (Alt < 0)
		R.z = -R.z;

	return(R);
}
////////////////////////////////////////////////////////////
//Vector Dot Product
/////////////////////////////////////////////////////////////
double CVMath::DotProd(const RECTVEC &a,const RECTVEC &b)
{
	double DotP;
		DotP = (double)(a.x * b.x + a.y * b.y + a.z * b.z);
	return(DotP);
}

////////////////////////////////////////////////////////////////
//FXYtoPol
//takes star position in frame x,y cords, center and view ang
//  and converts to polar cords relative to view frame center.
//  for converting viewed stars to sphere cords.
//Ang sign is positive CW, with Ang = 0 at three oclock.
//
/////////////////////////////////////////////////////////////////
POLARVEC CVMath::FXYtoPol(const int &x, const int &y, 
						  const int &xCen, const int &yCen, 
						  const float &HVAng)
{
	POLARVEC Out;
	float Rad,Ang,dx,dy;
	float RadB,AngB,PtoR;
	
	double DtoR = ( 2 * 3.14159265) / 360;
	double RtoD = 1 / DtoR;

	PtoR = HVAng / yCen;

	dx = (float)(x - xCen);
	dy = (float)(y - yCen);
	RadB = (float)sqrt(dx * dx + dy * dy);
	Rad = PtoR * RadB;

	//figure ang relative to twelve oclock
	AngB = (float)(RtoD * acos(dx/RadB));
	
	if(dy >= 0)		
			Ang = AngB;	
	else
			Ang = 360 - AngB;



	Out.Rad = Rad;
	Out.Ang = Ang;
	return(Out);
}

/////////////////////////////////////////////////////////////
//VecRot
//Inputs Ref opsition, dist and ang, finds point Dist at ang
//  from ref point
//Direction convention is ccw positive when faceing stars
//  from center.
//

/////////////////////////////////////////////////////////////
STARDATA CVMath::VecRot(const STARDATA &Org, 
						const float &Dist, const float &Ang)
{
	STARDATA SLoc,SV1;
	RECTVEC U,R,r;
	double DecB,Phi,Theta,RA,RAB,Dec;

	double DtoR = ( 2 * 3.14159265) / 360;
	double RtoD = 1 / DtoR;

	U = StoRect(Org.RA,Org.Dec);
	
	//Rotate Org toward North pole by Dist
	DecB = Org.Dec + Dist;
	if(DecB <= 90)
		{
			SV1.Dec = (float)DecB;
			SV1.RA = Org.RA;
		}
		else
		{
			SV1.Dec = float(90 - ( DecB - 90));
			if(Org.RA >= 180)
				SV1.RA = Org.RA - 180;
			else
				SV1.RA =Org.RA + 180;
		}
	r = StoRect(SV1.RA,SV1.Dec);

	//Generate rotation matrix
	double A11,A12,A13,A21,A22,A23,A31,A32,A33;
	double CosA,SinA;
	CosA = cos(-Ang * DtoR);
	SinA = sin(-Ang * DtoR);
	
	A11 = U.x*U.x + CosA * (1 - U.x*U.x);
	A12 = U.x*U.y * (1-CosA) - U.z * SinA;
	A13 = U.z*U.x * (1-CosA) + U.y * SinA;
	A21 = U.x*U.y * (1-CosA) + U.z * SinA;
	A22 = U.y*U.y + CosA * (1 - U.y*U.y);
	A23 = U.y*U.z * (1-CosA) - U.x * SinA;
	A31 = U.z*U.x * (1-CosA) - U.y * SinA;
	A32 = U.y*U.z * (1-CosA) + U.x * SinA;
	A33 = U.z*U.z + CosA * (1 - U.z * U.z);

	//Perform rotation R = Ar;
	R.x = A11 * r.x + A12 * r.y + A13 * r.z;
	R.y = A21 * r.x + A22 * r.y + A23 * r.z;
	R.z = A31 * r.x + A32 * r.y + A33 * r.z;



	//Convert back to Celest cords
	//Note: sin returns -90 to 90, cos returns 0 to 180.
	Phi = acos(R.z);
	
	//This to fix probs at north pole
	double Temp = R.x/sin(Phi);
	if(Temp > 1)
		Temp = 1;
	if (Temp < -1)
		Temp = -1;

	Theta = acos(Temp);
	RAB =Theta * RtoD;
	if(R.y >= 0)
		RA = RAB;
	else
		RA = 360 - RAB;

	DecB = Phi * RtoD;
	Dec = 90 - DecB;

	SLoc.RA = (float)RA;
	SLoc.Dec = (float)Dec;

	return(SLoc);
}

/////////////////////////////////////////////////////////////
//CDist
//  Cart (xy) dist
//  
//
//  
//
/////////////////////////////////////////////////////////////
float CVMath::CDist(const int &x1,const int &y1,
					const int &x2, const int &y2)
{
	float Dist;

	Dist = (float)sqrt( pow((float(x1 - x2)),2) 
			    +pow((float(y1 - y2)),2));

	return(Dist);
}

/////////////////////////////////////////////////////////////
//CDist
//  Cart (xy) dist
//  
//
//  
//
/////////////////////////////////////////////////////////////
void CVMath::Celest2AltAz(const double &Lat, const double &Long, 
						  const double &JD,
						    const double &RAdeg,const double &Dec,
							double &Alt, double &Az)
{
	double zd,az,rar,decr;
	site_info geo_loc;
	//body earth, pallas;
	//cat_entry star;
	double deltat = 0.0;
	double RAhours = RAdeg / 15;

	//geo_loc = {45.0, -75.0, 0.0, 10.0, 1010.0};
	geo_loc.latitude	= Lat;
	geo_loc.longitude	= Long;       
	geo_loc.height      = 0.0;    
	geo_loc.temperature = 10.0;   
	geo_loc.pressure    = 1010.0; 

	N.equ2hor (JD,deltat,0,0,
				&geo_loc, RAhours,Dec,
				0,
				&zd,&az,&rar,&decr);

	Alt = 90 - zd;
	Az = az;
	
}
/////////////////////////////////////////////////////////////
//
//  
//  
//
//  
//
/////////////////////////////////////////////////////////////
double CVMath::RectMag(const RECTVEC &Vec)
{
	double Mag;

	Mag = sqrt(Vec.x * Vec.x + Vec.y * Vec.y + Vec.z * Vec.z);
	

	return (Mag);
	
}

/////////////////////////////////////////////////////////////
//
//  
//  
//
//  
//
/////////////////////////////////////////////////////////////
void CVMath::RAdeg2RAhm(double RAdeg,
						 float &RAh,float &RAm)
{
	if(RAdeg < 0) RAdeg = RAdeg + 360.0;
	RAh = (float)((int)(RAdeg/15.0));
	RAm =(float)((RAdeg - RAh * 15.0) * 4.0);
	
}

void CVMath::RAdeg2RAhms(double RAdeg,
						 float *RAh,float *RAm, float *RAs)
{
	double rem;
	if(RAdeg < 0) RAdeg = RAdeg + 360;
	*RAh = (float)((int)(RAdeg/15));
	rem = RAdeg - (*RAh * 15);
	*RAm =(float)( (int)(rem * 4) );
	rem = rem - *RAm / 4;
	*RAs = (float)(rem * 240.0);
}

void CVMath::RAStr(double RAdeg, char *Out)
{
	char outstr[20];
	float h,m,s;
	strcpy(Out,"");

	RAdeg2RAhms(RAdeg,&h,&m,&s);
	sprintf( outstr, "%02.0f:%02.0f:%05.2f",
						h,m,s);
	strcat(Out,outstr);
}


void CVMath::deg2dms(double deg,
						 float *d,float *m, float *s)
{
	double rem;
	if(deg < 0) deg = deg + 360;
	*d = (float)((int)(deg));
	rem = deg - *d;
	*m =(float)( (int)(rem * 60) );
	rem = rem - *m/60;
	*s = (float)(rem * 60.0 * 60.0);
}

void CVMath::DecStr(double deg, char *Out)
{
	char outstr[20];
	float d,m,s;
	strcpy(Out,"");
	if(deg < 0)
	{
		deg = - deg;
		strcpy(Out,"-");
	}

	deg2dms(deg,&d,&m,&s);
	sprintf( outstr, "%2.0f:%02.0f:%04.1f",
						d,m,s);
	strcat(Out,outstr);
}

/////////////////////////////////////////////////////////////
//GetObjectData
//  Interface to astro, returns object(planet,sun,moon)
//  positionand dia(eventually)
//
// time eqs:
//	MODJD = astro.MJD((int)p_Day,(int)p_Month,(int)p_Year,(double)p_Hour);  
//	T = (MODJD-51544.5)/36525.0;
//	JD = MODJD+2400000.5;
//  
// Mode = vm_Equinox;	//0=astromeric,1=J2000,2=D1950
//  
// Note, Compared to p129, Montenbruck, Pfleger
//  for both astromeric and J2000 cases. All results
//  within 1.5" except Jupiter, which was within 5".
//  1/20/2000 jac...ok.
/////////////////////////////////////////////////////////////
void CVMath::GetObjectData(int Object,double JDin, int Mode,
						   double *RAdeg,double *Dec,
						   double *Dia)
{
	double T;

	//convert to MODJD, then julian centurys
	T =( (JDin-2400000.5) - 51544.5) / 36525.0;

	//int Mode;
	REAL33 A;

	double  IMODE;
    double  TEQX;
    double X,Y,Z, XP,YP,ZP, XS,YS,ZS;
    double L,B,R, LS,BS,RS, RA,DEC,DELTA,DELTA0;



	astro.SUN200(T,&LS,&BS,&RS);

	//set IMODE for GEOCEN function
	//Mode = 0;//0=atromeric,1=J200, 2= B1950
	if(Mode == 0)//astromeric
	{
		IMODE = 2;
	}
	else//other
	{
		IMODE = 1;
	}

	switch(Object)
	{	case 0:{L=0;B=0;R=0;break;}
		case 1:{astro.MER200(T,&L,&B,&R);break;}
		case 2:{astro.VEN200(T,&L,&B,&R);break;}
		case 3:{L=LS+180;B=-BS;R=RS;break;}
		case 4:{astro.MAR200(T,&L,&B,&R);break;}
		case 5:{astro.JUP200(T,&L,&B,&R);break;}
		case 6:{astro.SAT200(T,&L,&B,&R);break;}
		case 7:{astro.URA200(T,&L,&B,&R);break;}
		case 8:{astro.NEP200(T,&L,&B,&R);break;}
		case 9:{astro.PLU200(T,&L,&B,&R);break;}
	}
	
	astro.GEOCEN(T, L,B,R, LS,BS,RS, Object,(int)IMODE,
				&XP,&YP,&ZP, &XS,&YS,&ZS, &X,&Y,&Z,&DELTA0);

	int d=1;

	if(Mode == 0)//astromeric
	{
		astro.ECLEQU(T,&X,&Y,&Z); 
		astro.NUTEQU(T,&X,&Y,&Z); 
	}
	else//other
	{
		if(Mode == 1) TEQX = 0.0;//J2000
		if(Mode == 2) TEQX = -0.500002108;//B1950

		astro.PMATECL(T,TEQX,&A);
		astro.PRECART (A,&XP,&YP,&ZP);
		astro.PRECART (A,&X,&Y,&Z); 
		astro.ECLEQU (TEQX,&X,&Y,&Z);
	}

	astro.POLAR(XP,YP,ZP,&R,&B,&L);
	astro.POLAR(X,Y,Z,&DELTA,&DEC,&RA); 
	//RA=RA/15.0;

	*Dec = DEC;
	*RAdeg = RA;
	
	switch(Object)
	{	case 0:{*Dia = .5;break;}
		case 1:{*Dia = 0;break;}
		case 2:{*Dia = 0;break;}
		case 3:{*Dia = 0;break;}
		case 4:{*Dia = 0;break;}
		case 5:{*Dia = 0;break;}
		case 6:{*Dia = 0;break;}
		case 7:{*Dia = 0;break;}
		case 8:{*Dia = 0;break;}
		case 9:{*Dia = 0;break;}
	}
}

/////////////////////////////////////////////////////////////
//GetObjectData
//  Interface to astro, returns object(planet,sun,moon)
//  position and dia(eventually)
//  
//
//  
//
/////////////////////////////////////////////////////////////

void CVMath::GetMoonData(double JD, double *RAdeg,double *Dec,
						   double *Dia)
{
	double T,RA,DEC;

	//convert to MODJD, then julian centuries
	T =( (JD-2400000.5) - 51544.5) / 36525.0;

	astro.MINI_MOON(T, &RA, &DEC);

	*Dec = DEC;
	*RAdeg = RA * 15;
	*Dia = .5;
}

void CVMath::PlanetPosStr( char * out, double JulianDate,
						  int Equinox, double Lat, double Long)
{
	int Object;
	double RAdeg,Dec,Dia;
	double v_Alt,v_Az;

	char line[80];
	char outstr[40];

	strcpy(out, "Planet Positions:\n");

	for(Object=0;Object<=10;Object++)
	{
		if(Object !=3)
		{
			switch(Object)
			{	case 0:{strcpy(line,"Sun         ");break;}
				case 1:{strcpy(line,"Mercury ");break;}
				case 2:{strcpy(line,"Venus     ");break;}
				case 3:{strcpy(line,"Earth     ");break;}
				case 4:{strcpy(line,"Mars       ");break;}
				case 5:{strcpy(line,"Jupiter    ");break;}
				case 6:{strcpy(line,"Saturn     ");break;}
				case 7:{strcpy(line,"Uranus    ");break;}
				case 8:{strcpy(line,"Neptune  ");break;}
				case 9:{strcpy(line,"Pluto        ");break;}
				case 10:{strcpy(line,"Moon       ");break;}
			}

			if(Object == 10)
			{
				GetMoonData(JulianDate, &RAdeg,&Dec,
						    &Dia);
			}
			else//not moon
			{
				GetObjectData(Object,JulianDate, Equinox,
							   &RAdeg,&Dec,&Dia);
			}

			strcat(line,"RA=");
			RAStr(RAdeg,outstr);	
			strcat(line,outstr);
			strcat(line,"  Dec=");
			DecStr(Dec,outstr);
			strcat(line,outstr);

			Celest2AltAz(Lat,Long,JulianDate,
						RAdeg,Dec,v_Alt,v_Az);
			sprintf( outstr, " Alt=%-6.2f Az=%-6.2f",
					v_Alt,v_Az);
			strcat(line,outstr);
			strcat(out,line);strcat(out,"\n");
		}
	}
}
/////////////////////////////////////////////////////////////
//FigurePlanetInfo
//  Figures planet info into global PlanInfo array.
//  also sets data into PlanInfoStr Global.
//
//  Inputs: EMode = vm_Equinox;	//0=astromeric,1=J2000,2=D1950
//          CMode = vm_CentricMode // 0=geocentric, 1= topocentric
//          Zenith = STARDATA position of zenith, for topocentric mode
//
//  
//
/////////////////////////////////////////////////////////////

void CVMath::FigurePlanetInfo(double JD,int EMode,int CMode,
							  double Lat, double Long,
							  STARDATA Zenith)
{
	double RAdeg,Dec,Dia,Dist;
	double T;// MODJD;
	REAL33 A;
	double  IMODE;
    double  TEQX;
    double X,Y,Z, XP,YP,ZP, XS,YS,ZS;
    double L,B,R, LS,BS,RS, RA,DEC,DELTA,DELTA0;
	double LE,BE,RE,Dist_m,Erad_m;	//earth heliocentric
	double v_Alt,v_Az;
	int Object;
	char line[200];
	char outstr[80];
	RECTVEC GCVec,EVec,TCVec;

	//set IMODE for GEOCEN function
	//Mode = 0;//0=atromeric,1=J200, 2= B1950
	if(EMode == 0)IMODE = 2;	//astromeric
	else IMODE = 1;				//other process to date

	//build string header
	if(CMode == 0)
		strcpy(PlanInfoStr," Geocentric Planet Positions:\n");
	else if(CMode == 1)
		strcpy(PlanInfoStr,"Topocentric Planet Positions:\n");

	if(EMode == 0)
		strcat(PlanInfoStr,"Equinox of Date\n");
	else if(EMode == 1)
		strcat(PlanInfoStr,"Equinox J2000\n");
	else if(EMode == 2)
		strcat(PlanInfoStr,"Equinox B1950\n");

	//convert to MODJD, then julian centurys
	T =( (JD-2400000.5) - 51544.5) / 36525.0;

	//get sun position
	astro.SUN200(T,&LS,&BS,&RS);

	//heliocentric earth position
	LE=LS+180;
	BE=-BS;
	RE=RS;

	//figure all objects
	for(Object=0;Object<=10;Object++)
	{
		if(Object < 10 )//if object planet
		{
			//figure heliocentric coord L,B,R
			switch(Object)
			{	case 0:{L=0;B=0;R=0;break;}
				case 1:{astro.MER200(T,&L,&B,&R);break;}
				case 2:{astro.VEN200(T,&L,&B,&R);break;}
				case 3:{L=LS+180;B=-BS;R=RS;break;}
				case 4:{astro.MAR200(T,&L,&B,&R);break;}
				case 5:{astro.JUP200(T,&L,&B,&R);break;}
				case 6:{astro.SAT200(T,&L,&B,&R);break;}
				case 7:{astro.URA200(T,&L,&B,&R);break;}
				case 8:{astro.NEP200(T,&L,&B,&R);break;}
				case 9:{astro.PLU200(T,&L,&B,&R);break;}
			}

			PlanInfo[Object][6] = L;
			PlanInfo[Object][7] = B;
			PlanInfo[Object][8] = R;			
			
			astro.GEOCEN(T, L,B,R, LS,BS,RS, Object,(int)IMODE,
						&XP,&YP,&ZP, &XS,&YS,&ZS, &X,&Y,&Z,&DELTA0);

			Dist = DELTA0;

			//convert from AU to meters
			//(1AU=1.49597879 x 10^11 m)
			Dist_m = Dist *1.49597870E11;

			//Earth Radius
			Erad_m = 6.37814E5;

			int d=1;

			if(EMode == 0)//astromeric
			{
				astro.ECLEQU(T,&X,&Y,&Z); 
				astro.NUTEQU(T,&X,&Y,&Z); 
			}
			else//other
			{
				if(EMode == 1) TEQX = 0.0;//J2000
				if(EMode == 2) TEQX = -0.500002108;//B1950

				astro.PMATECL(T,TEQX,&A);
				astro.PRECART (A,&XP,&YP,&ZP);
				astro.PRECART (A,&X,&Y,&Z); 
				astro.ECLEQU (TEQX,&X,&Y,&Z);
			}

			astro.POLAR(XP,YP,ZP,&R,&B,&L);
			astro.POLAR(X,Y,Z,&DELTA,&DEC,&RA); 
			//RA=RA/15.0;

			Dec = DEC;
			RAdeg = RA;
			
			switch(Object)
			{	case 0:{Dia = .5;break;}
				case 1:{Dia = 0;break;}
				case 2:{Dia = 0;break;}
				case 3:{Dia = 0;break;}
				case 4:{Dia = 0;break;}
				case 5:{Dia = 0;break;}
				case 6:{Dia = 0;break;}
				case 7:{Dia = 0;break;}
				case 8:{Dia = 0;break;}
				case 9:{Dia = 0;break;}
			}

		}

		else
		if(Object == 10)//if mini_moon
		{

			GetMoonData(JD, &RAdeg,&Dec,
						    &Dia);
			PlanInfo[Object][6] = 0;
			PlanInfo[Object][7] = 0;
			PlanInfo[Object][8] = 0;
			Dist_m = 60 * Erad_m;
			Dia = .5;

		}

		else
		if(Object == 11)//if moon
		{
			double	ML,MB,MR;

			astro.MOON(T,&ML,&MB,&MR);
			astro.MOONEQU(T,&RAdeg,&Dec,&Dist);

			PlanInfo[Object][6] = ML;
			PlanInfo[Object][7] = MB;
			PlanInfo[Object][8] = MR;
			Dist_m = MR * Erad_m;
			Dia = .55;


		}




		//if topocentric, adjust
		if(CMode == 1)
		{
			
			GCVec = StoRect(RAdeg,Dec);
			GCVec.x *= Dist_m;
			GCVec.y *= Dist_m;
			GCVec.z *= Dist_m;

			//topo vector
			EVec = StoRect(Zenith.RA,Zenith.Dec);
			EVec.x *= Erad_m;
			EVec.y *= Erad_m;
			EVec.z *= Erad_m;

			//diff
			TCVec.x = GCVec.x - EVec.x;
			TCVec.y = GCVec.y - EVec.y;
			TCVec.z = GCVec.z - EVec.z;

			//convert to celest coords again...
			RecttoS(TCVec.x, TCVec.y, TCVec.z,
		             &RAdeg, &Dec );

		}

		//put info in data array
		PlanInfo[Object][0] = RAdeg;
		PlanInfo[Object][1] = Dec;
		PlanInfo[Object][2] = Dia;
		PlanInfo[Object][3] = Dist;
		
		//Build data line:
		switch(Object)
		{	case 0:{strcpy(line,"Sun         ");break;}
			case 1:{strcpy(line,"Mercury ");break;}
			case 2:{strcpy(line,"Venus     ");break;}
			case 3:{strcpy(line,"Earth     ");break;}
			case 4:{strcpy(line,"Mars       ");break;}
			case 5:{strcpy(line,"Jupiter    ");break;}
			case 6:{strcpy(line,"Saturn     ");break;}
			case 7:{strcpy(line,"Uranus    ");break;}
			case 8:{strcpy(line,"Neptune  ");break;}
			case 9:{strcpy(line,"Pluto        ");break;}
			case 10:{strcpy(line,"Moon       ");break;}
			case 11:{strcpy(line,"Moon2      ");break;}
		}

		strcat(line,"RA=");
		RAStr(RAdeg,outstr);	
		strcat(line,outstr);
		strcat(line,"  Dec=");
		DecStr(Dec,outstr);
		strcat(line,outstr);

		Celest2AltAz(Lat,Long,JD,
					RAdeg,Dec,v_Alt,v_Az);

		PlanInfo[Object][4] = v_Alt;
		PlanInfo[Object][5] = v_Az;

		sprintf( outstr, " Alt=%-6.2f Az=%-6.2f\n",
				v_Alt,v_Az);
		strcat(line,outstr);

		sprintf( outstr, " L=%-6.2f B=%-6.2f R=%6.4f",
				PlanInfo[Object][6],PlanInfo[Object][7],
				PlanInfo[Object][8]);
		strcat(line,outstr);


		strcat(PlanInfoStr,line);strcat(PlanInfoStr,"\n");


	}



}

/////////////////////////////////////////////////////////////
//GetPlanetInfo
//  Gets planet info, must be preceeded by cal to
//  FigurePlanetInfo.
	//PlanInfo[n][ ] = Object number
	//PlanInfo[n][0] = Radeg
	//PlanInfo[n][1] = Dec
	//PlanInfo[n][2] = Dia
	//PlanInfo[n][3] = Dist
	//PlanInfo[n][4] = Alt
	//PlanInfo[n][5] = Az
	//PlanInfo[n][6] = L
	//PlanInfo[n][7] = B
	//PlanInfo[n][8] = R
/////////////////////////////////////////////////////////////

void CVMath::GetPlanetInfo(int Object,
						   double *RAdeg,double *Dec,
						   double *Dia, double *Dist,
						   double *Alt, double *Az,
						   double *L, double *B,double *R)
{
	*RAdeg = PlanInfo[Object][0];
	*Dec = PlanInfo[Object][1];
	*Dia = PlanInfo[Object][2];
	*Dist = PlanInfo[Object][3];
	*Alt = PlanInfo[Object][4];
	*Az = PlanInfo[Object][5];
	*L = PlanInfo[Object][6];
	*B = PlanInfo[Object][7];
	*R = PlanInfo[Object][8];
}





/////////////////////////////////////////////////////////////
//
//  
// 
//  
//
//  
//
/////////////////////////////////////////////////////////////


