//starcat, Star Catalog Visualization Tool
// (C) 2000  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


// astro.cpp: implementation of the Castro class.
//
//////////////////////////////////////////////////////////////////////

//#include ""
#include "astro.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Castro::Castro()
{

}

Castro::~Castro()
{

}

//(*-----------------------------------------------------------------------*)
//(* SN: sine function (degrees)                                           *)
//(*-----------------------------------------------------------------------*) 
//FUNCTION SN(X:REAL):REAL;
double Castro::SN(double X)
{
	double SN;
	double RAD=0.0174532925199433; 
 
	SN = sin(X*RAD); 

	return(SN);
}

//(*-----------------------------------------------------------------------*)
//(* CS: cosine function (degrees)                                         *)
//(*-----------------------------------------------------------------------*)
//FUNCTION CS(X:REAL):REAL;
double Castro::CS(double X)
{
	double CS;
	double RAD=0.0174532925199433; 
 
	CS = cos(X*RAD); 

	return(CS);
}

//(*-----------------------------------------------------------------------*)
//(* TN: tangent function (degrees)                                        *)
//(*-----------------------------------------------------------------------*)
//FUNCTION TN(X:REAL):REAL;
double Castro::TN(double X)
{
	double RAD=0.0174532925199433;
	double XX,TN;

    XX = X*RAD; 
	TN = sin(XX)/cos(XX);

	return(TN);
}

//(*-----------------------------------------------------------------------*)
//(* ATN: arctangent function (degrees)                                    *)
//(*-----------------------------------------------------------------------*)
//FUNCTION ATN(X:REAL):REAL;
double Castro::ATN(double X)
{
	double ATN;
	double RAD = 0.0174532925199433;  
 
    ATN = atan(X)/RAD;
	return(ATN);

}

//(*-----------------------------------------------------------------------*)
//(* ATN2: arctangent of y/x for two arguments                             *)
//(*       (correct quadrant; -180 deg <= ATN2 <= +180 deg)                *)
//(*-----------------------------------------------------------------------*)
//FUNCTION ATN2(Y,X:REAL):REAL;
double Castro::ATN2(double Y, double X)
{
  double RAD=0.0174532925199433;
  double   AX,AY,PHI;
  double ATN2;

	if(X == 0.0 && Y == 0.0)
	{
		ATN2 = 0.0;
	}
	else
	{
		AX = fabs(X); 
		AY = fabs(Y); 

		if(AX>AY)
		{
			PHI = atan(AY/AX)/RAD;
		}
		else
		{
			PHI = 90.0 - atan(AX/AY)/RAD;
		}
		
		if(X < 0.0)  PHI =180.0-PHI;
		if(Y < 0.0)  PHI = -PHI;

		ATN2 = PHI;
	}

	return(ATN2);
}

//(*-----------------------------------------------------------------------*)
//(* POLAR: conversion of cartesian coordinates (x,y,z)                    *)
//(*        into polar coordinates (r,theta,phi)                           *)
//(*        (theta in [-90 deg,+90 deg]; phi in [0 deg,+360 deg])          *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE POLAR(X,Y,Z:REAL;VAR R,THETA,PHI:REAL);
//  VAR RHO: REAL;
//  BEGIN
//    RHO:=X*X+Y*Y;  R:=SQRT(RHO+Z*Z);  
//    PHI:=ATN2(Y,X); IF PHI<0 THEN PHI:=PHI+360.0;
//    RHO:=SQRT(RHO); THETA:=ATN2(Z,RHO);
//  END;

void Castro::POLAR(double X, double Y, double Z,
				   double *R, double *THETA, double *PHI)
{
	double RHO;

	RHO = X*X+Y*Y;  
	*R = sqrt(RHO+Z*Z);  
    *PHI = ATN2(Y,X); 
	if(*PHI<0) *PHI = *PHI + 360.0;
    RHO = sqrt(RHO); 
	*THETA = ATN2(Z,RHO);
}

//(*-----------------------------------------------------------------------*)
//(* CART: conversion of polar coordinates (r,theta,phi)                   *)
//(*       into cartesian coordinates (x,y,z)                              *)
//(*       (theta in [-90 deg,+90 deg]; phi in [-360 deg,+360 deg])        *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE CART(R,THETA,PHI: REAL; VAR X,Y,Z: REAL);
void Castro::CART(double R, double THETA, double PHI,
				  double *X, double *Y, double *Z)
{
  double RCST;

    RCST = R * CS(THETA);
    *X   = RCST * CS(PHI); 
	*Y = RCST * SN(PHI); 
	*Z = R * SN(THETA);
}

//(*-----------------------------------------------------------------------*)
//(* DMS: conversion of degrees and fractions of a degree                  *)
//(*      into degrees, minutes and seconds                                *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE DMS(DD:REAL;VAR D,M:INTEGER;VAR S:REAL);
//  VAR D1:REAL; 
//  BEGIN
//    D1:=ABS(DD);  D:=TRUNC(D1);  
//    D1:=(D1-D)*60.0;  M:=TRUNC(D1);  S:=(D1-M)*60.0;
//    IF (DD<0) THEN 
//      IF (D<>0) THEN D:=-D ELSE IF (M<>0) THEN M:=-M ELSE S:=-S;
//  END;

void Castro::DMS(double DD, 
				 int *D, int *M, double *S)
{
	double D1;

	D1 = fabs(DD);  
	*D = (int)D1;  
    D1 = (D1-*D)*60.0;
	*M = (int)(D1);
	*S = (D1-*M)*60.0;
    if(DD<0) 
	{
		if(*D !=0 )  
		{
			*D = - *D; 
		}
		else
		{
			if(*M != 0) 
			{
				*M = -*M; 
			}
			else
			{
				*S = -*S;
			}
		}
	}
}

//(*-----------------------------------------------------------------------*)
//(* MJD: Modified Julian Date                                             *)
//(*      The routine is valid for any date since 4713 BC.                 *)
//(*      Julian calendar is used up to 1582 October 4,                    *)
//(*      Gregorian calendar is used from 1582 October 15 onwards.         *)
//(*-----------------------------------------------------------------------*)
//FUNCTION MJD(DAY,MONTH,YEAR:INTEGER;HOUR:REAL):REAL;
//  VAR A: REAL; B: INTEGER;
//  BEGIN
//    A:=10000.0*YEAR+100.0*MONTH+DAY;
//    IF (MONTH<=2) THEN BEGIN MONTH:=MONTH+12; YEAR:=YEAR-1 END;
//    IF (A<=15821004.1)
//      THEN B:=-2+TRUNC((YEAR+4716)/4)-1179
//      ELSE B:=TRUNC(YEAR/400)-TRUNC(YEAR/100)+TRUNC(YEAR/4);
//    A:=365.0*YEAR-679004.0;
//    MJD:=A+B+TRUNC(30.6001*(MONTH+1))+DAY+HOUR/24.0;
//  END;

double Castro::MJD(int DAY, int MONTH, int YEAR, double HOUR)
{
	double A,MJD;
	int B;

	A = 10000.0*YEAR+100.0*MONTH+DAY;

	if(MONTH <= 2)
	{
		MONTH = MONTH+12; 
		YEAR = YEAR-1;
	}

	if(A<=15821004.1)
    {
		B = -2+(int)((YEAR+4716)/4)-1179;
	}
    else
	{
		B = (int)(YEAR/400)-(int)(YEAR/100)+(int)(YEAR/4);
	}

	A = 365.0*YEAR-679004.0;
	MJD = A+B+(int)(30.6001*(MONTH+1))+DAY+HOUR/24.0;

	return(MJD);
}


//(*-----------------------------------------------------------------------*)
//(* ECLEQU: Conversion of ecliptic into equatorial coordinates            *)
//(*         (T: equinox in Julian centuries since J2000)                  *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE ECLEQU(T:REAL;VAR X,Y,Z:REAL);
//  VAR EPS,C,S,V: REAL;
//  BEGIN
//    EPS:=23.43929111-(46.8150+(0.00059-0.001813*T)*T)*T/3600.0;
//    C:=CS(EPS);  S:=SN(EPS);
//    V:=+C*Y-S*Z;  Z:=+S*Y+C*Z;  Y:=V;
//  END;

void Castro::ECLEQU(double TT, 
			double *X, double *Y, double *Z)
{
	double EPS,C,S,V;

	EPS = 23.43929111-(46.8150+(0.00059-0.001813*TT)*TT)*TT/3600.0;
	C = CS(EPS);  
	S = SN(EPS);
	V = +C * *Y - S * *Z;  
	*Z = +S * *Y + C * *Z ;  
	*Y = V;
}

//(*-----------------------------------------------------------------------*)
//(* PMATECL: calculates the precession matrix A[i,j] for                  *)
//(*          transforming ecliptic coordinates from equinox T1 to T2      *)
//(*          ( T=(JD-2451545.0)/36525 )                                   *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE PMATECL(T1,T2:REAL;VAR A: REAL33);
//  CONST SEC=3600.0;
//  VAR DT,PPI,PI,PA: REAL;
//      C1,S1,C2,S2,C3,S3: REAL;
//  BEGIN
//    DT:=T2-T1;
//    PPI := 174.876383889 +( ((3289.4789+0.60622*T1)*T1) +
//              ((-869.8089-0.50491*T1) + 0.03536*DT)*DT )/SEC;
//    PI  := ( (47.0029-(0.06603-0.000598*T1)*T1)+
//             ((-0.03302+0.000598*T1)+0.000060*DT)*DT )*DT/SEC;
//    PA  := ( (5029.0966+(2.22226-0.000042*T1)*T1)+
//             ((1.11113-0.000042*T1)-0.000006*DT)*DT )*DT/SEC;
//    C1:=CS(PPI+PA);  C2:=CS(PI);  C3:=CS(PPI);
//    S1:=SN(PPI+PA);  S2:=SN(PI);  S3:=SN(PPI);
//    A[1,1]:=+C1*C3+S1*C2*S3; A[1,2]:=+C1*S3-S1*C2*C3; A[1,3]:=-S1*S2;
//    A[2,1]:=+S1*C3-C1*C2*S3; A[2,2]:=+S1*S3+C1*C2*C3; A[2,3]:=+C1*S2;
//    A[3,1]:=+S2*S3;          A[3,2]:=-S2*C3;          A[3,3]:=+C2;
//  END;

void Castro::PMATECL(double T1, double T2, 
						REAL33 *A)
{
	double SEC=3600.0;
	double DT,PPI,PI,PA;
	double C1,S1,C2,S2,C3,S3;

	DT = T2-T1;

    PPI = 174.876383889 +( ((3289.4789+0.60622*T1)*T1) +
              ((-869.8089-0.50491*T1) + 0.03536*DT)*DT )/SEC;
    PI  = ( (47.0029-(0.06603-0.000598*T1)*T1)+
             ((-0.03302+0.000598*T1)+0.000060*DT)*DT )*DT/SEC;
    PA  = ( (5029.0966+(2.22226-0.000042*T1)*T1)+
             ((1.11113-0.000042*T1)-0.000006*DT)*DT )*DT/SEC;
    C1 = CS(PPI+PA);  
	C2 = CS(PI);  
	C3 = CS(PPI);
    S1 = SN(PPI+PA);  
	S2 = SN(PI);  
	S3 = SN(PPI);
    A->A11 = +C1*C3+S1*C2*S3;
	A->A12 =+C1*S3-S1*C2*C3; 
	A->A13 =-S1*S2;
    A->A21 =+S1*C3-C1*C2*S3; 
	A->A22 =+S1*S3+C1*C2*C3; 
	A->A23 =+C1*S2;
    A->A31 =+S2*S3;          
	A->A32 =-S2*C3;          
	A->A33 =+C2;

}

//(*-----------------------------------------------------------------------*)
//(* PRECART: calculate change of coordinates due to precession            *)
//(*          for given transformation matrix A[i,j]                       *)
//(*          (to be used with PMATECL und PMATEQU)                        *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE PRECART(A:REAL33; VAR X,Y,Z:REAL);
//  VAR U,V,W: REAL;
//  BEGIN
//    U := A[1,1]*X+A[1,2]*Y+A[1,3]*Z;
//    V := A[2,1]*X+A[2,2]*Y+A[2,3]*Z;
//    W := A[3,1]*X+A[3,2]*Y+A[3,3]*Z;
//    X:=U; Y:=V; Z:=W;
//  END;

void Castro::PRECART(REAL33 A, double *X, double *Y,double *Z)
{
	double U,V,W;
	U = A.A11 * *X + A.A12 * *Y + A.A13 * *Z;
    V = A.A21 * *X + A.A22 * *Y + A.A23 * *Z;
	W = A.A31 * *X + A.A32 * *Y + A.A33 * *Z;
    *X = U; 
	*Y = V; 
	*Z = W;
}

//(*-----------------------------------------------------------------------*)
//(* NUTEQU: transformation of mean to true coordinates                    *)
//(*         (including terms >0.1" according to IAU 1980)                 *)
//(*         T = (JD-2451545.0)/36525.0                                    *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE NUTEQU(T:REAL;VAR X,Y,Z:REAL);
//  CONST ARC=206264.8062;          (* arcseconds per radian = 3600*180/pi *)
//        P2 =6.283185307;          (* 2*pi                                *)
//  VAR   LS,D,F,N,EPS : REAL;
//        DPSI,DEPS,C,S: REAL;
//        DX,DY,DZ     : REAL;
//  FUNCTION FRAC(X:REAL):REAL;
//    (* with several compilers it may be necessary to replace TRUNC *)
//    (* by LONG_TRUNC or INT if T<-24!                              *)
//    BEGIN  FRAC:=X-TRUNC(X) END;
//  BEGIN
//    LS  := P2*FRAC(0.993133+  99.997306*T); (* mean anomaly Sun          *)
//    D   := P2*FRAC(0.827362+1236.853087*T); (* diff. longitude Moon-Sun  *)
//    F   := P2*FRAC(0.259089+1342.227826*T); (* mean argument of latitude *)
//    N   := P2*FRAC(0.347346-   5.372447*T); (* longit. ascending node    *)
//    EPS := 0.4090928-2.2696E-4*T;           (* obliquity of the ecliptic *)
//    DPSI := ( -17.200*SIN(N)   - 1.319*SIN(2*(F-D+N)) - 0.227*SIN(2*(F+N))
//              + 0.206*SIN(2*N) + 0.143*SIN(LS) ) / ARC;
//    DEPS := ( + 9.203*COS(N)   + 0.574*COS(2*(F-D+N)) + 0.098*COS(2*(F+N))
//              - 0.090*COS(2*N)                 ) / ARC;
//    C := DPSI*COS(EPS);  S := DPSI*SIN(EPS);
//    DX := -(C*Y+S*Z); DY := (C*X-DEPS*Z); DZ := (S*X+DEPS*Y);
//    X  :=  X + DX;        Y  := Y + DY;       Z  := Z + DZ;
//  END;

void Castro::NUTEQU(double TT,
					 double *X, double *Y,double *Z)
{
	double ARC=206264.8062;//(* arcseconds per radian = 3600*180/pi *)
	double P2 =6.283185307;//(* 2*pi *)

	double LS,D,F,N,EPS;
	double DPSI,DEPS,C,S;
	double DX,DY,DZ;
	LS  = P2*NU_FRAC(0.993133+  99.997306*TT); //(* mean anomaly Sun          *)
	D   = P2*NU_FRAC(0.827362+1236.853087*TT); //(* diff. longitude Moon-Sun  *)
	F   = P2*NU_FRAC(0.259089+1342.227826*TT); //(* mean argument of latitude *)
	N   = P2*NU_FRAC(0.347346-   5.372447*TT); //(* longit. ascending node    *)
	EPS = 0.4090928-2.2696E-4*TT;           //(* obliquity of the ecliptic *)
	DPSI = ( -17.200*sin(N)   - 1.319*sin(2*(F-D+N)) - 0.227*sin(2*(F+N))
				 + 0.206*sin(2*N) + 0.143*sin(LS) ) / ARC;
	DEPS = ( + 9.203*cos(N)   + 0.574*cos(2*(F-D+N)) + 0.098*cos(2*(F+N))
				 - 0.090*cos(2*N)                 ) / ARC;
	C = DPSI*cos(EPS);  
	S = DPSI*sin(EPS);
	DX = -(C * *Y + S * *Z); 
	DY = (C * *X - DEPS * *Z); 
	DZ = (S * *X + DEPS * *Y);
	*X  =  *X + DX;        
	*Y  = *Y + DY;       
	*Z  = *Z + DZ;
}

//  FUNCTION FRAC(X:REAL):REAL;
//    (* with several compilers it may be necessary to replace TRUNC *)
//    (* by LONG_TRUNC or INT if T<-24!                              *)
//    BEGIN  FRAC:=X-TRUNC(X) END;
double Castro::NU_FRAC(double X)
{
	return((double)( X - (int)X) );
}

//(*-----------------------------------------------------------------------*)
//(*                                                                       *)
//* GEOCEN: geocentric coordinates (geometric and light-time corrected)   *)
//(*                                                                       *)
//(*   T:        time in Julian centuries since J2000                      *)
//(*             T=(JD-2451545.0)/36525.0                                  *)
//(*   LP,BP,RP: ecliptic heliocentric coordinates of the planet           *)
//(*   LS,BS,RS: ecliptic geocentric   coordinates of the sun              *)
//(*                                                                       *)
//(*   IPLAN:    planet (0=Sun,1=Mercury,2=Venus,3=Earth,...,9=Pluto       *)
//(*   IMODE:    desired type of coordinates (see description of X,Y,Z)    *)
//(*             (0=geometric,1=astrometric,2=apparent)                    *)
//(*   XP,YP,ZP: ecliptic heliocentric coordinates of the planet           *)
//(*   XS,YS,ZS: ecliptic geocentric coordinates of the Sun                *)
//(*   X, Y, Z : ecliptic geocentric cordinates of the planet (geometric   *)
//(*             if IMODE=0, astrometric if IMODE=1, apparent if IMODE=2)  *)
//(*   DELTA0:   geocentric distance (geometric)                           *)
//(*                                                                       *)
//(*   (all angles in degrees, distances in AU)                            *)
//(*                                                                       *)
//(*-----------------------------------------------------------------------*)
/*
PROCEDURE GEOCEN(T, LP,BP,RP, LS,BS,RS: REAL; IPLAN,IMODE: INTEGER;
                 VAR XP,YP,ZP, XS,YS,ZS, X,Y,Z,DELTA0: REAL);

  CONST P2=6.283185307;

  VAR DL,DB,DR, DLS,DBS,DRS, FAC: REAL;
      VX,VY,VZ, VXS,VYS,VZS, M  : REAL;

  FUNCTION FRAC(X:REAL):REAL;
    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1; FRAC:=X  END;

  PROCEDURE POSVEL(L,B,R,DL,DB,DR: REAL; VAR X,Y,Z,VX,VY,VZ:REAL);
    VAR  CL,SL,CB,SB: REAL;
    BEGIN
      CL:=CS(L); SL:=SN(L); CB:=CS(B); SB:=SN(B);
      X := R*CL*CB;  VX := DR*CL*CB-DL*R*SL*CB-DB*R*CL*SB;
      Y := R*SL*CB;  VY := DR*SL*CB+DL*R*CL*CB-DB*R*SL*SB;
      Z := R*SB;     VZ := DR*SB              +DB*R*CB;
    END;

  BEGIN

    DL:=0.0; DB:=0.0; DR:=0.0; DLS:=0.0; DBS:=0.0; DRS:=0.0;

    IF (IMODE>0) THEN

      BEGIN

        M := P2*FRAC(0.9931266+ 99.9973604*T);              (* Sun     *)
        DLS := 172.00+5.75*SIN(M);  DRS := 2.87*COS(M); DBS := 0.0;

        (* dl,db in 1e-4 rad/d, dr in 1e-4 AU/d *)
        CASE IPLAN OF
          0: BEGIN  DL:=0.0; DB:=0.0; DR:=0.0;  END;        (* Sun     *)
          1: BEGIN                                          (* Mercury *)
               M := P2*FRAC(0.4855407+415.2014314*T);
               DL := 714.00+292.66*COS(M)+71.96*COS(2*M)+18.16*COS(3*M)
                     +4.61*COS(4*M)+3.81*SIN(2*M)+2.43*SIN(3*M)
                     +1.08*SIN(4*M);
               DR := 55.94*SIN(M)+11.36*SIN(2*M)+2.60*SIN(3*M);
               DB := 73.40*COS(M)+29.82*COS(2*M)+10.22*COS(3*M)
                     +3.28*COS(4*M)-40.44*SIN(M)-16.55*SIN(2*M)
                     -5.56*SIN(3*M)-1.72*SIN(4*M);
             END;
          2: BEGIN                                          (* Venus   *)
               M := P2*FRAC(0.1400197+162.5494552*T);
               DL := 280.00+3.79*COS(M);   DR := 1.37*SIN(M);
               DB := 9.54*COS(M)-13.57*SIN(M);
             END;
          3: BEGIN  DL:=DLS; DR:=DRS; DB:=-DBS;  END;       (* Earth   *)
          4: BEGIN                                          (* Mars    *)
               M := P2*FRAC(0.0538553+53.1662736*T);
               DL := 91.50+17.07*COS(M)+2.03*COS(2*M);
               DR := 12.98*SIN(M)+1.21*COS(2*M);
               DB :=  0.83*COS(M)+2.80*SIN(M);
             END;
          5: BEGIN                                          (* Jupiter *)
               M := P2*FRAC(0.0565314+8.4302963*T);
               DL := 14.50+1.41*COS(M); DR:=3.66*SIN(M); DB:=0.33*SIN(M);
             END;
          6: BEGIN                                          (* Saturn  *)
               M := P2*FRAC(0.8829867+3.3947688*T);
               DL := 5.84+0.65*COS(M); DR:=3.09*SIN(M); DB:=0.24*COS(M);
             END;
          7: BEGIN                                          (* Uranus  *)
               M := P2*FRAC(0.3967117+1.1902849*T);
               DL := 2.05+0.19*COS(M); DR:=1.86*SIN(M); DB:=-0.03*SIN(M);
             END;
          8: BEGIN                                          (* Neptune *)
               M := P2*FRAC(0.7214906+0.6068526*T);
               DL := 1.04+0.02*COS(M); DR:=0.27*SIN(M); DB:=0.03*SIN(M);
             END;
          9: BEGIN                                          (* Pluto   *)
               M := P2*FRAC(0.0385795+0.4026667*T);
               DL := 0.69+0.34*COS(M)+0.12*COS(2*M)+0.05*COS(3*M);
               DR := 6.66*SIN(M)+1.64*SIN(2*M);
               DB := -0.08*COS(M)-0.17*SIN(M)-0.09*SIN(2*M);
             END;
        END;

      END;

    POSVEL (LS,BS,RS,DLS,DBS,DRS,XS,YS,ZS,VXS,VYS,VZS);
    POSVEL (LP,BP,RP,DL ,DB ,DR, XP,YP,ZP,VX ,VY ,VZ );
    X:=XP+XS;  Y:=YP+YS;  Z:=ZP+ZS;   DELTA0 := SQRT(X*X+Y*Y+Z*Z);
    IF IPLAN=3 THEN BEGIN X:=0.0; Y:=0.0; Z:=0.0; DELTA0:=0.0 END;

    FAC := 0.00578 * DELTA0 * 1E-4;
    CASE IMODE OF
      1: BEGIN X:=X-FAC*VX;  Y:=Y-FAC*VY;  Z:=Z-FAC*VZ; END;
      2: BEGIN X:=X-FAC*(VX+VXS); Y:=Y-FAC*(VY+VYS); Z:=Z-FAC*(VZ+VZS);END;
     END;

  END;
*/

//PROCEDURE GEOCEN(T, LP,BP,RP, LS,BS,RS: REAL; IPLAN,IMODE: INTEGER;
//                 VAR XP,YP,ZP, XS,YS,ZS, X,Y,Z,DELTA0: REAL);
void Castro::GEOCEN(double TT, double LP,double BP,double RP, 
					double LS,double BS,double RS,
					int IPLAN, int IMODE,
					double *XP,double *YP,double *ZP,
					double *XS,double *YS,double *ZS, 
					double *X,double *Y,double *Z,
					double  *DELTA0)
{
	double P2=6.283185307;
	double DL,DB,DR, DLS,DBS,DRS, FAC;
	double VX,VY,VZ, VXS,VYS,VZS, M;

	DL=0.0; 
	DB=0.0;
	DR=0.0; 
	DLS=0.0; 
	DBS=0.0; 
	DRS=0.0;

	if(IMODE>0)
	{
		M = P2*GC_FRAC(0.9931266+ 99.9973604*TT); //(* Sun     *)
        DLS = 172.00+5.75*sin(M);  
		DRS = 2.87*cos(M); 
		DBS = 0.0;  //(* dl,db in 1e-4 rad/d, dr in 1e-4 AU/d *)

		switch(IPLAN)
		{
			case 0://(* Sun     *)
			{
				DL=0.0; DB=0.0; DR=0.0; 
				break;
			}

			case 1://(* Mercury *)
			{
				M = P2*GC_FRAC(0.4855407+415.2014314*TT);
				DL = 714.00+292.66*cos(M)+71.96*cos(2*M)+18.16*cos(3*M)
                     +4.61*cos(4*M)+3.81*sin(2*M)+2.43*sin(3*M)
                     +1.08*sin(4*M);
				DR = 55.94*sin(M)+11.36*sin(2*M)+2.60*sin(3*M);
				DB = 73.40*cos(M)+29.82*cos(2*M)+10.22*cos(3*M)
                     +3.28*cos(4*M)-40.44*sin(M)-16.55*sin(2*M)
                     -5.56*sin(3*M)-1.72*sin(4*M);
				break;
			}

			case 2://venus
			{
				M = P2*GC_FRAC(0.1400197+162.5494552*TT);
				DL = 280.00+3.79*cos(M);   DR = 1.37*sin(M);
				DB = 9.54*cos(M)-13.57*sin(M);
				break;
			}

			case 3://earth
			{
				DL=DLS; DR=DRS; DB=-DBS;
				break;
			}

			case 4://mars
			{
				M = P2*GC_FRAC(0.0538553+53.1662736*TT);
				DL = 91.50+17.07*cos(M)+2.03*cos(2*M);
				DR = 12.98*sin(M)+1.21*sin(2*M);
				DB =  0.83*cos(M)+2.80*sin(M);
				break;
			}

			case 5://jup
			{
				M = P2*GC_FRAC(0.0565314+8.4302963*TT);
				DL = 14.50+1.41*cos(M); 
				DR=3.66*sin(M); 
				DB=0.33*sin(M);
				break;
			}

			case 6://sat
			{
				M = P2*GC_FRAC(0.8829867+3.3947688*TT);
				DL = 5.84+0.65*cos(M); 
				DR=3.09*sin(M); 
				DB=0.24*cos(M);
				break;
			}

			case 7://uran
			{
				M = P2*GC_FRAC(0.3967117+1.1902849*TT);
				DL = 2.05+0.19*cos(M); 
				DR=1.86*sin(M); 
				DB=-0.03*sin(M);
				break;
			}

			case 8://nept
			{
				M = P2*GC_FRAC(0.7214906+0.6068526*TT);
				DL = 1.04+0.02*cos(M); 
				DR=0.27*sin(M); 
				DB=0.03*sin(M);
				break;
			}

			case 9://pluto
			{
				M = P2*GC_FRAC(0.0385795+0.4026667*TT);
				DL = 0.69+0.34*cos(M)+0.12*cos(2*M)+0.05*cos(3*M);
				DR = 6.66*sin(M)+1.64*sin(2*M);
				DB = -0.08*cos(M)-0.17*sin(M)-0.09*sin(2*M);
				break;
			}
		}
	}

	//note,XS,YS,ZS pointers passed in, VXS.. local:
	GC_POSVEL (LS,BS,RS,DLS,DBS,DRS,XS,YS,ZS,&VXS,&VYS,&VZS);
    GC_POSVEL (LP,BP,RP,DL ,DB ,DR, XP,YP,ZP,&VX ,&VY ,&VZ );

	*X=*XP+*XS; 
	*Y=*YP+*YS;  
	*Z=*ZP+*ZS;   
	*DELTA0 = sqrt(*X * *X + *Y * *Y + *Z * *Z);

	if(IPLAN==3)
	{
		*X=0.0; 
		*Y=0.0; 
		*Z=0.0; 
		*DELTA0=0.0;
	}

	 FAC = 0.00578 * *DELTA0 * 1E-4;

	switch(IMODE)
		{
			case 1:
			{		
				*X = *X - FAC * VX;  
				*Y = *Y - FAC * VY;  
				*Z = *Z - FAC * VZ;	
				break;
			}
			case 2:
			{
				*X=*X-FAC*(VX+VXS); 
				*Y=*Y-FAC*(VY+VYS); 
				*Z=*Z-FAC*(VZ+VZS);
				break;
			}
		}


}

//  FUNCTION FRAC(X:REAL):REAL;
//    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1; FRAC:=X  END;
double Castro::GC_FRAC(double X)
{
	X = X - (int)X;
	if(X < 0) X = X + 1;

	return(X);
}

//  PROCEDURE POSVEL(L,B,R,DL,DB,DR: REAL; VAR X,Y,Z,VX,VY,VZ:REAL);
//    VAR  CL,SL,CB,SB: REAL;
//    BEGIN
//      CL:=CS(L); SL:=SN(L); CB:=CS(B); SB:=SN(B);
//      X := R*CL*CB;  VX := DR*CL*CB-DL*R*SL*CB-DB*R*CL*SB;
//      Y := R*SL*CB;  VY := DR*SL*CB+DL*R*CL*CB-DB*R*SL*SB;
//      Z := R*SB;     VZ := DR*SB              +DB*R*CB;
//    END;
//  BEGIN
void Castro::GC_POSVEL(double L,double B,double R,double DL,
					   double DB,double DR,
					   double *X,double *Y,double *Z,
					   double *VX,double *VY,double *VZ)
{
	double CL,SL,CB,SB;

	CL = CS(L); 
	SL = SN(L); 
	CB = CS(B); 
	SB = SN(B);
	*X = R * CL* CB;  
	*VX = DR * CL * CB - DL * R * SL* CB - DB *  R * CL *SB;
	*Y = R*SL*CB;  
	*VY = DR*SL*CB+DL*R*CL*CB-DB*R*SL*SB;
	*Z = R*SB;     
	*VZ = DR*SB+DB*R*CB;

}

//FRAC for planets
//////////////////////////////////////////////////////
//  FUNCTION FRAC(X:REAL):REAL;
//    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1.0; FRAC:=X  END;

double Castro::FRAC(double X)
{
	X = X - (int)X;
	if(X < 0) X = X + 1;

	return(X);
}

/////////////////////////////////////////////////////
//ADDTHE for planets
//  PROCEDURE ADDTHE(C1,S1,C2,S2:REAL; VAR C,S:REAL);
//    BEGIN  C:=C1*C2-S1*S2; S:=S1*C2+C1*S2; END;
/////////////////////////////////////////////////////
void Castro::ADDTHE(double C1,double S1,double C2,double S2,
					double *C,double *S)
{
	*C=C1*C2-S1*S2;
	*S=S1*C2+C1*S2;
}

//(*-----------------------------------------------------------------------*)
//(* SAT200: Saturn; ecliptic coordinates L,B,R (in deg and AU)            *)
//(*         equinox of date                                               *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE SAT200(T:REAL;VAR L,B,R:REAL);
void Castro::SAT200(double TT,
					double *L,double *B, double *R)
{
	int I;
	double P2=6.283185307;
	T = TT;//put in global to allow sub access

	DL=0.0; DR=0.0; DB=0.0;
    M5=P2*FRAC(0.0565314+8.4302963*T); 
	M6=P2*FRAC(0.8829867+3.3947688*T);
    M7=P2*FRAC(0.3969537+1.1902586*T); 
	M8=P2*FRAC(0.7208473+0.6068623*T);
    C6[0]=1.0; 
	S6[0]=0.0;  
	C6[1]=cos(M6); 
	S6[1]=sin(M6);
   
	for(I=2;I<=11;I++)
	{
		ADDTHE(C6[I-1],S6[I-1],C6[1],S6[1],&C6[I],&S6[I]);
	}

	S_PERTJUP(); 
	S_PERTURA(); 
	S_PERTNEP(); 
	S_PERTJUR();

    *L= 360.0*FRAC(0.2561136 + M6/P2 + ((5018.6+T*1.9)*T +DL)/1296.0E3 );
    *R= 9.557584 - 0.000186*T  +  DR*1.0E-5;
    *B= ( 175.1 - 10.2*T + DB ) / 3600.0;
}


/////////////////////////////////////////////////////
//S_TERM, for jupitor
//  PROCEDURE TERM(I6,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
//    BEGIN
//      IF IT=0 THEN ADDTHE(C6[I6],S6[I6],C[I],S[I],U,V)
//              ELSE BEGIN U:=U*T; V:=V*T END;
//      DL:=DL+DLC*U+DLS*V; DR:=DR+DRC*U+DRS*V; DB:=DB+DBC*U+DBS*V;
//    END;
void Castro::S_TERM(int I6,int I,int IT,
					double DLC,double DLS,double DRC,double DRS,
					double DBC,double DBS)
{
	if(IT == 0)
	{
		ADDTHE(C6[I6],S6[I6],C[I +6],S[I +6],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T;
	}

	DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;

}

/////////////////////////////////////////////////////
// S_PERTJUP, for sat
//  PROCEDURE PERTJUP;  (* Kepler terms and perturbations by Jupiter *)
/////////////////////////////////////////////////////
void Castro::S_PERTJUP()
{
	int I;

	C[0 +6]=1.0; 
	S[0 +6]=0.0; 
	C[1 +6]=cos(M5); 
	S[1 +6]=sin(M5);

	for(I=0;I>=-5;I--)
	{
		ADDTHE(C[I +6],S[I +6],C[1 +6],-S[1 +6],&C[I-1 +6],&S[I-1 +6]);
	}

	S_TERM( 0,-1,0,   12.0,   -1.4,   -13.9,    6.4,    1.2,  -1.8);
    S_TERM( 0,-2,0,    0.0,   -0.2,    -0.9,    1.0,    0.0,  -0.1);
    S_TERM( 1, 1,0,    0.9,    0.4,    -1.8,    1.9,    0.2,   0.2);
    S_TERM( 1, 0,0, -348.3,22907.7,-52915.5, -752.2,-3266.5,8314.4);
    S_TERM( 1, 0,1, -225.2, -146.2,   337.7, -521.3,   79.6,  17.4);
    S_TERM( 1, 0,2,    1.3,   -1.4,     3.2,    2.9,    0.1,  -0.4);
    S_TERM( 1,-1,0,   -1.0,  -30.7,   108.6, -815.0,   -3.6,  -9.3);
    S_TERM( 1,-2,0,   -2.0,   -2.7,    -2.1,  -11.9,   -0.1,  -0.4);
    S_TERM( 2, 1,0,    0.1,    0.2,    -1.0,    0.3,    0.0,   0.0);
    S_TERM( 2, 0,0,   44.2,  724.0, -1464.3,  -34.7, -188.7, 459.1);
    S_TERM( 2, 0,1,  -17.0,  -11.3,    18.9,  -28.6,    1.0,  -3.7);
    S_TERM( 2,-1,0,   -3.5, -426.6,  -546.5,  -26.5,   -1.6,  -2.7);
    S_TERM( 2,-1,1,    3.5,   -2.2,    -2.6,   -4.3,    0.0,   0.0);
    S_TERM( 2,-2,0,   10.5,  -30.9,  -130.5,  -52.3,   -1.9,   0.2);
    S_TERM( 2,-3,0,   -0.2,   -0.4,    -1.2,   -0.1,   -0.1,   0.0);
    S_TERM( 3, 0,0,    6.5,   30.5,   -61.1,    0.4,  -11.6,  28.1);
    S_TERM( 3, 0,1,   -1.2,   -0.7,     1.1,   -1.8,   -0.2,  -0.6);
    S_TERM( 3,-1,0,   29.0,  -40.2,    98.2,   45.3,    3.2,  -9.4);
    S_TERM( 3,-1,1,    0.6,    0.6,    -1.0,    1.3,    0.0,   0.0);
    S_TERM( 3,-2,0,  -27.0,  -21.1,   -68.5,    8.1,  -19.8,   5.4);
    S_TERM( 3,-2,1,    0.9,   -0.5,    -0.4,   -2.0,   -0.1,  -0.8);
    S_TERM( 3,-3,0,   -5.4,   -4.1,   -19.1,   26.2,   -0.1,  -0.1);
    S_TERM( 4, 0,0,    0.6,    1.4,    -3.0,   -0.2,   -0.6,   1.6);
    S_TERM( 4,-1,0,    1.5,   -2.5,    12.4,    4.7,    1.0,  -1.1);
    S_TERM( 4,-2,0, -821.9,   -9.6,   -26.0, 1873.6,  -70.5,  -4.4);
    S_TERM( 4,-2,1,    4.1,  -21.9,   -50.3,   -9.9,    0.7,  -3.0);
    S_TERM( 4,-3,0,   -2.0,   -4.7,   -19.3,    8.2,   -0.1,  -0.3);
    S_TERM( 4,-4,0,   -1.5,    1.3,     6.5,    7.3,    0.0,   0.0);
    S_TERM( 5,-2,0,-2627.6,-1277.3,   117.4, -344.1,  -13.8,  -4.3);
    S_TERM( 5,-2,1,   63.0,  -98.6,    12.7,    6.7,    0.1,  -0.2);
    S_TERM( 5,-2,2,    1.7,    1.2,    -0.2,    0.3,    0.0,   0.0);
    S_TERM( 5,-3,0,    0.4,   -3.6,   -11.3,   -1.6,    0.0,  -0.3);
    S_TERM( 5,-4,0,   -1.4,    0.3,     1.5,    6.3,   -0.1,   0.0);
    S_TERM( 5,-5,0,    0.3,    0.6,     3.0,   -1.7,    0.0,   0.0);
    S_TERM( 6,-2,0, -146.7,  -73.7,   166.4, -334.3,  -43.6, -46.7);
    S_TERM( 6,-2,1,    5.2,   -6.8,    15.1,   11.4,    1.7,  -1.0);
    S_TERM( 6,-3,0,    1.5,   -2.9,    -2.2,   -1.3,    0.1,  -0.1);
    S_TERM( 6,-4,0,   -0.7,   -0.2,    -0.7,    2.8,    0.0,   0.0);
    S_TERM( 6,-5,0,    0.0,    0.5,     2.5,   -0.1,    0.0,   0.0);
    S_TERM( 6,-6,0,    0.3,   -0.1,    -0.3,   -1.2,    0.0,   0.0);
    S_TERM( 7,-2,0,   -9.6,   -3.9,     9.6,  -18.6,   -4.7,  -5.3);
    S_TERM( 7,-2,1,    0.4,   -0.5,     1.0,    0.9,    0.3,  -0.1);
    S_TERM( 7,-3,0,    3.0,    5.3,     7.5,   -3.5,    0.0,   0.0);
    S_TERM( 7,-4,0,    0.2,    0.4,     1.6,   -1.3,    0.0,   0.0);
    S_TERM( 7,-5,0,   -0.1,    0.2,     1.0,    0.5,    0.0,   0.0);
    S_TERM( 7,-6,0,    0.2,    0.0,     0.2,   -1.0,    0.0,   0.0);
    S_TERM( 8,-2,0,   -0.7,   -0.2,     0.6,   -1.2,   -0.4,  -0.4);
    S_TERM( 8,-3,0,    0.5,    1.0,    -2.0,    1.5,    0.1,   0.2);
    S_TERM( 8,-4,0,    0.4,    1.3,     3.6,   -0.9,    0.0,  -0.1);
    S_TERM( 9,-4,0,    4.0,   -8.7,   -19.9,   -9.9,    0.2,  -0.4);
    S_TERM( 9,-4,1,    0.5,    0.3,     0.8,   -1.8,    0.0,   0.0);
    S_TERM(10,-4,0,   21.3,  -16.8,     3.3,    3.3,    0.2,  -0.2);
    S_TERM(10,-4,1,    1.0,    1.7,    -0.4,    0.4,    0.0,   0.0);
    S_TERM(11,-4,0,    1.6,   -1.3,     3.0,    3.7,    0.8,  -0.2);

}

/////////////////////////////////////////////////////
// S_PERTURA, for sat
//  PROCEDURE PERTURA;  (* perturbations by Uranus *)
/////////////////////////////////////////////////////
void Castro::S_PERTURA()
{
	int I;

	C[-1 +6]=cos(M7); 
	S[-1 +6]=-sin(M7);

	for(I=-1;I>=-4;I--)
	{
		ADDTHE(C[I +6],S[I +6],C[-1 +6],S[-1 +6],&C[I-1 +6],&S[I-1 +6]);
	}

	S_TERM( 0,-1,0,    1.0,    0.7,     0.4,   -1.5,    0.1,   0.0);
    S_TERM( 0,-2,0,    0.0,   -0.4,    -1.1,    0.1,   -0.1,  -0.1);
    S_TERM( 0,-3,0,   -0.9,   -1.2,    -2.7,    2.1,   -0.5,  -0.3);
    S_TERM( 1,-1,0,    7.8,   -1.5,     2.3,   12.7,    0.0,   0.0);
    S_TERM( 1,-2,0,   -1.1,   -8.1,     5.2,   -0.3,   -0.3,  -0.3);
    S_TERM( 1,-3,0,  -16.4,  -21.0,    -2.1,    0.0,    0.4,   0.0);
    S_TERM( 2,-1,0,    0.6,   -0.1,     0.1,    1.2,    0.1,   0.0);
    S_TERM( 2,-2,0,   -4.9,  -11.7,    31.5,  -13.3,    0.0,  -0.2);
    S_TERM( 2,-3,0,   19.1,   10.0,   -22.1,   42.1,    0.1,  -1.1);
    S_TERM( 2,-4,0,    0.9,   -0.1,     0.1,    1.4,    0.0,   0.0);
    S_TERM( 3,-2,0,   -0.4,   -0.9,     1.7,   -0.8,    0.0,  -0.3);
    S_TERM( 3,-3,0,    2.3,    0.0,     1.0,    5.7,    0.3,   0.3);
    S_TERM( 3,-4,0,    0.3,   -0.7,     2.0,    0.7,    0.0,   0.0);
    S_TERM( 3,-5,0,   -0.1,   -0.4,     1.1,   -0.3,    0.0,   0.0);
}

/////////////////////////////////////////////////////
// S_PERTNEP, for sat
//  PROCEDURE PERTNEP;  (* perturbations by Neptune *)
/////////////////////////////////////////////////////
void Castro::S_PERTNEP()
{

	C[-1 +6]=cos(M8); 
	S[-1 +6]=-sin(M8);

	ADDTHE(C[-1 +6],S[-1 +6],C[-1 +6],S[-1 +6],&C[-2 +6],&S[-2 +6]);
	S_TERM( 1,-1,0,   -1.3,   -1.2,     2.3,   -2.5,    0.0,   0.0);
    S_TERM( 1,-2,0,    1.0,   -0.1,     0.1,    1.4,    0.0,   0.0);
    S_TERM( 2,-2,0,    1.1,   -0.1,     0.2,    3.3,    0.0,   0.0);
}

/////////////////////////////////////////////////////
// S_PERTNEP, for sat
//  PROCEDURE PERTJUR;  (* perturbations by Jupiter and Uranus *)
/////////////////////////////////////////////////////
void Castro::S_PERTJUR()
{
	double PHI,X,Y;

	 PHI=(-2*M5+5*M6-3*M7); X=cos(PHI); Y=sin(PHI);
     DL=DL-0.8*X-0.1*Y; DR=DR-0.2*X+1.8*Y; DB=DB+0.3*X+0.5*Y;
     ADDTHE(X,Y,C6[1],S6[1],&X,&Y);
     DL=DL+(+2.4-0.7*T)*X+(27.8-0.4*T)*Y; DR=DR+2.1*X-0.2*Y;
     ADDTHE(X,Y,C6[1],S6[1],&X,&Y);
     DL=DL+0.1*X+1.6*Y; DR=DR-3.6*X+0.3*Y; DB=DB-0.2*X+0.6*Y;	
}

///////////////////////////////////////////////////////////
//SUN200
///////////////////////////////////////////////////////////
//PROCEDURE SUN200(T:REAL;VAR L,B,R:REAL);
void Castro::SUN200(double TT, double *L, double *B,double *R)
{
	int I;
	double P2=6.283185307;
	T=TT;

    DL=0.0; DR=0.0; DB=0.0;
    M2=P2*FRAC(0.1387306+162.5485917*T);
    M3=P2*FRAC(0.9931266+99.9973604*T);
    M4=P2*FRAC(0.0543250+ 53.1666028*T);
    M5=P2*FRAC(0.0551750+ 8.4293972*T);
    M6=P2*FRAC(0.8816500+  3.3938722*T); 
	D =P2*FRAC(0.8274+1236.8531*T);
    A =P2*FRAC(0.3749+1325.5524*T);      
	UU=P2*FRAC(0.2591+1342.2278*T);
    C3[0 +1]=1.0;     S3[0 +1]=0.0;
    C3[1 +1]=cos(M3); S3[1 +1]=sin(M3);  
	C3[-1 +1]=C3[1 +1]; S3[-1 +1]=-S3[1 +1];

	for(I=2;I<=7;I++)
	{
		ADDTHE(C3[I-1 +1],S3[I-1 +1],C3[1 +1],S3[1 +1],&C3[I +1],&S3[I +1]);
	}

    SN_PERTVEN(); 
	SN_PERTMAR(); 
	SN_PERTJUP(); 
	SN_PERTSAT(); 
	SN_PERTMOO();

    DL=DL + 6.40*sin(P2*(0.6983+0.0561*T))+1.87*sin(P2*(0.5764+0.4174*T))
           + 0.27*sin(P2*(0.4189+0.3306*T))+0.20*sin(P2*(0.3581+2.4814*T));
    *L= 360.0*FRAC(0.7859453 + M3/P2 + ((6191.2+1.1*T)*T+DL)/1296.0E3 );
    *R= 1.0001398 - 0.0000007*T  +  DR*1E-6;
    *B= DB/3600.0;

}


/////////////////////////////////////////////////////
//SN_TERM, for sun
//  PROCEDURE TERM(I6,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
//    BEGIN
//      IF IT=0 THEN ADDTHE(C6[I6],S6[I6],C[I],S[I],U,V)
//              ELSE BEGIN U:=U*T; V:=V*T END;
//      DL:=DL+DLC*U+DLS*V; DR:=DR+DRC*U+DRS*V; DB:=DB+DBC*U+DBS*V;
//    END;
void Castro::SN_TERM(int I1,int II,int IT,
					double DLC,double DLS,double DRC,double DRS,
					double DBC,double DBS)
{
	if(IT == 0)
	{
		ADDTHE(C3[I1 +1],S3[I1 +1],C[II +8],S[II +8],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T;
	}

	DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;

}

//////////////////////////////////////////////////////////////////
// S_PERTNEP, for sat
//  PROCEDURE PERTJUR;  (* perturbations by Jupiter and Uranus *)
//////////////////////////////////////////////////////////////////
//  PROCEDURE PERTVEN;  (* Keplerian terms and perturbations by Venus *)
void Castro::SN_PERTVEN()
{
    int I;

	C[0 +8]=1.0; 
	S[0 +8]=0.0; 
	C[-1 +8]=cos(M2); 
	S[-1 +8]=-cos(M2);

	for(I=-1;I>= -5;I--)
	{
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
    }  
	SN_TERM(1, 0,0,-0.22,6892.76,-16707.37, -0.54, 0.00, 0.00);
    SN_TERM(1, 0,1,-0.06, -17.35,    42.04, -0.15, 0.00, 0.00);
    SN_TERM(1, 0,2,-0.01,  -0.05,     0.13, -0.02, 0.00, 0.00);
    SN_TERM(2, 0,0, 0.00,  71.98,  -139.57,  0.00, 0.00, 0.00);
    SN_TERM(2, 0,1, 0.00,  -0.36,     0.70,  0.00, 0.00, 0.00);
    SN_TERM(3, 0,0, 0.00,   1.04,    -1.75,  0.00, 0.00, 0.00);
    SN_TERM(0,-1,0, 0.03,  -0.07,    -0.16, -0.07, 0.02,-0.02);
    SN_TERM(1,-1,0, 2.35,  -4.23,    -4.75, -2.64, 0.00, 0.00);
    SN_TERM(1,-2,0,-0.10,   0.06,     0.12,  0.20, 0.02, 0.00);
    SN_TERM(2,-1,0,-0.06,  -0.03,     0.20, -0.01, 0.01,-0.09);
    SN_TERM(2,-2,0,-4.70,   2.90,     8.28, 13.42, 0.01,-0.01);
    SN_TERM(3,-2,0, 1.80,  -1.74,    -1.44, -1.57, 0.04,-0.06);
    SN_TERM(3,-3,0,-0.67,   0.03,     0.11,  2.43, 0.01, 0.00);
    SN_TERM(4,-2,0, 0.03,  -0.03,     0.10,  0.09, 0.01,-0.01);
    SN_TERM(4,-3,0, 1.51,  -0.40,    -0.88, -3.36, 0.18,-0.10);
    SN_TERM(4,-4,0,-0.19,  -0.09,    -0.38,  0.77, 0.00, 0.00);
    SN_TERM(5,-3,0, 0.76,  -0.68,     0.30,  0.37, 0.01, 0.00);
    SN_TERM(5,-4,0,-0.14,  -0.04,    -0.11,  0.43,-0.03, 0.00);
    SN_TERM(5,-5,0,-0.05,  -0.07,    -0.31,  0.21, 0.00, 0.00);
    SN_TERM(6,-4,0, 0.15,  -0.04,    -0.06, -0.21, 0.01, 0.00);
    SN_TERM(6,-5,0,-0.03,  -0.03,    -0.09,  0.09,-0.01, 0.00);
    SN_TERM(6,-6,0, 0.00,  -0.04,    -0.18,  0.02, 0.00, 0.00);
}

//////////////////////////////////////////////////////////////////
// SN_PERTMAR()
//  PROCEDURE PERTMAR;  (* perturbations by Jupiter and Uranus *)
//////////////////////////////////////////////////////////////////
//  PROCEDURE PERTMAR;  (* perturbations by Mars *)
void Castro::SN_PERTMAR()
{
    int I;
 
    C[-1 +8]=cos(M4); 
	S[-1 +8]=-sin(M4);

	for (I=-1;I>=-7;I--)
	{
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
    }
	
	SN_TERM(1,-1,0,-0.22,   0.17,    -0.21, -0.27, 0.00, 0.00);
    SN_TERM(1,-2,0,-1.66,   0.62,     0.16,  0.28, 0.00, 0.00);
    SN_TERM(2,-2,0, 1.96,   0.57,    -1.32,  4.55, 0.00, 0.01);
    SN_TERM(2,-3,0, 0.40,   0.15,    -0.17,  0.46, 0.00, 0.00);
    SN_TERM(2,-4,0, 0.53,   0.26,     0.09, -0.22, 0.00, 0.00);
    SN_TERM(3,-3,0, 0.05,   0.12,    -0.35,  0.15, 0.00, 0.00);
    SN_TERM(3,-4,0,-0.13,  -0.48,     1.06, -0.29, 0.01, 0.00);
    SN_TERM(3,-5,0,-0.04,  -0.20,     0.20, -0.04, 0.00, 0.00);
    SN_TERM(4,-4,0, 0.00,  -0.03,     0.10,  0.04, 0.00, 0.00);
    SN_TERM(4,-5,0, 0.05,  -0.07,     0.20,  0.14, 0.00, 0.00);
    SN_TERM(4,-6,0,-0.10,   0.11,    -0.23, -0.22, 0.00, 0.00);
    SN_TERM(5,-7,0,-0.05,   0.00,     0.01, -0.14, 0.00, 0.00);
    SN_TERM(5,-8,0, 0.05,   0.01,    -0.02,  0.10, 0.00, 0.00);
}

//////////////////////////////////////////////////////////////////
// SN_PERTJUP()
//  PROCEDURE PERTJUP;  (* perturbations by Jupiter and Uranus *)
//////////////////////////////////////////////////////////////////
//  PROCEDURE PERTJUP;  (* perturbations by Jupiter *)
void Castro::SN_PERTJUP()
{ 
    int I;
  
    C[-1 +8]=cos(M5); 
	S[-1 +8]=-sin(M5);

	for(I=-1;I>=-3;I--)
	{
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	}
    SN_TERM(-1,-1,0,0.01,   0.07,     0.18, -0.02, 0.00,-0.02);
    SN_TERM(0,-1,0,-0.31,   2.58,     0.52,  0.34, 0.02, 0.00);
    SN_TERM(1,-1,0,-7.21,  -0.06,     0.13,-16.27, 0.00,-0.02);
    SN_TERM(1,-2,0,-0.54,  -1.52,     3.09, -1.12, 0.01,-0.17);
    SN_TERM(1,-3,0,-0.03,  -0.21,     0.38, -0.06, 0.00,-0.02);
    SN_TERM(2,-1,0,-0.16,   0.05,    -0.18, -0.31, 0.01, 0.00);
    SN_TERM(2,-2,0, 0.14,  -2.73,     9.23,  0.48, 0.00, 0.00);
    SN_TERM(2,-3,0, 0.07,  -0.55,     1.83,  0.25, 0.01, 0.00);
    SN_TERM(2,-4,0, 0.02,  -0.08,     0.25,  0.06, 0.00, 0.00);
    SN_TERM(3,-2,0, 0.01,  -0.07,     0.16,  0.04, 0.00, 0.00);
    SN_TERM(3,-3,0,-0.16,  -0.03,     0.08, -0.64, 0.00, 0.00);
    SN_TERM(3,-4,0,-0.04,  -0.01,     0.03, -0.17, 0.00, 0.00);

}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
void Castro::SN_PERTSAT()
{
    C[-1 +8]=cos(M6); 
	S[-1 +8]=-sin(M6);

    ADDTHE(C[-1 +8],S[-1 +8],C[-1 +8],S[-1 +8],&C[-2 +8],&S[-2 +8]);
    SN_TERM(0,-1,0, 0.00,   0.32,     0.01,  0.00, 0.00, 0.00);
    SN_TERM(1,-1,0,-0.08,  -0.41,     0.97, -0.18, 0.00,-0.01);
    SN_TERM(1,-2,0, 0.04,   0.10,    -0.23,  0.10, 0.00, 0.00);
    SN_TERM(2,-2,0, 0.04,   0.10,    -0.35,  0.13, 0.00, 0.00);
}

//  PROCEDURE PERTMOO;  (* difference between the Earth-Moon      *)
//    BEGIN             (* barycenter and the center of the Earth *)
void Castro::SN_PERTMOO()
{
      DL = DL +  6.45*sin(D) - 0.42*sin(D-A) + 0.18*sin(D+A)
                              + 0.17*sin(D-M3) - 0.06*sin(D+M3);
      DR = DR + 30.76*cos(D) - 3.06*cos(D-A)+ 0.85*cos(D+A)
                              - 0.58*cos(D+M3) + 0.57*cos(D-M3);
      DB = DB + 0.576*sin(UU);
}

//(*-----------------------------------------------------------------------*)
//(* MER200: Mercury; ecliptic coordinates L,B,R (in deg and AU)           *)
//(*         equinox of date                                               *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE MER200(T:REAL;VAR L,B,R:REAL);
void Castro::MER200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;

	T = TT;
 
    DL=0.0; DR=0.0; DB=0.0;
    M1=P2*FRAC(0.4855407+415.2014314*T);
    M2=P2*FRAC(0.1394222+162.5490444*T);
    M3=P2*FRAC(0.9937861+ 99.9978139*T);
    M5=P2*FRAC(0.0558417+  8.4298417*T);
    M6=P2*FRAC(0.8823333+  3.3943333*T);
    C1[0 +1]=1.0;     
	S1[0 +1]=0.0;
    C1[1 +1]=cos(M1); 
	S1[1 +1]=sin(M1);  
	C1[-1 +1]=C1[1 +1]; 
	S1[-1 +1]=-S1[1 +1];

	for(I=2;I<=9;I++)
	{
		ADDTHE(C1[I-1 +1],S1[I-1 +1],C1[1 +1],S1[1 +1],&C1[I +1],&S1[I +1]);
	}

    M_PERTVEN(); 
	M_PERTEAR(); 
	M_PERTJUP(); 
	M_PERTSAT();

    DL = DL + (2.8+3.2*T);
    *L= 360.0*FRAC(0.2151379 + M1/P2 + ((5601.7+1.1*T)*T+DL)/1296.0E3 );
    *R= 0.3952829 + 0.0000016*T  +  DR*1.0E-6;
    *B= ( -2522.15 + (-30.18 + 0.04*T) * T  +  DB ) / 3600.0;

}

//  PROCEDURE TERM(I1,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::M_TERM(int I1, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS)
{
	if(IT==0)
	{
		ADDTHE(C1[I1 +1],S1[I1 +1],C[I +5],S[I +5],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T;
	}

	DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;
}

//  PROCEDURE PERTVEN;  (* Kepler terms and perturbations by Venus *)
void Castro::M_PERTVEN()
{
    int I;

    C[0 +5]=1.0; 
	S[0 +5]=0.0;  
	C[-1 +5]=cos(M2); 
	S[-1 +5]=-sin(M2);

	for(I=-1;I>=-4;I--)
	{
		ADDTHE(C[I +5],S[I +5],C[-1 +5],S[-1 +5],&C[I-1 +5],&S[I-1 +5]);
	}

      M_TERM( 1, 0,0, 259.74,84547.39,-78342.34, 0.01,11683.22,21203.79);
      M_TERM( 1, 0,1,   2.30,    5.04,    -7.52, 0.02,  138.55,  -71.01);
      M_TERM( 1, 0,2,   0.01,   -0.01,     0.01, 0.01,   -0.19,   -0.54);
      M_TERM( 2, 0,0,-549.71,10394.44, -7955.45, 0.00, 2390.29, 4306.79);
      M_TERM( 2, 0,1,  -4.77,    8.97,    -1.53, 0.00,   28.49,  -14.18);
      M_TERM( 2, 0,2,   0.00,    0.00,     0.00, 0.00,   -0.04,   -0.11);
      M_TERM( 3, 0,0,-234.04, 1748.74, -1212.86, 0.00,  535.41,  984.33);
      M_TERM( 3, 0,1,  -2.03,    3.48,    -0.35, 0.00,    6.56,   -2.91);
      M_TERM( 4, 0,0, -77.64,  332.63,  -219.23, 0.00,  124.40,  237.03);
      M_TERM( 4, 0,1,  -0.70,    1.10,    -0.08, 0.00,    1.59,   -0.59);
      M_TERM( 5, 0,0, -23.59,   67.28,   -43.54, 0.00,   29.44,   58.77);
      M_TERM( 5, 0,1,  -0.23,    0.32,    -0.02, 0.00,    0.39,   -0.11);
      M_TERM( 6, 0,0,  -6.86,   14.06,    -9.18, 0.00,    7.03,   14.84);
      M_TERM( 6, 0,1,  -0.07,    0.09,    -0.01, 0.00,    0.10,   -0.02);
      M_TERM( 7, 0,0,  -1.94,    2.98,    -2.02, 0.00,    1.69,    3.80);
      M_TERM( 8, 0,0,  -0.54,    0.63,    -0.46, 0.00,    0.41,    0.98);
      M_TERM( 9, 0,0,  -0.15,    0.13,    -0.11, 0.00,    0.10,    0.25);
      M_TERM(-1,-2,0,  -0.17,   -0.06,    -0.05, 0.14,   -0.06,   -0.07);
      M_TERM( 0,-1,0,   0.24,   -0.16,    -0.11,-0.16,    0.04,   -0.01);
      M_TERM( 0,-2,0,  -0.68,   -0.25,    -0.26, 0.73,   -0.16,   -0.18);
      M_TERM( 0,-5,0,   0.37,    0.08,     0.06,-0.28,    0.13,    0.12);
      M_TERM( 1,-1,0,   0.58,   -0.41,     0.26, 0.36,    0.01,   -0.01);
      M_TERM( 1,-2,0,  -3.51,   -1.23,     0.23,-0.63,   -0.05,   -0.06);
      M_TERM( 1,-3,0,   0.08,    0.53,    -0.11, 0.04,    0.02,   -0.09);
      M_TERM( 1,-5,0,   1.44,    0.31,     0.30,-1.39,    0.34,    0.29);
      M_TERM( 2,-1,0,   0.15,   -0.11,     0.09, 0.12,    0.02,   -0.04);
      M_TERM( 2,-2,0,  -1.99,   -0.68,     0.65,-1.91,   -0.20,    0.03);
      M_TERM( 2,-3,0,  -0.34,   -1.28,     0.97,-0.26,    0.03,    0.03);
      M_TERM( 2,-4,0,  -0.33,    0.35,    -0.13,-0.13,   -0.01,    0.00);
      M_TERM( 2,-5,0,   7.19,    1.56,    -0.05, 0.12,    0.06,    0.05);
      M_TERM( 3,-2,0,  -0.52,   -0.18,     0.13,-0.39,   -0.16,    0.03);
      M_TERM( 3,-3,0,  -0.11,   -0.42,     0.36,-0.10,   -0.05,   -0.05);
      M_TERM( 3,-4,0,  -0.19,    0.22,    -0.23,-0.20,   -0.01,    0.02);
      M_TERM( 3,-5,0,   2.77,    0.49,    -0.45, 2.56,    0.40,   -0.12);
      M_TERM( 4,-5,0,   0.67,    0.12,    -0.09, 0.47,    0.24,   -0.08);
      M_TERM( 5,-5,0,   0.18,    0.03,    -0.02, 0.12,    0.09,   -0.03);
}

//  PROCEDURE PERTEAR;  (* perturbations by the Earth *)
void Castro::M_PERTEAR()
{
    int I;

    C[-1 +5]=cos(M3); 
	S[-1 +5]=-sin(M3);

	for(I=-1;I>=-3;I--)
	{
		ADDTHE(C[I +5],S[I +5],C[-1 +5],S[-1 +5],&C[I-1 +5],&S[I-1 +5]);
	}

      M_TERM( 0,-4,0,  -0.11,   -0.07,    -0.08, 0.11,   -0.02,   -0.04);
      M_TERM( 1,-1,0,   0.10,   -0.20,     0.15, 0.07,    0.00,    0.00);
      M_TERM( 1,-2,0,  -0.35,    0.28,    -0.13,-0.17,   -0.01,    0.00);
      M_TERM( 1,-4,0,  -0.67,   -0.45,     0.00, 0.01,   -0.01,   -0.01);
      M_TERM( 2,-2,0,  -0.20,    0.16,    -0.16,-0.20,   -0.01,    0.02);
      M_TERM( 2,-3,0,   0.13,   -0.02,     0.02, 0.14,    0.01,    0.00);
      M_TERM( 2,-4,0,  -0.33,   -0.18,     0.17,-0.31,   -0.04,    0.00);

}

//PROCEDURE PERTJUP;  (* perturbations by Jupiter *)
void Castro::M_PERTJUP()
{
    int I;

    C[-1 +5]=cos(M5); 
	S[-1 +5]=-sin(M5);

	for(I=-1;I>=-2;I--)
	{
		ADDTHE(C[I +5],S[I +5],C[-1 +5],S[-1 +5],&C[I-1 +5],&S[I-1 +5]);
	}

      M_TERM(-1,-1,0,  -0.08,    0.16,     0.15, 0.08,   -0.04,    0.01);
      M_TERM(-1,-2,0,   0.10,   -0.06,    -0.07,-0.12,    0.07,   -0.01);
      M_TERM( 0,-1,0,  -0.31,    0.48,    -0.02, 0.13,   -0.03,   -0.02);
      M_TERM( 0,-2,0,   0.42,   -0.26,    -0.38,-0.50,    0.20,   -0.03);
      M_TERM( 1,-1,0,  -0.70,    0.01,    -0.02,-0.63,    0.00,    0.03);
      M_TERM( 1,-2,0,   2.61,   -1.97,     1.74, 2.32,    0.01,    0.01);
      M_TERM( 1,-3,0,   0.32,   -0.15,     0.13, 0.28,    0.00,    0.00);
      M_TERM( 2,-1,0,  -0.18,    0.01,     0.00,-0.13,   -0.03,    0.03);
      M_TERM( 2,-2,0,   0.75,   -0.56,     0.45, 0.60,    0.08,   -0.17);
      M_TERM( 3,-2,0,   0.20,   -0.15,     0.10, 0.14,    0.04,   -0.08);
}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
void Castro::M_PERTSAT()
{
      C[-2 +5]=cos(2*M6); 
	  S[-2 +5]=-sin(2*M6);

      M_TERM( 1,-2,0,  -0.19,    0.33,     0.00, 0.00,    0.00,    0.00);

}


//(*-----------------------------------------------------------------------*)
//(* VEN200: Venus; ecliptic coordinates L,B,R (in deg and AU)             *)
//(*         equinox of date                                               *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE VEN200(T:REAL;VAR L,B,R:REAL);
void Castro::VEN200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;

	T = TT;

    DL=0.0; DR=0.0; DB=0.0;
    M1=P2*FRAC(0.4861431+415.2018375*T);
    M2=P2*FRAC(0.1400197+162.5494552*T);
    M3=P2*FRAC(0.9944153+ 99.9982208*T);
    M4=P2*FRAC(0.0556297+ 53.1674631*T);
    M5=P2*FRAC(0.0567028+  8.4305083*T);
    M6=P2*FRAC(0.8830539+  3.3947206*T);
    C2[0]=1.0; S2[0]=0.0; C2[1]=cos(M2); S2[1]=sin(M2);

	for(I=2;I<=8;I++)
	{
		ADDTHE(C2[I-1],S2[I-1],C2[1],S2[1],&C2[I],&S2[I]);
	}

    V_PERTMER(); 
	V_PERTEAR(); 
	V_PERTMAR(); 
	V_PERTJUP(); 
	V_PERTSAT();

    DL=DL + 2.74*sin(P2*(0.0764+0.4174*T))
           + 0.27*sin(P2*(0.9201+0.3307*T));
    DL=DL + (1.9+1.8*T);
    *L= 360.0*FRAC(0.3654783 + M2/P2 + ((5071.2+1.1*T)*T+DL)/1296.0E3 );
    *R= 0.7233482 - 0.0000002*T  +  DR*1.0E-6;
    *B= ( -67.70 + ( 0.04 + 0.01*T) * T  +  DB ) / 3600.0;
}

//PROCEDURE TERM(I1,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::V_TERM(int I1, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS)
{
	if(IT==0)
	{
		ADDTHE(C2[I1],S2[I1],C[I +8],S[I +8],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T;
	}

    DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;

}

//  PROCEDURE PERTMER;  (* perturbations by Mercury *)
void Castro::V_PERTMER()
{

      C[0 +8]=1.0; 
	  S[0 +8]=0.0; 
	  C[-1 +8]=cos(M1); 
	  S[-1 +8]=-sin(M1);

      ADDTHE(C[-1 +8],S[-1 +8],C[-1 +8],S[-1 +8],&C[-2 +8],&S[-2 +8]);

      V_TERM(1,-1,0,   0.00,   0.00,    0.06, -0.09,   0.01,   0.00);
      V_TERM(2,-1,0,   0.25,  -0.09,   -0.09, -0.27,   0.00,   0.00);
      V_TERM(4,-2,0,  -0.07,  -0.08,   -0.14,  0.14,  -0.01,  -0.01);
      V_TERM(5,-2,0,  -0.35,   0.08,    0.02,  0.09,   0.00,   0.00);

}

//PROCEDURE PERTEAR;  (* Kepler terms and perturbations by the Earth *)
void Castro::V_PERTEAR()
{
    int I;

	
      C[-1 +8]=cos(M3); S[-1 +8]=-sin(M3);

	for(I=-1;I>=-7;I--)
	{
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	}

      V_TERM(1, 0,0,   2.37,2793.23,-4899.07,  0.11,9995.27,7027.22);
      V_TERM(1, 0,1,   0.10, -19.65,   34.40,  0.22,  64.95, -86.10);
      V_TERM(1, 0,2,   0.06,   0.04,   -0.07,  0.11,  -0.55,  -0.07);
      V_TERM(2, 0,0,-170.42,  73.13,  -16.59,  0.00,  67.71,  47.56);
      V_TERM(2, 0,1,   0.93,   2.91,    0.23,  0.00,  -0.03,  -0.92);
      V_TERM(3, 0,0,  -2.31,   0.90,   -0.08,  0.00,   0.04,   2.09);
      V_TERM(1,-1,0,  -2.38,  -4.27,    3.27, -1.82,   0.00,   0.00);
      V_TERM(1,-2,0,   0.09,   0.00,   -0.08,  0.05,  -0.02,  -0.25);
      V_TERM(2,-2,0,  -9.57,  -5.93,    8.57,-13.83,  -0.01,  -0.01);
      V_TERM(2,-3,0,  -2.47,  -2.40,    0.83, -0.95,   0.16,   0.24);
      V_TERM(3,-2,0,  -0.09,  -0.05,    0.08, -0.13,  -0.28,   0.12);
      V_TERM(3,-3,0,   7.12,   0.32,   -0.62, 13.76,  -0.07,   0.01);
      V_TERM(3,-4,0,  -0.65,  -0.17,    0.18, -0.73,   0.10,   0.05);
      V_TERM(3,-5,0,  -1.08,  -0.95,   -0.17,  0.22,  -0.03,  -0.03);
      V_TERM(4,-3,0,   0.06,   0.00,   -0.01,  0.08,   0.14,  -0.18);
      V_TERM(4,-4,0,   0.93,  -0.46,    1.06,  2.13,  -0.01,   0.01);
      V_TERM(4,-5,0,  -1.53,   0.38,   -0.64, -2.54,   0.27,   0.00);
      V_TERM(4,-6,0,  -0.17,  -0.05,    0.03, -0.11,   0.02,   0.00);
      V_TERM(5,-5,0,   0.18,  -0.28,    0.71,  0.47,  -0.02,   0.04);
      V_TERM(5,-6,0,   0.15,  -0.14,    0.30,  0.31,  -0.04,   0.03);
      V_TERM(5,-7,0,  -0.08,   0.02,   -0.03, -0.11,   0.01,   0.00);
      V_TERM(5,-8,0,  -0.23,   0.00,    0.01, -0.04,   0.00,   0.00);
      V_TERM(6,-6,0,   0.01,  -0.14,    0.39,  0.04,   0.00,  -0.01);
      V_TERM(6,-7,0,   0.02,  -0.05,    0.12,  0.04,  -0.01,   0.01);
      V_TERM(6,-8,0,   0.10,  -0.10,    0.19,  0.19,  -0.02,   0.02);
      V_TERM(7,-7,0,  -0.03,  -0.06,    0.18, -0.08,   0.00,   0.00);
      V_TERM(8,-8,0,  -0.03,  -0.02,    0.06, -0.08,   0.00,   0.00);

}

//PROCEDURE PERTMAR;  (* perturbations by Mars *)
void Castro::V_PERTMAR()
{
    int I;
	
      C[-1 +8]=cos(M4); 
	  S[-1 +8]=-sin(M4);

	  for(I=-1;I>=-2;I--)
	  {
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	  }

      V_TERM(1,-3,0,  -0.65,   1.02,   -0.04, -0.02,  -0.02,   0.00);
      V_TERM(2,-2,0,  -0.05,   0.04,   -0.09, -0.10,   0.00,   0.00);
      V_TERM(2,-3,0,  -0.50,   0.45,   -0.79, -0.89,   0.01,   0.03);

}

//  PROCEDURE PERTJUP;  (* perturbations by Jupiter *)
void Castro::V_PERTJUP()
{
   int I;

      C[-1 +8]=cos(M5); 
	  S[-1 +8]=-sin(M5);

	  for(I=-1;I>=-2;I--)
	  {
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	  }

      V_TERM(0,-1,0,  -0.05,   1.56,    0.16,  0.04,  -0.08,  -0.04);
      V_TERM(1,-1,0,  -2.62,   1.40,   -2.35, -4.40,   0.02,   0.03);
      V_TERM(1,-2,0,  -0.47,  -0.08,    0.12, -0.76,   0.04,  -0.18);
      V_TERM(2,-2,0,  -0.73,  -0.51,    1.27, -1.82,  -0.01,   0.01);
      V_TERM(2,-3,0,  -0.14,  -0.10,    0.25, -0.34,   0.00,   0.00);
      V_TERM(3,-3,0,  -0.01,   0.04,   -0.11, -0.02,   0.00,   0.00);
}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
void Castro::V_PERTSAT()
{

      C[-1 +8]=cos(M6); S[-1 +8]=-sin(M6);
      V_TERM(0,-1,0,   0.00,   0.21,    0.00,  0.00,   0.00,  -0.01);
      V_TERM(1,-1,0,  -0.11,  -0.14,    0.24, -0.20,   0.01,   0.00);
}

//(*-----------------------------------------------------------------------*)
//(* MAR200: Mars; ecliptic coordinates L,B,R (in deg and AU)              *)
//(*         equinox of date                                               *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE MAR200(T:REAL;VAR L,B,R:REAL);
void Castro::MAR200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;

	T=TT;

    DL=0.0; DR=0.0; DB=0.0;
    M2=P2*FRAC(0.1382208+162.5482542*T);
    M3=P2*FRAC(0.9926208+99.9970236*T);
    M4=P2*FRAC(0.0538553+ 53.1662736*T);
    M5=P2*FRAC(0.0548944+ 8.4290611*T);
    M6=P2*FRAC(0.8811167+  3.3935250*T);
    C4[0 +2]=1.0; 
	S4[0 +2]=0.0;  
	C4[1 +2]=cos(M4); 
	S4[1 +2]=sin(M4);

	for(I=2;I<=16;I++)
	{
    ADDTHE(C4[I-1 +2],S4[I-1 +2],C4[1 +2],S4[1 +2],&C4[I +2],&S4[I +2]);
	}

	for(I=-2;I<=-1;I++)
	{
		C4[I +2]=C4[-I +2]; 
		S4[I +2]=-S4[-I +2];
	}

    MA_PERTVEN();
	MA_PERTEAR(); 
	MA_PERTJUP(); 
	MA_PERTSAT();

    DL=DL + 52.49*sin(P2*(0.1868+0.0549*T))
           +  0.61*sin(P2*(0.9220+0.3307*T))
           +  0.32*sin(P2*(0.4731+2.1485*T))
           +  0.28*sin(P2*(0.9467+0.1133*T));
    DL=DL + (0.14+0.87*T-0.11*T*T);
    *L= 360.0*FRAC(0.9334591 + M4/P2 + ((6615.5+1.1*T)*T+DL)/1296.0E3 );
    *R= 1.5303352 + 0.0000131*T  +  DR*1.0E-6;
    *B= ( 596.32 + (-2.92 - 0.10*T) * T  +  DB ) / 3600.0;
}


//  PROCEDURE TERM(I1,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::MA_TERM(int I1, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS)
{
    if(IT==0)
	{
		ADDTHE(C4[I1 +2],S4[I1 +2],C[I +9],S[I +9],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T ;
	}

    DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;
}

//PROCEDURE PERTVEN; (* perturbations by Venus *)
void Castro::MA_PERTVEN()
{
    C[0 +9]=1.0; 
	S[0 +9]=0.0; 
	C[-1 +9]=cos(M2); 
	S[-1 +9]=-sin(M2);

    ADDTHE(C[-1 +9],S[-1 +9],C[-1 +9],S[-1 +9],&C[-2 +9],&S[-2 +9]);

    MA_TERM( 0,-1,0, -0.01,   -0.03,      0.10, -0.04,    0.00,   0.00);
    MA_TERM( 1,-1,0,  0.05,    0.10,     -2.08,  0.75,    0.00,   0.00);
    MA_TERM( 2,-1,0, -0.25,   -0.57,     -2.58,  1.18,    0.05,  -0.04);
    MA_TERM( 2,-2,0,  0.02,    0.02,      0.13, -0.14,    0.00,   0.00);
    MA_TERM( 3,-1,0,  3.41,    5.38,      1.87, -1.15,    0.01,  -0.01);
    MA_TERM( 3,-2,0,  0.02,    0.02,      0.11, -0.13,    0.00,   0.00);
    MA_TERM( 4,-1,0,  0.32,    0.49,     -1.88,  1.21,   -0.07,   0.07);
    MA_TERM( 4,-2,0,  0.03,    0.03,      0.12, -0.14,    0.00,   0.00);
    MA_TERM( 5,-1,0,  0.04,    0.06,     -0.17,  0.11,   -0.01,   0.01);
    MA_TERM( 5,-2,0,  0.11,    0.09,      0.35, -0.43,   -0.01,   0.01);
    MA_TERM( 6,-2,0, -0.36,   -0.28,     -0.20,  0.25,    0.00,   0.00);
    MA_TERM( 7,-2,0, -0.03,   -0.03,      0.11, -0.13,    0.00,  -0.01);
}


//PROCEDURE PERTEAR;  (* Kepler terms and perturbations by the Earth *)
void Castro::MA_PERTEAR()
{
    int I;

    C[-1 +9]=cos(M3); 
	S[-1 +9]=-sin(M3);

	  for(I=-1;I>= -8;I--)
	  {
		ADDTHE(C[I +9],S[I +9],C[-1 +9],S[-1 +9],&C[I-1 +9],&S[I-1 +9]);
	  }

      MA_TERM( 1, 0,0, -5.32,38481.97,-141856.04,  0.40,-6321.67,1876.89);
      MA_TERM( 1, 0,1, -1.12,   37.98,   -138.67, -2.93,   37.28, 117.48);
      MA_TERM( 1, 0,2, -0.32,   -0.03,      0.12, -1.19,    1.04,  -0.40);
      MA_TERM( 2, 0,0, 28.28, 2285.80,  -6608.37,  0.00, -589.35, 174.81);
      MA_TERM( 2, 0,1,  1.64,    3.37,    -12.93,  0.00,    2.89,  11.10);
      MA_TERM( 2, 0,2,  0.00,    0.00,      0.00,  0.00,    0.10,  -0.03);
      MA_TERM( 3, 0,0,  5.31,  189.29,   -461.81,  0.00,  -61.98,  18.53);
      MA_TERM( 3, 0,1,  0.31,    0.35,     -1.36,  0.00,    0.25,   1.19);
      MA_TERM( 4, 0,0,  0.81,   17.96,    -38.26,  0.00,   -6.88,   2.08);
      MA_TERM( 4, 0,1,  0.05,    0.04,     -0.15,  0.00,    0.02,   0.14);
      MA_TERM( 5, 0,0,  0.11,    1.83,     -3.48,  0.00,   -0.79,   0.24);
      MA_TERM( 6, 0,0,  0.02,    0.20,     -0.34,  0.00,   -0.09,   0.03);
      MA_TERM(-1,-1,0,  0.09,    0.06,      0.14, -0.22,    0.02,  -0.02);
      MA_TERM( 0,-1,0,  0.72,    0.49,      1.55, -2.31,    0.12,  -0.10);
      MA_TERM( 1,-1,0,  7.00,    4.92,     13.93,-20.48,    0.08,  -0.13);
      MA_TERM( 2,-1,0, 13.08,    4.89,     -4.53, 10.01,   -0.05,   0.13);
      MA_TERM( 2,-2,0,  0.14,    0.05,     -0.48, -2.66,    0.01,   0.14);
      MA_TERM( 3,-1,0,  1.38,    0.56,     -2.00,  4.85,   -0.01,   0.19);
      MA_TERM( 3,-2,0, -6.85,    2.68,      8.38, 21.42,    0.00,   0.03);
      MA_TERM( 3,-3,0, -0.08,    0.20,      1.20,  0.46,    0.00,   0.00);
      MA_TERM( 4,-1,0,  0.16,    0.07,     -0.19,  0.47,   -0.01,   0.05);
      MA_TERM( 4,-2,0, -4.41,    2.14,     -3.33, -7.21,   -0.07,  -0.09);
      MA_TERM( 4,-3,0, -0.12,    0.33,      2.22,  0.72,   -0.03,  -0.02);
      MA_TERM( 4,-4,0, -0.04,   -0.06,     -0.36,  0.23,    0.00,   0.00);
      MA_TERM( 5,-2,0, -0.44,    0.21,     -0.70, -1.46,   -0.06,  -0.07);
      MA_TERM( 5,-3,0,  0.48,   -2.60,     -7.25, -1.37,    0.00,   0.00);
      MA_TERM( 5,-4,0, -0.09,   -0.12,     -0.66,  0.50,    0.00,   0.00);
      MA_TERM( 5,-5,0,  0.03,    0.00,      0.01, -0.17,    0.00,   0.00);
      MA_TERM( 6,-2,0, -0.05,    0.03,     -0.07, -0.15,   -0.01,  -0.01);
      MA_TERM( 6,-3,0,  0.10,   -0.96,      2.36,  0.30,    0.04,   0.00);
      MA_TERM( 6,-4,0, -0.17,   -0.20,     -1.09,  0.94,    0.02,  -0.02);
      MA_TERM( 6,-5,0,  0.05,    0.00,      0.00, -0.30,    0.00,   0.00);
      MA_TERM( 7,-3,0,  0.01,   -0.10,      0.32,  0.04,    0.02,   0.00);
      MA_TERM( 7,-4,0,  0.86,    0.77,      1.86, -2.01,    0.01,  -0.01);
      MA_TERM( 7,-5,0,  0.09,   -0.01,     -0.05, -0.44,    0.00,   0.00);
      MA_TERM( 7,-6,0, -0.01,    0.02,      0.10,  0.08,    0.00,   0.00);
      MA_TERM( 8,-4,0,  0.20,    0.16,     -0.53,  0.64,   -0.01,   0.02);
      MA_TERM( 8,-5,0,  0.17,   -0.03,     -0.14, -0.84,    0.00,   0.01);
      MA_TERM( 8,-6,0, -0.02,    0.03,      0.16,  0.09,    0.00,   0.00);
      MA_TERM( 9,-5,0, -0.55,    0.15,      0.30,  1.10,    0.00,   0.00);
      MA_TERM( 9,-6,0, -0.02,    0.04,      0.20,  0.10,    0.00,   0.00);
      MA_TERM(10,-5,0, -0.09,    0.03,     -0.10, -0.33,    0.00,  -0.01);
      MA_TERM(10,-6,0, -0.05,    0.11,      0.48,  0.21,   -0.01,   0.00);
      MA_TERM(11,-6,0,  0.10,   -0.35,     -0.52, -0.15,    0.00,   0.00);
      MA_TERM(11,-7,0, -0.01,   -0.02,     -0.10,  0.07,    0.00,   0.00);
      MA_TERM(12,-6,0,  0.01,   -0.04,      0.18,  0.04,    0.01,   0.00);
      MA_TERM(12,-7,0, -0.05,   -0.07,     -0.29,  0.20,    0.01,   0.00);
      MA_TERM(13,-7,0,  0.23,    0.27,      0.25, -0.21,    0.00,   0.00);
      MA_TERM(14,-7,0,  0.02,    0.03,     -0.10,  0.09,    0.00,   0.00);
      MA_TERM(14,-8,0,  0.05,    0.01,      0.03, -0.23,    0.00,   0.03);
      MA_TERM(15,-8,0, -1.53,    0.27,      0.06,  0.42,    0.00,   0.00);
      MA_TERM(16,-8,0, -0.14,    0.02,     -0.10, -0.55,   -0.01,  -0.02);
      MA_TERM(16,-9,0,  0.03,   -0.06,     -0.25, -0.11,    0.00,   0.00);
}

//PROCEDURE PERTJUP; (* perturbations by Jupiter *)
void Castro::MA_PERTJUP()
{
    int I;

      C[-1 +9]=cos(M5); 
	  S[-1 +9]=-sin(M5);

	for(I=-1;I>=-4;I--)
	{
      ADDTHE(C[I +9],S[I +9],C[-1 +9],S[-1 +9],&C[I-1 +9],&S[I-1 +9]);
	}


      MA_TERM(-2,-1,0,  0.05,    0.03,      0.08, -0.14,    0.01,  -0.01);
      MA_TERM(-1,-1,0,  0.39,    0.27,      0.92, -1.50,   -0.03,  -0.06);
      MA_TERM(-1,-2,0, -0.16,    0.03,      0.13,  0.67,   -0.01,   0.06);
      MA_TERM(-1,-3,0, -0.02,    0.01,      0.05,  0.09,    0.00,   0.01);
      MA_TERM( 0,-1,0,  3.56,    1.13,     -5.41, -7.18,   -0.25,  -0.24);
      MA_TERM( 0,-2,0, -1.44,    0.25,      1.24,  7.96,    0.02,   0.31);
      MA_TERM( 0,-3,0, -0.21,    0.11,      0.55,  1.04,    0.01,   0.05);
      MA_TERM( 0,-4,0, -0.02,    0.02,      0.11,  0.11,    0.00,   0.01);
      MA_TERM( 1,-1,0, 16.67,  -19.15,     61.00, 53.36,   -0.06,  -0.07);
      MA_TERM( 1,-2,0,-21.64,    3.18,     -7.77,-54.64,   -0.31,   0.50);
      MA_TERM( 1,-3,0, -2.82,    1.45,     -2.53, -5.73,    0.01,   0.07);
      MA_TERM( 1,-4,0, -0.31,    0.28,     -0.34, -0.51,    0.00,   0.00);
      MA_TERM( 2,-1,0,  2.15,   -2.29,      7.04,  6.94,    0.33,   0.19);
      MA_TERM( 2,-2,0,-15.69,    3.31,    -15.70,-73.17,   -0.17,  -0.25);
      MA_TERM( 2,-3,0, -1.73,    1.95,     -9.19, -7.20,    0.02,  -0.03);
      MA_TERM( 2,-4,0, -0.01,    0.33,     -1.42,  0.08,    0.01,  -0.01);
      MA_TERM( 2,-5,0,  0.03,    0.03,     -0.13,  0.12,    0.00,   0.00);
      MA_TERM( 3,-1,0,  0.26,   -0.28,      0.73,  0.71,    0.08,   0.04);
      MA_TERM( 3,-2,0, -2.06,    0.46,     -1.61, -6.72,   -0.13,  -0.25);
      MA_TERM( 3,-3,0, -1.28,   -0.27,      2.21, -6.90,   -0.04,  -0.02);
      MA_TERM( 3,-4,0, -0.22,    0.08,     -0.44, -1.25,    0.00,   0.01);
      MA_TERM( 3,-5,0, -0.02,    0.03,     -0.15, -0.08,    0.00,   0.00);
      MA_TERM( 4,-1,0,  0.03,   -0.03,      0.08,  0.08,    0.01,   0.01);
      MA_TERM( 4,-2,0, -0.26,    0.06,     -0.17, -0.70,   -0.03,  -0.05);
      MA_TERM( 4,-3,0, -0.20,   -0.05,      0.22, -0.79,   -0.01,  -0.02);
      MA_TERM( 4,-4,0, -0.11,   -0.14,      0.93, -0.60,    0.00,   0.00);
      MA_TERM( 4,-5,0, -0.04,   -0.02,      0.09, -0.23,    0.00,   0.00);
      MA_TERM( 5,-4,0, -0.02,   -0.03,      0.13, -0.09,    0.00,   0.00);
      MA_TERM( 5,-5,0,  0.00,   -0.03,      0.21,  0.01,    0.00,   0.00);
}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
void Castro::MA_PERTSAT()
{
    int I;
 
    C[-1 +9]=cos(M6); 
	S[-1 +9]=-sin(M6);

	for(I=-1;I>=-3;I--)
	{
      ADDTHE(C[I +9],S[I +9],C[-1 +9],S[-1 +9],&C[I-1 +9],&S[I-1 +9]);
	}

      MA_TERM(-1,-1,0,  0.03,    0.13,      0.48, -0.13,    0.02,   0.00);
      MA_TERM( 0,-1,0,  0.27,    0.84,      0.40, -0.43,    0.01,  -0.01);
      MA_TERM( 0,-2,0,  0.12,   -0.04,     -0.33, -0.55,   -0.01,  -0.02);
      MA_TERM( 0,-3,0,  0.02,   -0.01,     -0.07, -0.08,    0.00,   0.00);
      MA_TERM( 1,-1,0,  1.12,    0.76,     -2.66,  3.91,   -0.01,   0.01);
      MA_TERM( 1,-2,0,  1.49,   -0.95,      3.07,  4.83,    0.04,  -0.05);
      MA_TERM( 1,-3,0,  0.21,   -0.18,      0.55,  0.64,    0.00,   0.00);
      MA_TERM( 2,-1,0,  0.12,    0.10,     -0.29,  0.34,   -0.01,   0.02);
      MA_TERM( 2,-2,0,  0.51,   -0.36,      1.61,  2.25,    0.03,   0.01);
      MA_TERM( 2,-3,0,  0.10,   -0.10,      0.50,  0.43,    0.00,   0.00);
      MA_TERM( 2,-4,0,  0.01,   -0.02,      0.11,  0.05,    0.00,   0.00);
      MA_TERM( 3,-2,0,  0.07,   -0.05,      0.16,  0.22,    0.01,   0.01);
}

//(*-----------------------------------------------------------------------*)
//(* JUP200: Jupiter; ecliptic coordinates L,B,R (in deg and AU)           *)
//(*         equinox of date                                               *)
//(*         T: time in Julian centuries since J2000                       *)
//(*            = (JED-2451545.0)/36525                                    *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE JUP200(T:REAL;VAR L,B,R:REAL);
void Castro::JUP200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;

	T=TT;
	
    DL=0.0; DR=0.0; DB=0.0;
    M5=P2*FRAC(0.0565314+8.4302963*T); M6=P2*FRAC(0.8829867+3.3947688*T);
    M7=P2*FRAC(0.3969537+1.1902586*T);
    C5[0 +1]=1.0;     
	S5[0 +1]=0.0;
    C5[1 +1]=cos(M5); 
	S5[1 +1]=sin(M5);  
	C5[-1 +1]=C5[1 +1]; 
	S5[-1 +1]=-S5[1 +1];

	for(I=2;I<=5;I++)
	{
		ADDTHE(C5[I-1 +1],S5[I-1 +1],C5[1 +1],S5[1 +1],&C5[I +1],&S5[I +1]);
	}

    J_PERTSAT(); 
	J_PERTURA(); 
	J_PERTSUR();

    *L= 360.0*FRAC(0.0388910 + M5/P2 + ((5025.2+0.8*T)*T+DL)/1296.0E3 );
    *R= 5.208873 + 0.000041*T  +  DR*1.0E-5;
    *B= ( 227.3 - 0.3*T + DB ) / 3600.0;

}

//  PROCEDURE TERM(I5,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::J_TERM(int I5, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS)
{
	if(IT==0)
	{
		ADDTHE(C5[I5 +1],S5[I5 +1],C[I +10],S[I +10],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T ;
	}

	DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;
}

//  PROCEDURE PERTSAT;  (* Kepler terms and perturbations by Saturn *)
void Castro::J_PERTSAT()
{
    int I;

      C[0 +10]=1.0; 
	  S[0 +10]=0.0; 
	  C[-1 +10]=cos(M6); 
	  S[-1 +10]=-sin(M6);


		for(I=-1;I>=-9;I--)
		{
			ADDTHE(C[I +10],S[I +10],C[-1 +10],S[-1 +10],&C[I-1 +10],&S[I-1 +10]);
		}

      J_TERM(-1, -1,0,  -0.2,    1.4,     2.0,   0.6,    0.1, -0.2);
      J_TERM( 0, -1,0,   9.4,    8.9,     3.9,  -8.3,   -0.4, -1.4);
      J_TERM( 0, -2,0,   5.6,   -3.0,    -5.4,  -5.7,   -2.0,  0.0);
      J_TERM( 0, -3,0,  -4.0,   -0.1,     0.0,   5.5,    0.0,  0.0);
      J_TERM( 0, -5,0,   3.3,   -1.6,    -1.6,  -3.1,   -0.5, -1.2);
      J_TERM( 1,  0,0,-113.1,19998.6,-25208.2,-142.2,-4670.7,288.9);
      J_TERM( 1,  0,1, -76.1,   66.9,   -84.2, -95.8,   21.6, 29.4);
      J_TERM( 1,  0,2,  -0.5,   -0.3,     0.4,  -0.7,    0.1, -0.1);
      J_TERM( 1, -1,0,  78.8,  -14.5,    11.5,  64.4,   -0.2,  0.2);
      J_TERM( 1, -2,0,  -2.0, -132.4,    28.8,   4.3,   -1.7,  0.4);
      J_TERM( 1, -2,1,  -1.1,   -0.7,     0.2,  -0.3,    0.0,  0.0);
      J_TERM( 1, -3,0,  -7.5,   -6.8,    -0.4,  -1.1,    0.6, -0.9);
      J_TERM( 1, -4,0,   0.7,    0.7,     0.6,  -1.1,    0.0, -0.2);
      J_TERM( 1, -5,0,  51.5,  -26.0,   -32.5, -64.4,   -4.9,-12.4);
      J_TERM( 1, -5,1,  -1.2,   -2.2,    -2.7,   1.5,   -0.4,  0.3);
      J_TERM( 2,  0,0,  -3.4,  632.0,  -610.6,  -6.5, -226.8, 12.7);
      J_TERM( 2,  0,1,  -4.2,    3.8,    -4.1,  -4.5,    0.2,  0.6);
      J_TERM( 2, -1,0,   5.3,   -0.7,     0.7,   6.1,    0.2,  1.1);
      J_TERM( 2, -2,0, -76.4, -185.1,   260.2,-108.0,    1.6,  0.0);
      J_TERM( 2, -3,0,  66.7,   47.8,   -51.4,  69.8,    0.9,  0.3);
      J_TERM( 2, -3,1,   0.6,   -1.0,     1.0,   0.6,    0.0,  0.0);
      J_TERM( 2, -4,0,  17.0,    1.4,    -1.8,   9.6,    0.0, -0.1);
      J_TERM( 2, -5,0,1066.2, -518.3,    -1.3, -23.9,    1.8, -0.3);
      J_TERM( 2, -5,1, -25.4,  -40.3,    -0.9,   0.3,    0.0,  0.0);
      J_TERM( 2, -5,2,  -0.7,    0.5,     0.0,   0.0,    0.0,  0.0);
      J_TERM( 3,  0,0,  -0.1,   28.0,   -22.1,  -0.2,  -12.5,  0.7);
      J_TERM( 3, -2,0,  -5.0,  -11.5,    11.7,  -5.4,    2.1, -1.0);
      J_TERM( 3, -3,0,  16.9,   -6.4,    13.4,  26.9,   -0.5,  0.8);
      J_TERM( 3, -4,0,   7.2,  -13.3,    20.9,  10.5,    0.1, -0.1);
      J_TERM( 3, -5,0,  68.5,  134.3,  -166.9,  86.5,    7.1, 15.2);
      J_TERM( 3, -5,1,   3.5,   -2.7,     3.4,   4.3,    0.5, -0.4);
      J_TERM( 3, -6,0,   0.6,    1.0,    -0.9,   0.5,    0.0,  0.0);
      J_TERM( 3, -7,0,  -1.1,    1.7,    -0.4,  -0.2,    0.0,  0.0);
      J_TERM( 4,  0,0,   0.0,    1.4,    -1.0,   0.0,   -0.6,  0.0);
      J_TERM( 4, -2,0,  -0.3,   -0.7,     0.4,  -0.2,    0.2, -0.1);
      J_TERM( 4, -3,0,   1.1,   -0.6,     0.9,   1.2,    0.1,  0.2);
      J_TERM( 4, -4,0,   3.2,    1.7,    -4.1,   5.8,    0.2,  0.1);
      J_TERM( 4, -5,0,   6.7,    8.7,    -9.3,   8.7,   -1.1,  1.6);
      J_TERM( 4, -6,0,   1.5,   -0.3,     0.6,   2.4,    0.0,  0.0);
      J_TERM( 4, -7,0,  -1.9,    2.3,    -3.2,  -2.7,    0.0, -0.1);
      J_TERM( 4, -8,0,   0.4,   -1.8,     1.9,   0.5,    0.0,  0.0);
      J_TERM( 4, -9,0,  -0.2,   -0.5,     0.3,  -0.1,    0.0,  0.0);
      J_TERM( 4,-10,0,  -8.6,   -6.8,    -0.4,   0.1,    0.0,  0.0);
      J_TERM( 4,-10,1,  -0.5,    0.6,     0.0,   0.0,    0.0,  0.0);
      J_TERM( 5, -5,0,  -0.1,    1.5,    -2.5,  -0.8,   -0.1,  0.1);
      J_TERM( 5, -6,0,   0.1,    0.8,    -1.6,   0.1,    0.0,  0.0);
      J_TERM( 5, -9,0,  -0.5,   -0.1,     0.1,  -0.8,    0.0,  0.0);
      J_TERM( 5,-10,0,   2.5,   -2.2,     2.8,   3.1,    0.1, -0.2);
}

//  PROCEDURE PERTURA;  (* perturbations by Uranus *)
void Castro::J_PERTURA()
{
  
 
      C[-1 +10]=cos(M7); 
	  S[-1 +10]=-sin(M7);

      ADDTHE(C[-1 +10],S[-1 +10],C[-1 +10],S[-1 +10],&C[-2 +10],&S[-2 +10]);

      J_TERM( 1, -1,0,   0.4,    0.9,     0.0,   0.0,    0.0,  0.0);
      J_TERM( 1, -2,0,   0.4,    0.4,    -0.4,   0.3,    0.0,  0.0);
}

//  PROCEDURE PERTSUR;  (* perturbations by Saturn and Uranus *)
void Castro::J_PERTSUR()
{
	double PHI,X,Y;

    PHI=(2*M5-6*M6+3*M7); X=cos(PHI); Y=sin(PHI);
    DL=DL-0.8*X+8.5*Y; DR=DR-0.1*X;
    ADDTHE(X,Y,C5[1 +1],S5[1 +1],&X,&Y);
    DL=DL+0.4*X+0.5*Y; DR=DR-0.7*X+0.5*Y; DB=DB-0.1*X;
}

//(*-----------------------------------------------------------------------*)
//(* URA200: Uranus; ecliptic coordinates L,B,R (in deg and AU)            *)
//(*         equinox of date                                               *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE URA200(T:REAL;VAR L,B,R:REAL);
void Castro::URA200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;

	T=TT;
	
    DL=0.0; DR=0.0; DB=0.0;
    M5=P2*FRAC(0.0564472+8.4302889*T); M6=P2*FRAC(0.8829611+3.3947583*T);
    M7=P2*FRAC(0.3967117+1.1902849*T); M8=P2*FRAC(0.7216833+0.6068528*T);
    C7[0 +2]=1.0; 
	S7[0 +2]=0.0; 
	C7[1 +2]=cos(M7); 
	S7[1 +2]=sin(M7);

	for(I=2;I<=7;I++)
	{
		ADDTHE(C7[I-1 +2],S7[I-1 +2],C7[1 +2],S7[1 +2],&C7[I +2],&S7[I +2]);
	}

	for(I=1;I<=2;I++)
	{
		C7[-I +2]=C7[I +2]; 
		S7[-I +2]=-S7[I +2];
	}

    U_PERTJUP(); 
	U_PERTSAT(); 
	U_PERTNEP(); 
	U_PERTJSU();

    *L= 360.0*FRAC(0.4734843 + M7/P2 + ((5082.3+34.2*T)*T+DL)/1296.0E3 );
    *R= 19.211991 + (-0.000333-0.000005*T)*T  +  DR*1.0E-5;
    *B= (-130.61 + (-0.54+0.04*T)*T + DB ) / 3600.0;
}

//  PROCEDURE TERM(I7,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::U_TERM(int I7, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS)
{
	if(IT==0)
	{
		ADDTHE(C7[I7 +2],S7[I7 +2],C[I +8],S[I +8],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T ;
	}
    DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;
}

//  PROCEDURE PERTJUP;  (* perturbations by Jupiter *)
void Castro::U_PERTJUP()
{

      C[0 +8]=1.0; 
	  S[0 +8]=0.0; 
	  C[-1 +8]=cos(M5); 
	  S[-1 +8]=-sin(M5);

      ADDTHE(C[-1 +8],S[-1 +8],C[-1 +8],S[-1 +8],&C[-2 +8],&S[-2 +8]);

      U_TERM(-1,-1,0,  0.0,    0.0,    -0.1,   1.7,  -0.1,   0.0);
      U_TERM( 0,-1,0,  0.5,   -1.2,    18.9,   9.1,  -0.9,   0.1);
      U_TERM( 1,-1,0,-21.2,   48.7,  -455.5,-198.8,   0.0,   0.0);
      U_TERM( 1,-2,0, -0.5,    1.2,   -10.9,  -4.8,   0.0,   0.0);
      U_TERM( 2,-1,0, -1.3,    3.2,   -23.2, -11.1,   0.3,   0.1);
      U_TERM( 2,-2,0, -0.2,    0.2,     1.1,   1.5,   0.0,   0.0);
      U_TERM( 3,-1,0,  0.0,    0.2,    -1.8,   0.4,   0.0,   0.0);
}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
void Castro::U_PERTSAT()
{
    int I;

    C[-1]=cos(M6); 
	S[-1]=-sin(M6);

	for(I=-1;I>=-3;I--)
	{
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	}
      U_TERM( 0,-1,0,  1.4,   -0.5,    -6.4,   9.0,  -0.4,  -0.8);
      U_TERM( 1,-1,0,-18.6,  -12.6,    36.7,-336.8,   1.0,   0.3);
      U_TERM( 1,-2,0, -0.7,   -0.3,     0.5,  -7.5,   0.1,   0.0);
      U_TERM( 2,-1,0, 20.0, -141.6,  -587.1,-107.0,   3.1,  -0.8);
      U_TERM( 2,-1,1,  1.0,    1.4,     5.8,  -4.0,   0.0,   0.0);
      U_TERM( 2,-2,0,  1.6,   -3.8,   -35.6, -16.0,   0.0,   0.0);
      U_TERM( 3,-1,0, 75.3, -100.9,   128.9,  77.5,  -0.8,   0.1);
      U_TERM( 3,-1,1,  0.2,    1.8,    -1.9,   0.3,   0.0,   0.0);
      U_TERM( 3,-2,0,  2.3,   -1.3,    -9.5, -17.9,   0.0,   0.1);
      U_TERM( 3,-3,0, -0.7,   -0.5,    -4.9,   6.8,   0.0,   0.0);
      U_TERM( 4,-1,0,  3.4,   -5.0,    21.6,  14.3,  -0.8,  -0.5);
      U_TERM( 4,-2,0,  1.9,    0.1,     1.2, -12.1,   0.0,   0.0);
      U_TERM( 4,-3,0, -0.1,   -0.4,    -3.9,   1.2,   0.0,   0.0);
      U_TERM( 4,-4,0, -0.2,    0.1,     1.6,   1.8,   0.0,   0.0);
      U_TERM( 5,-1,0,  0.2,   -0.3,     1.0,   0.6,  -0.1,   0.0);
      U_TERM( 5,-2,0, -2.2,   -2.2,    -7.7,   8.5,   0.0,   0.0);
      U_TERM( 5,-3,0,  0.1,   -0.2,    -1.4,  -0.4,   0.0,   0.0);
      U_TERM( 5,-4,0, -0.1,    0.0,     0.1,   1.2,   0.0,   0.0);
      U_TERM( 6,-2,0, -0.2,   -0.6,     1.4,  -0.7,   0.0,   0.0);
}

//  PROCEDURE PERTNEP;  (* Kepler terms and perturbations by Neptune  *)
void Castro::U_PERTNEP()
{
    int I;

    C[-1]=cos(M8); 
	S[-1]=-sin(M8);

	for(I=-1;I>=-7;I--)
	{
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	}

      U_TERM( 1, 0,0,-78.1,19518.1,-90718.2,-334.7,2759.5,-311.9);
      U_TERM( 1, 0,1,-81.6,  107.7,  -497.4,-379.5,  -2.8, -43.7);
      U_TERM( 1, 0,2, -6.6,   -3.1,    14.4, -30.6,  -0.4,  -0.5);
      U_TERM( 1, 0,3,  0.0,   -0.5,     2.4,   0.0,   0.0,   0.0);
      U_TERM( 2, 0,0, -2.4,  586.1, -2145.2, -15.3, 130.6, -14.3);
      U_TERM( 2, 0,1, -4.5,    6.6,   -24.2, -17.8,   0.7,  -1.6);
      U_TERM( 2, 0,2, -0.4,    0.0,     0.1,  -1.4,   0.0,   0.0);
      U_TERM( 3, 0,0,  0.0,   24.5,   -76.2,  -0.6,   7.0,  -0.7);
      U_TERM( 3, 0,1, -0.2,    0.4,    -1.4,  -0.8,   0.1,  -0.1);
      U_TERM( 4, 0,0,  0.0,    1.1,    -3.0,   0.1,   0.4,   0.0);
      U_TERM(-1,-1,0, -0.2,    0.2,     0.7,   0.7,  -0.1,   0.0);
      U_TERM( 0,-1,0, -2.8,    2.5,     8.7,  10.5,  -0.4,  -0.1);
      U_TERM( 1,-1,0,-28.4,   20.3,   -51.4, -72.0,   0.0,   0.0);
      U_TERM( 1,-2,0, -0.6,   -0.1,     4.2, -14.6,   0.2,   0.4);
      U_TERM( 1,-3,0,  0.2,    0.5,     3.4,  -1.6,  -0.1,   0.1);
      U_TERM( 2,-1,0, -1.8,    1.3,    -5.5,  -7.7,   0.0,   0.3);
      U_TERM( 2,-2,0, 29.4,   10.2,   -29.0,  83.2,   0.0,   0.0);
      U_TERM( 2,-3,0,  8.8,   17.8,   -41.9,  21.5,  -0.1,  -0.3);
      U_TERM( 2,-4,0,  0.0,    0.1,    -2.1,  -0.9,   0.1,   0.0);
      U_TERM( 3,-2,0,  1.5,    0.5,    -1.7,   5.1,   0.1,  -0.2);
      U_TERM( 3,-3,0,  4.4,   14.6,   -84.3,  25.2,   0.1,  -0.1);
      U_TERM( 3,-4,0,  2.4,   -4.5,    12.0,   6.2,   0.0,   0.0);
      U_TERM( 3,-5,0,  2.9,   -0.9,     2.1,   6.2,   0.0,   0.0);
      U_TERM( 4,-3,0,  0.3,    1.0,    -4.0,   1.1,   0.1,  -0.1);
      U_TERM( 4,-4,0,  2.1,   -2.7,    17.9,  14.0,   0.0,   0.0);
      U_TERM( 4,-5,0,  3.0,   -0.4,     2.3,  17.6,  -0.1,  -0.1);
      U_TERM( 4,-6,0, -0.6,   -0.5,     1.1,  -1.6,   0.0,   0.0);
      U_TERM( 5,-4,0,  0.2,   -0.2,     1.0,   0.8,   0.0,   0.0);
      U_TERM( 5,-5,0, -0.9,   -0.1,     0.6,  -7.1,   0.0,   0.0);
      U_TERM( 5,-6,0, -0.5,   -0.6,     3.8,  -3.6,   0.0,   0.0);
      U_TERM( 5,-7,0,  0.0,   -0.5,     3.0,   0.1,   0.0,   0.0);
      U_TERM( 6,-6,0,  0.2,    0.3,    -2.7,   1.6,   0.0,   0.0);
      U_TERM( 6,-7,0, -0.1,    0.2,    -2.0,  -0.4,   0.0,   0.0);
      U_TERM( 7,-7,0,  0.1,   -0.2,     1.3,   0.5,   0.0,   0.0);
      U_TERM( 7,-8,0,  0.1,    0.0,     0.4,   0.9,   0.0,   0.0);
}

//  PROCEDURE PERTJSU;  (* perturbations by Jupiter and Saturn *)
void Castro::U_PERTJSU()
{
    int I;

      C[-1 +8]=cos(M6);         
	  S[-1 +8]=-sin(M6);
      C[-4 +8]=cos(-4*M6+2*M5); 
	  S[-4 +8]= sin(-4*M6+2*M5);

	  for(I=-4;I>=-5;I--)
	  {
		ADDTHE(C[I +8],S[I +8],C[-1 +8],S[-1 +8],&C[I-1 +8],&S[I-1 +8]);
	  }

      U_TERM(-2,-4,0, -0.7,    0.4,    -1.5,  -2.5,   0.0,   0.0);
      U_TERM(-1,-4,0, -0.1,   -0.1,    -2.2,   1.0,   0.0,   0.0);
      U_TERM( 1,-5,0,  0.1,   -0.4,     1.4,   0.2,   0.0,   0.0);
      U_TERM( 1,-6,0,  0.4,    0.5,    -0.8,  -0.8,   0.0,   0.0);
      U_TERM( 2,-6,0,  5.7,    6.3,    28.5, -25.5,   0.0,   0.0);
      U_TERM( 2,-6,1,  0.1,   -0.2,    -1.1,  -0.6,   0.0,   0.0);
      U_TERM( 3,-6,0, -1.4,   29.2,   -11.4,   1.1,   0.0,   0.0);
      U_TERM( 3,-6,1,  0.8,   -0.4,     0.2,   0.3,   0.0,   0.0);
      U_TERM( 4,-6,0,  0.0,    1.3,    -6.0,  -0.1,   0.0,   0.0);
}

//(*-----------------------------------------------------------------------*)
//(* NEP200: Neptune; ecliptic coordinates L,B,R (in deg and AU)           *)
//(*         equinox of date                                               *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE NEP200(T:REAL;VAR L,B,R:REAL);
void Castro::NEP200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;
	T=TT;

	DL=0.0; DR=0.0; DB=0.0;
    M5=P2*FRAC(0.0563867+8.4298907*T); M6=P2*FRAC(0.8825086+3.3957748*T);
    M7=P2*FRAC(0.3965358+1.1902851*T); M8=P2*FRAC(0.7214906+0.6068526*T);
    C8[0]=1.0; S8[0]=0.0; C8[1]=cos(M8); S8[1]=sin(M8);

	for(I=2;I<=6;I++)
	{
		ADDTHE(C8[I-1],S8[I-1],C8[1],S8[1],&C8[I],&S8[I]);
	}


    N_PERTJUP(); N_PERTSAT(); N_PERTURA();

    *L= 360.0*FRAC(0.1254046 + M8/P2 + ((4982.8-21.3*T)*T+DL)/1296.0E3 );
    *R= 30.072984 + (0.001234+0.000003*T) * T  +  DR*1.0E-5;
    *B= (  54.77 + ( 0.26 + 0.06*T) * T  +  DB ) / 3600.0;
}

//  PROCEDURE TERM(I1,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::N_TERM(int I1, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS)
{
	if(IT==0)
	{		
		ADDTHE(C8[I1],S8[I1],C[I +6],S[I +6],&U,&V);
	}
	else
	{
		U=U*T; 
		V=V*T ;
	}
      
	DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V; 
	DB=DB+DBC*U+DBS*V;
}

//  PROCEDURE PERTJUP;  (* perturbations by Jupiter *)
void Castro::N_PERTJUP()
{

      C[0 +6]=1.0; 
	  S[0 +6]=0.0; 
	  C[-1 +6]=cos(M5); 
	  S[-1 +6]=-sin(M5);

      ADDTHE(C[-1 +6],S[-1 +6],C[-1 +6],S[-1 +6],&C[-2 +6],&S[-2 +6]);
      N_TERM(0,-1,0,  0.1,   0.1,    -3.0,   1.8,   -0.3, -0.3);
      N_TERM(1, 0,0,  0.0,   0.0,   -15.9,   9.0,    0.0,  0.0);
      N_TERM(1,-1,0,-17.6, -29.3,   416.1,-250.0,    0.0,  0.0);
      N_TERM(1,-2,0, -0.4,  -0.7,    10.4,  -6.2,    0.0,  0.0);
      N_TERM(2,-1,0, -0.2,  -0.4,     2.4,  -1.4,    0.4, -0.3);
}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
void Castro::N_PERTSAT()
{

      C[0 +6]=1.0; 
	  S[0 +6]=0.0; 
	  C[-1 +6]=cos(M6); 
	  S[-1 +6]=-sin(M6);

      ADDTHE(C[-1 +6],S[-1 +6],C[-1 +6],S[-1 +6],&C[-2 +6],&S[-2 +6]);

      N_TERM(0,-1,0, -0.1,   0.0,     0.2,  -1.8,   -0.1, -0.5);
      N_TERM(1, 0,0,  0.0,   0.0,    -8.3, -10.4,    0.0,  0.0);
      N_TERM(1,-1,0, 13.6, -12.7,   187.5, 201.1,    0.0,  0.0);
      N_TERM(1,-2,0,  0.4,  -0.4,     4.5,   4.5,    0.0,  0.0);
      N_TERM(2,-1,0,  0.4,  -0.1,     1.7,  -3.2,    0.2,  0.2);
      N_TERM(2,-2,0, -0.1,   0.0,    -0.2,   2.7,    0.0,  0.0);
}

//  PROCEDURE PERTURA;  (* Kepler terms and perturbations by Uranus *)
void Castro::N_PERTURA()
{
    int I;


      C[0 +6]=1.0; 
	  S[0 +6]=0.0; 
	  C[-1 +6]=cos(M7); 
	  S[-1 +6]=-sin(M7);

	  for(I=-1;I>=-5;I--)
	  {
		ADDTHE(C[I +6],S[I +6],C[-1 +6],S[-1 +6],&C[I-1 +6],&S[I-1 +6]);
	  }

      N_TERM(1, 0,0, 32.3,3549.5,-25880.2, 235.8,-6360.5,374.0);
      N_TERM(1, 0,1, 31.2,  34.4,  -251.4, 227.4,   34.9, 29.3);
      N_TERM(1, 0,2, -1.4,   3.9,   -28.6, -10.1,    0.0, -0.9);
      N_TERM(2, 0,0,  6.1,  68.0,  -111.4,   2.0,  -54.7,  3.7);
      N_TERM(2, 0,1,  0.8,  -0.2,    -2.1,   2.0,   -0.2,  0.8);
      N_TERM(3, 0,0,  0.1,   1.0,    -0.7,   0.0,   -0.8,  0.1);
      N_TERM(0,-1,0, -0.1,  -0.3,    -3.6,   0.0,    0.0,  0.0);
      N_TERM(1, 0,0,  0.0,   0.0,     5.5,  -6.9,    0.1,  0.0);
      N_TERM(1,-1,0, -2.2,  -1.6,  -116.3, 163.6,    0.0, -0.1);
      N_TERM(1,-2,0,  0.2,   0.1,    -1.2,   0.4,    0.0, -0.1);
      N_TERM(2,-1,0,  4.2,  -1.1,    -4.4, -34.6,   -0.2,  0.1);
      N_TERM(2,-2,0,  8.6,  -2.9,   -33.4, -97.0,    0.2,  0.1);
      N_TERM(3,-1,0,  0.1,  -0.2,     2.1,  -1.2,    0.0,  0.1);
      N_TERM(3,-2,0, -4.6,   9.3,    38.2,  19.8,    0.1,  0.1);
      N_TERM(3,-3,0, -0.5,   1.7,    23.5,   7.0,    0.0,  0.0);
      N_TERM(4,-2,0,  0.2,   0.8,     3.3,  -1.5,   -0.2, -0.1);
      N_TERM(4,-3,0,  0.9,   1.7,    17.9,  -9.1,   -0.1,  0.0);
      N_TERM(4,-4,0, -0.4,  -0.4,    -6.2,   4.8,    0.0,  0.0);
      N_TERM(5,-3,0, -1.6,  -0.5,    -2.2,   7.0,    0.0,  0.0);
      N_TERM(5,-4,0, -0.4,  -0.1,    -0.7,   5.5,    0.0,  0.0);
      N_TERM(5,-5,0,  0.2,   0.0,     0.0,  -3.5,    0.0,  0.0);
      N_TERM(6,-4,0, -0.3,   0.2,     2.1,   2.7,    0.0,  0.0);
      N_TERM(6,-5,0,  0.1,  -0.1,    -1.4,  -1.4,    0.0,  0.0);
      N_TERM(6,-6,0, -0.1,   0.1,     1.4,   0.7,    0.0,  0.0);
}

//(*-----------------------------------------------------------------------*)
//(* PLU200: Pluto; ecliptic coordinates L,B,R (in deg and AU)             *)
//(*         equinox of date; only valid between 1890 and 2100!!           *)
//(*         (T: time in Julian centuries since J2000)                     *)
//(*         (   = (JED-2451545.0)/36525             )                     *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE PLU200(T:REAL;VAR L,B,R:REAL);
void Castro::PLU200(double TT, double *L, double *B, double *R)
{
	int I;
	double P2=6.283185307;
	double LLL,BBB;
	T=TT;

	DL=0.0; DR=0.0; DB=0.0;

    M5=P2*FRAC(0.0565314+8.4302963*T); 
	M6=P2*FRAC(0.8829867+3.3947688*T);
    M9=P2*FRAC(0.0385795+0.4026667*T);

    C9[0]=1.0;
	S9[0]=0.0;  
	C9[1]=cos(M9); 
	S9[1]=sin(M9);

	for(I=2;I<=6;I++)
	{
		ADDTHE(C9[I-1],S9[I-1],C9[1],S9[1],&C9[I],&S9[I]);
	}

    P_PERTJUP(); 
	P_PERTSAT(); 
	P_PERTJUS();
    *L= 360.0*FRAC( 0.6232469 + M9/P2 + DL/1296.0E3 );
    *R= 40.7247248  +  DR * 1.0E-5;
    *B= -3.909434  +  DB / 3600.0;
	LLL = *L;
	BBB = *B;
    P_PREC(T,&LLL,&BBB);
	*L = LLL;
	*B = BBB;
}

//  PROCEDURE TERM(I9,I:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:REAL);
void Castro::P_TERM(int I9,int I,double DLC,double DLS,
					double DRC,double DRS,double DBC,double DBS)
{
	double U,V;

	ADDTHE(C9[I9],S9[I9],C[I +3],S[I +3],&U,&V);
	DL=DL+DLC*U+DLS*V; 
	DR=DR+DRC*U+DRS*V;
	DB=DB+DBC*U+DBS*V;
}

//  PROCEDURE PERTJUP;  (* Kepler terms and perturbations by Jupiter *)
void Castro::P_PERTJUP()
{
    int I;

      C[0 +3]=1.0; 
	  S[0 +3]=0.0;  
	  C[1 +3]=cos(M5); 
	  S[1 +3]=sin(M5);

	for(I=0;I>=-1;I--)
	{
		ADDTHE(C[I +3],S[I +3],C[1 +3],-S[1 +3],&C[I-1 +3],&S[I-1 +3]);
	}

      ADDTHE(C[1 +3],S[1 +3],C[1 +3],S[1 +3],&C[2 +3],&S[2 +3]);

      P_TERM(1, 0,   0.06,100924.08,-960396.0,15965.1,51987.68,-24288.76);
      P_TERM(2, 0,3274.74, 17835.12,-118252.2, 3632.4,12687.49, -6049.72);
      P_TERM(3, 0,1543.52,  4631.99, -21446.6, 1167.0, 3504.00, -1853.10);
      P_TERM(4, 0, 688.99,  1227.08,  -4823.4,  213.5, 1048.19,  -648.26);
      P_TERM(5, 0, 242.27,   415.93,  -1075.4,  140.6,  302.33,  -209.76);
      P_TERM(6, 0, 138.41,   110.91,   -308.8,  -55.3,  109.52,   -93.82);
      P_TERM(3,-1,  -0.99,     5.06,    -25.6,   19.8,    1.26,    -1.96);
      P_TERM(2,-1,   7.15,     5.61,    -96.7,   57.2,    1.64,    -2.16);
      P_TERM(1,-1,  10.79,    23.13,   -390.4,  236.4,   -0.33,     0.86);
      P_TERM(0, 1,  -0.23,     4.43,    102.8,   63.2,    3.15,     0.34);
      P_TERM(1, 1,  -1.10,    -0.92,     11.8,   -2.3,    0.43,     0.14);
      P_TERM(2, 1,   0.62,     0.84,      2.3,    0.7,    0.05,    -0.04);
      P_TERM(3, 1,  -0.38,    -0.45,      1.2,   -0.8,    0.04,     0.05);
      P_TERM(4, 1,   0.17,     0.25,      0.0,    0.2,   -0.01,    -0.01);
      P_TERM(3,-2,   0.06,     0.07,     -0.6,    0.3,    0.03,    -0.03);
      P_TERM(2,-2,   0.13,     0.20,     -2.2,    1.5,    0.03,    -0.07);
      P_TERM(1,-2,   0.32,     0.49,     -9.4,    5.7,   -0.01,     0.03);
      P_TERM(0,-2,  -0.04,    -0.07,      2.6,   -1.5,    0.07,    -0.02);
}

//  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
	void Castro::P_PERTSAT()
{
    int I;

      C[1 +3]=cos(M6); 
	  S[1 +3]=sin(M6);

		for(I=0;I>=-1;I--)
		{
			ADDTHE(C[I +3],S[I +3],C[1 +3],-S[1 +3],&C[I-1 +3],&S[I-1 +3]);
		}

      P_TERM(1,-1, -29.47,    75.97,   -106.4, -204.9,  -40.71,   -17.55);
      P_TERM(0, 1, -13.88,    18.20,     42.6,  -46.1,    1.13,     0.43);
      P_TERM(1, 1,   5.81,   -23.48,     15.0,   -6.8,   -7.48,     3.07);
      P_TERM(2, 1, -10.27,    14.16,     -7.9,    0.4,    2.43,    -0.09);
      P_TERM(3, 1,   6.86,   -10.66,      7.3,   -0.3,   -2.25,     0.69);
      P_TERM(2,-2,   4.32,     2.00,      0.0,   -2.2,   -0.24,     0.12);
      P_TERM(1,-2,  -5.04,    -0.83,     -9.2,   -3.1,    0.79,    -0.24);
      P_TERM(0,-2,   4.25,     2.48,     -5.9,   -3.3,    0.58,     0.02);
}

//  PROCEDURE PERTJUS;  (* perturbations by Jupiter and Saturn *)
void Castro::P_PERTJUS()
{
    double PHI,X,Y;
 
      PHI=(M5-M6); X=cos(PHI); Y=sin(PHI);
      DL=DL-9.11*X+0.12*Y; 
	  DR=DR-3.4*X-3.3*Y; 
	  DB=DB+0.81*X+0.78*Y;
      ADDTHE(X,Y,C9[1],S9[1],&X,&Y);
      DL=DL+5.92*X+0.25*Y; 
	  DR=DR+2.3*X-3.8*Y; 
	  DB=DB-0.67*X-0.51*Y;
}

// PROCEDURE PREC(T:REAL;VAR L,B:REAL); (* precess. 1950->equinox of date *)
void Castro::P_PREC(double TT,double *L,double *B)
{
	double P2=6.283185307;
    double DEG=57.2957795;
    double D,PPI,PI,P,C1,S1,C2,S2,C3,S3,X,Y,Z;
	double BBB,LLL;
	T=TT;

      D=T+0.5; LLL=*L/DEG; BBB=*B/DEG;
      PPI=3.044; PI=2.28E-4*D; P=(0.0243764+5.39E-6*D)*D;
      C1=cos(PI); C2=cos(BBB); C3=cos(PPI-LLL);
      S1=sin(PI); S2=sin(BBB); S3=sin(PPI-LLL);
      X=C2*C3; Y=C1*C2*S3-S1*S2; Z=S1*C2*S3+C1*S2;
      BBB = DEG * atan( Z / sqrt((1.0-Z)*(1.0+Z)) );

      if(X>0)
	  { 
		LLL=360.0*FRAC((PPI+P-atan(Y/X))/P2);
	  }
	  else
	  {
		LLL=360.0*FRAC((PPI+P-atan(Y/X))/P2+0.5);
	  }
	  *L=LLL;
	  *B=BBB;
}

//(*-----------------------------------------------------------------------*)
//(* MINI_MOON: low precision lunar coordinates (approx. 5'/1')            *)
//(*            T  : time in Julian centuries since J2000                  *)
//(*                 ( T=(JD-2451545)/36525 )                              *)
//(*            RA : right ascension (in h; equinox of date)               *)
//(*            DEC: declination (in deg; equinox of date)                 *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE MINI_MOON (T: REAL; VAR RA,DEC: REAL);
void Castro::MINI_MOON (double T, double *RA, double *DEC)
{
	double P2 =6.283185307; 
	double ARC=206264.8062;
	double COSEPS=0.91748; 
	double SINEPS=0.39778;  //(* cos/sin(obliquity ecliptic)  *)
	double L0,L,LS,F,D,H,S,N,DL,CB;
	double L_MOON,B_MOON,V,W,X,Y,Z,RHO;

    //(* mean elements of lunar orbit *)
    L0=   FRAC(0.606433+1336.855225*T); //(* mean longitude Moon (in rev) *)   
    L =P2*FRAC(0.374897+1325.552410*T); //(* mean anomaly of the Moon     *)
    LS=P2*FRAC(0.993133+  99.997361*T); //(* mean anomaly of the Sun      *)
    D =P2*FRAC(0.827361+1236.853086*T); //(* diff. longitude Moon-Sun     *)
    F =P2*FRAC(0.259086+1342.227825*T); //(* mean argument of latitude    *)
    DL = +22640*sin(L) - 4586*sin(L-2*D) + 2370*sin(2*D) +  769*sin(2*L) 
          -668*sin(LS)- 412*sin(2*F) - 212*sin(2*L-2*D) - 206*sin(L+LS-2*D)
          +192*sin(L+2*D) - 165*sin(LS-2*D) - 125*sin(D) - 110*sin(L+LS)
          +148*sin(L-LS) - 55*sin(2*F-2*D);
    S = F + (DL+412*sin(2*F)+541*sin(LS)) / ARC; 
    H = F-2*D;
    N = -526*sin(H) + 44*sin(L+H) - 31*sin(-L+H) - 23*sin(LS+H) 
         + 11*sin(-LS+H) -25*sin(-2*L+F) + 21*sin(-L+F);
    L_MOON = P2 * FRAC ( L0 + DL/1296E3 ); //(* in rad *)
    B_MOON = ( 18520.0*sin(S) + N ) / ARC; //(* in rad *)
    //(* equatorial coordinates *)
    CB=cos(B_MOON);
    X=CB*cos(L_MOON); V=CB*sin(L_MOON); W=sin(B_MOON);
    Y=COSEPS*V-SINEPS*W; Z=SINEPS*V+COSEPS*W; RHO=sqrt(1.0-Z*Z);
    *DEC = (360.0/P2)*atan(Z/RHO); 
    *RA  = ( 48.0/P2)*atan(Y/(X+RHO));

	if(*RA<0)  *RA=*RA+24.0;
}

/////////////////////////////////////////////////////////////////////
//MOON Notes:
//  Moon routines translated from Montenbruck and Pfleger:
//  The MINI_MOON routine is working and gives position with
//  accuracy of about 1 min.  The MOON routine shoud provide 
//  about 1 second of position accuracy... but I have not got
//  it working... yet, but the code is here.  To view the positions
//  from both set the number of objects in CSCmdWindow::PlotView
//  from 10 to 11, MOON will plot in gray pink.  Also you will need
//  to change the loop in CVMath::FigurePlanInfo() from 10 
//  to eleven as well.
//
//  
//////////////////////////////////////////////////////////////////////

//(*-----------------------------------------------------------------------*)
//(* MOONEQU: geocentric equatorial coordinates of the Moon                *)
//(*          referred to the true equinox of date                         *)
//(*   T   time in Julian centuries ephemeris time since J2000             *)
//(*       ( T = (JD-2451545.0)/36525 )                                    *)
//(*   RA  right ascension (deg)                                           *)
//(*   DEC declination (deg)                                               *)
//(*   R   distance (in earth radii)                                       *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE MOONEQU(T:REAL;VAR RA,DEC,R:REAL);
void Castro::MOONEQU(double TT,double *RA,double *DEC,double *R)
{
	double L,B,X,Y,Z;
	//T=TT;

    MOON(TT,&L,&B,R);              //(* ecliptic coordinates (mean equinox    *)
    CART(*R,B,L,&X,&Y,&Z);          //(*                              of date) *)
    ECLEQU(TT,&X,&Y,&Z);            //(* transform into equatorial coordinates *)
    NUTEQU(TT,&X,&Y,&Z);            //(* nutation                              *)
    POLAR(X,Y,Z,R,DEC,RA);
}

//(*-----------------------------------------------------------------------*)
//(* MOON: analytical lunar theory by E.W.Brown (Improved Lunar Ephemeris) *)
//(*       with an accuracy of approx. 1"                                  *)
//(*                                                                       *)
//(*       T:      time in Julian centuries since J2000 (Ephemeris Time)   *)
//(*               (T=(JD-2451545.0)/36525.0)                              *)
//(*       LAMBDA: geocentric ecliptic longitude (equinox of date)         *)
//(*       BETA:   geocentric ecliptic latitude  (equinox of date)         *)
//(*       R:      geocentric distance (in Earth radii)                    *)
//(*                                                                       *)
//(*-----------------------------------------------------------------------*)
//PROCEDURE MOON ( T:REAL; VAR LAMBDA,BETA,R: REAL );
void Castro::MOON(double TT, double *LAMBDA, double *BETA,double *R)
{
	const double PI2 = 6.283185308;  //(* 2*pi;  pi=3.141592654...        *)
	const double ARC = 206264.81;    //(* 3600*180/pi = arcsec per radian *)

	//these moved to class global, as they were
	//procedure globals in the pascal code:
	//double DGAM,
	//double FAC;
	//double DLAM,
	double N;//,GAM1C,SINPI;
	//double L0, L, LS, F, D ,
	double SS ;
	//double DL0,DL,DLS,DF,DD,
	//double DS;
	//CO,SI: ARRAY[-6..6,1..4] OF REAL;
	//use +6 for first, 0 for second
	//double CO[13][5],SICO[13][5];

	T = TT;

	INIT(); 
	SOLAR1(); 
	SOLAR2(); 
	SOLAR3(); 
	SOLARN(&N);  
	PLANETARY(&DLAM);
   
	*LAMBDA = 360.0*FRAC( (L0+DLAM/ARC) / PI2 );
   
	SS    = F + DS/ARC;   
	FAC  = 1.000002708+139.978*DGAM;   
	*BETA = (FAC*(18518.511+1.189+GAM1C)*sin(SS)-6.24*sin(3*SS)+N) / 3600.0;   
	SINPI = SINPI * 0.999953253;   
	*R     = ARC / SINPI;
}


//  (* calculate sin(phi); phi in units of 1 revolution = 360 degrees   *)
//  FUNCTION SINE (PHI:REAL):REAL;
double Castro::SINE(double PHI)
{
	double PI2 = 6.283185308;  //(* 2*pi;  pi=3.141592654...        *)
    double SS = sin(PI2*FRAC(PHI));
	return(SS);
}

//  (* calculate long-periodic changes of the mean elements             *)
//  (* l,l',F,D and L0 as well as dgamma                                *)
//  PROCEDURE LONG_PERIODIC ( T: REAL; VAR DL0,DL,DLS,DF,DD,DGAM: REAL );
void Castro::LONG_PERIODIC(double T_p, double *DL0_p,
					double *DL_p,double *DLS_p,
					double *DF_p,double *DD_p,
					double *DGAM_p)
{
    double SS1,SS2,SS3,SS4,SS5,SS6,SS7;

	SS1=SINE(0.19833+0.05611*T_p); SS2=SINE(0.27869+0.04508*T_p);
	SS3=SINE(0.16827-0.36903*T_p); SS4=SINE(0.34734-5.37261*T_p);
	SS5=SINE(0.10498-5.37899*T_p); SS6=SINE(0.42681-0.41855*T_p);
	SS7=SINE(0.14943-5.37511*T_p);
	*DL0_p= 0.84*SS1+0.31*SS2+14.27*SS3+ 7.26*SS4+ 0.28*SS5+0.24*SS6;
	*DL_p = 2.94*SS1+0.31*SS2+14.27*SS3+ 9.34*SS4+ 1.12*SS5+0.83*SS6;
	*DLS_p =-6.40*SS1                                   -1.89*SS6;
	*DF_p = 0.21*SS1+0.31*SS2+14.27*SS3-88.70*SS4-15.30*SS5+0.24*SS6-1.86*SS7;
	*DD_p = *DL0_p-*DLS_p;
	*DGAM_p  = -3332E-9 * SINE(0.59734-5.37261*T_p)
			-539E-9 * SINE(0.35498-5.37899*T_p)
			 -64E-9 * SINE(0.39943-5.37511*T_p);
}

//(* INIT: calculates the mean elements and their sine and cosine   *)
//(* l mean anomaly of the Moon     l' mean anomaly of the Sun      *)
//(* F mean distance from the node  D  mean elongation from the Sun *)
//  PROCEDURE INIT;
void Castro::INIT()
{
    int I,J,MAX;
    double T2,ARG,FAC;
	double PI2 = 6.283185308;  //(* 2*pi;  pi=3.141592654...        *)
	double ARC = 206264.81;    //(* 3600*180/pi = arcsec per radian *)


	T2=T*T;
	DLAM =0; 
	DS=0; 
	GAM1C=0; 
	SINPI=3422.7000;
	LONG_PERIODIC( T, &DL0,&DL,&DLS,&DF,&DD,&DGAM );
	L0 = PI2*FRAC(0.60643382+1336.85522467*T-0.00000313*T2) + DL0/ARC;
	L  = PI2*FRAC(0.37489701+1325.55240982*T+0.00002565*T2) + DL /ARC;
	LS = PI2*FRAC(0.99312619+  99.99735956*T-0.00000044*T2) + DLS/ARC;
	F  = PI2*FRAC(0.25909118+1342.22782980*T-0.00000892*T2) + DF /ARC;
	D  = PI2*FRAC(0.82736186+1236.85308708*T-0.00000397*T2) + DD /ARC;

	//FOR I := 1 TO 4 DO
	for(I=1;I<=4;I++)
	{
		switch(I)
		{
			case 1:{ARG=L;  
					MAX=4; 
					FAC=1.000002208;
					break;}
			case 2:{ARG=LS; 
					MAX=3; 
					FAC=0.997504612-0.002495388*T;
					break;}
			case 3:{ARG=F;  
					MAX=4; 
					FAC=1.000002708+139.978*DGAM;
					break;}
			case 4:{ARG=D;  
					MAX=6; 
					FAC=1.0;
					break;}
		}
	

		CO[0 +6][I]=1.0; 
		CO[1 +6][I]=cos(ARG)*FAC;
		SI[0 +6][I]=0.0; 
		SI[1 +6][I]=cos(ARG)*FAC;

		//FOR J := 2 TO MAX DO
		for(J=2;J<= MAX;J++)
		{
			ADDTHE(CO[J-1 +6][I],SI[J-1 +6][I],
				CO[1 +6][I],SI[1 +6][I],&CO[J +6][I],&SI[J +6][I]);
		}
		//FOR J := 1 TO MAX DO
		for(J=1;J<= MAX;J++)
		{
			CO[-J +6][I]=CO[J +6][I]; 
			SI[-J +6][I]=-SI[J +6][I];
		}
	}

	
}

//(* TERM calculates X=cos(p*arg1+q*arg2+r*arg3+s*arg4) and   *)
//(*                 Y=sin(p*arg1+q*arg2+r*arg3+s*arg4)       *)
//PROCEDURE TERM(P,Q,R,S:INTEGER;VAR X,Y:REAL);
void Castro::TERM(int P,int Q,int R,int S,
				  double *X,double *Y)
{
    int I[5];//: ARRAY[1..4] OF INTEGER;  
	int K;

	double XX,YY,XXX,YYY,da,db;
	//XX = *X;
	//YY = *Y;

	I[1]=P; I[2]=Q; I[3]=R; I[4]=S;  
	XX=1.0; YY=0.0;
	//FOR K:=1 TO 4 DO
	for(K=1;K<=4;K++)
	{
		da = CO[I[K] +6][K];
		db = SI[I[K] +6][K];
        if(I[K]!=0) 
		{
			ADDTHE(XX,YY,CO[I[K] +6][K],SI[I[K] +6][K],&XXX,&YYY);
			XX=XXX;
			YY=YYY;
		}
	}
	*X = XX;
	*Y = YY;
}

//  PROCEDURE ADDSOL(COEFFL,COEFFS,COEFFG,COEFFP:REAL;P,Q,R,S:INTEGER);
void Castro::ADDSOL(double COEFFL,double COEFFS,
					double COEFFG,double COEFFP,
					int P,int Q,int R,int S)
{
	double  X,Y;

	TERM(P,Q,R,S,&X,&Y);
	DLAM =DLAM +COEFFL*Y; DS   =DS   +COEFFS*Y;
	GAM1C=GAM1C+COEFFG*X; SINPI=SINPI+COEFFP*X;
}

//  PROCEDURE SOLAR1;
void Castro::SOLAR1()
  {
	int dum;
	dum = 0;
	dum = 1;

      ADDSOL(   13.902,   14.06,-0.001,   0.2607,0, 0, 0, 4);
      ADDSOL(    0.403,   -4.01,+0.394,   0.0023,0, 0, 0, 3);
      ADDSOL( 2369.912, 2373.36,+0.601,  28.2333,0, 0, 0, 2);
      ADDSOL( -125.154, -112.79,-0.725,  -0.9781,0, 0, 0, 1);
      ADDSOL(    1.979,    6.98,-0.445,   0.0433,1, 0, 0, 4);
      ADDSOL(  191.953,  192.72,+0.029,   3.0861,1, 0, 0, 2);
      ADDSOL(   -8.466,  -13.51,+0.455,  -0.1093,1, 0, 0, 1);
      ADDSOL(22639.500,22609.07,+0.079, 186.5398,1, 0, 0, 0);
      ADDSOL(   18.609,    3.59,-0.094,   0.0118,1, 0, 0,-1);////
      ADDSOL(-4586.465,-4578.13,-0.077,  34.3117,1, 0, 0,-2);
      ADDSOL(   +3.215,    5.44,+0.192,  -0.0386,1, 0, 0,-3);
      ADDSOL(  -38.428,  -38.64,+0.001,   0.6008,1, 0, 0,-4);
      ADDSOL(   -0.393,   -1.43,-0.092,   0.0086,1, 0, 0,-6);
      ADDSOL(   -0.289,   -1.59,+0.123,  -0.0053,0, 1, 0, 4);
      ADDSOL(  -24.420,  -25.10,+0.040,  -0.3000,0, 1, 0, 2);
      ADDSOL(   18.023,   17.93,+0.007,   0.1494,0, 1, 0, 1);
      ADDSOL( -668.146, -126.98,-1.302,  -0.3997,0, 1, 0, 0);
      ADDSOL(    0.560,    0.32,-0.001,  -0.0037,0, 1, 0,-1);
      ADDSOL( -165.145, -165.06,+0.054,   1.9178,0, 1, 0,-2);
      ADDSOL(   -1.877,   -6.46,-0.416,   0.0339,0, 1, 0,-4);
      ADDSOL(    0.213,    1.02,-0.074,   0.0054,2, 0, 0, 4);
      ADDSOL(   14.387,   14.78,-0.017,   0.2833,2, 0, 0, 2);
      ADDSOL(   -0.586,   -1.20,+0.054,  -0.0100,2, 0, 0, 1);
      ADDSOL(  769.016,  767.96,+0.107,  10.1657,2, 0, 0, 0);
      ADDSOL(   +1.750,    2.01,-0.018,   0.0155,2, 0, 0,-1);////
      ADDSOL( -211.656, -152.53,+5.679,  -0.3039,2, 0, 0,-2);
      ADDSOL(   +1.225,    0.91,-0.030,  -0.0088,2, 0, 0,-3);
      ADDSOL(  -30.773,  -34.07,-0.308,   0.3722,2, 0, 0,-4);
      ADDSOL(   -0.570,   -1.40,-0.074,   0.0109,2, 0, 0,-6);
      ADDSOL(   -2.921,  -11.75,+0.787,  -0.0484,1, 1, 0, 2);
      ADDSOL(   +1.267,    1.52,-0.022,   0.0164,1, 1, 0, 1);
      ADDSOL( -109.673, -115.18,+0.461,  -0.9490,1, 1, 0, 0);
      ADDSOL( -205.962, -182.36,+2.056,  +1.4437,1, 1, 0,-2);
      ADDSOL(    0.233,    0.36, 0.012,  -0.0025,1, 1, 0,-3);
}

//  PROCEDURE SOLAR2;
void Castro::SOLAR2()
  {
      ADDSOL(    0.283,    1.53,-0.111,  +0.0060,1,-1, 0,+4);
      ADDSOL(   14.577,   31.70,-1.540,  +0.2302,1,-1, 0, 2);
      ADDSOL(  147.687,  138.76,+0.679,  +1.1528,1,-1, 0, 0);
      ADDSOL(   -1.089,    0.55,+0.021,   0.0   ,1,-1, 0,-1);
      ADDSOL(   28.475,   23.59,-0.443,  -0.2257,1,-1, 0,-2);
      ADDSOL(   -0.276,   -0.38,-0.006,  -0.0036,1,-1, 0,-3);
      ADDSOL(    0.636,    2.27,+0.146,  -0.0102,1,-1, 0,-4);
      ADDSOL(   -0.189,   -1.68,+0.131,  -0.0028,0, 2, 0, 2);
      ADDSOL(   -7.486,   -0.66,-0.037,  -0.0086,0, 2, 0, 0);
      ADDSOL(   -8.096,  -16.35,-0.740,   0.0918,0, 2, 0,-2);
      ADDSOL(   -5.741,   -0.04, 0.0  ,  -0.0009,0, 0, 2, 2);
      ADDSOL(    0.255,    0.0 , 0.0  ,   0.0   ,0, 0, 2, 1);
      ADDSOL( -411.608,   -0.20, 0.0  ,  -0.0124,0, 0, 2, 0);
      ADDSOL(    0.584,    0.84, 0.0  ,  +0.0071,0, 0, 2,-1);
      ADDSOL(  -55.173,  -52.14, 0.0  ,  -0.1052,0, 0, 2,-2);
      ADDSOL(    0.254,    0.25, 0.0  ,  -0.0017,0, 0, 2,-3);
      ADDSOL(   +0.025,   -1.67, 0.0  ,  +0.0031,0, 0, 2,-4);
      ADDSOL(    1.060,    2.96,-0.166,   0.0243,3, 0, 0,+2);
      ADDSOL(   36.124,   50.64,-1.300,   0.6215,3, 0, 0, 0);
      ADDSOL(  -13.193,  -16.40,+0.258,  -0.1187,3, 0, 0,-2);
      ADDSOL(   -1.187,   -0.74,+0.042,   0.0074,3, 0, 0,-4);
      ADDSOL(   -0.293,   -0.31,-0.002,   0.0046,3, 0, 0,-6);
      ADDSOL(   -0.290,   -1.45,+0.116,  -0.0051,2, 1, 0, 2);
      ADDSOL(   -7.649,  -10.56,+0.259,  -0.1038,2, 1, 0, 0);
      ADDSOL(   -8.627,   -7.59,+0.078,  -0.0192,2, 1, 0,-2);
      ADDSOL(   -2.740,   -2.54,+0.022,   0.0324,2, 1, 0,-4);
      ADDSOL(    1.181,    3.32,-0.212,   0.0213,2,-1, 0,+2);
      ADDSOL(    9.703,   11.67,-0.151,   0.1268,2,-1, 0, 0);
      ADDSOL(   -0.352,   -0.37,+0.001,  -0.0028,2,-1, 0,-1);
      ADDSOL(   -2.494,   -1.17,-0.003,  -0.0017,2,-1, 0,-2);
      ADDSOL(    0.360,    0.20,-0.012,  -0.0043,2,-1, 0,-4);
      ADDSOL(   -1.167,   -1.25,+0.008,  -0.0106,1, 2, 0, 0);
      ADDSOL(   -7.412,   -6.12,+0.117,   0.0484,1, 2, 0,-2);
      ADDSOL(   -0.311,   -0.65,-0.032,   0.0044,1, 2, 0,-4);
      ADDSOL(   +0.757,    1.82,-0.105,   0.0112,1,-2, 0, 2);
      ADDSOL(   +2.580,    2.32,+0.027,   0.0196,1,-2, 0, 0);
      ADDSOL(   +2.533,    2.40,-0.014,  -0.0212,1,-2, 0,-2);
      ADDSOL(   -0.344,   -0.57,-0.025,  +0.0036,0, 3, 0,-2);
      ADDSOL(   -0.992,   -0.02, 0.0  ,   0.0   ,1, 0, 2, 2);
      ADDSOL(  -45.099,   -0.02, 0.0  ,  -0.0010,1, 0, 2, 0);
      ADDSOL(   -0.179,   -9.52, 0.0  ,  -0.0833,1, 0, 2,-2);
      ADDSOL(   -0.301,   -0.33, 0.0  ,   0.0014,1, 0, 2,-4);
      ADDSOL(   -6.382,   -3.37, 0.0  ,  -0.0481,1, 0,-2, 2);
      ADDSOL(   39.528,   85.13, 0.0  ,  -0.7136,1, 0,-2, 0);
      ADDSOL(    9.366,    0.71, 0.0  ,  -0.0112,1, 0,-2,-2);
      ADDSOL(    0.202,    0.02, 0.0  ,   0.0   ,1, 0,-2,-4);
}

//  PROCEDURE SOLAR3;
void Castro::SOLAR3()
{
      ADDSOL(    0.415,    0.10, 0.0  ,  0.0013,0, 1, 2, 0);
      ADDSOL(   -2.152,   -2.26, 0.0  , -0.0066,0, 1, 2,-2);
      ADDSOL(   -1.440,   -1.30, 0.0  , +0.0014,0, 1,-2, 2);
      ADDSOL(    0.384,   -0.04, 0.0  ,  0.0   ,0, 1,-2,-2);
      ADDSOL(   +1.938,   +3.60,-0.145, +0.0401,4, 0, 0, 0);
      ADDSOL(   -0.952,   -1.58,+0.052, -0.0130,4, 0, 0,-2);
      ADDSOL(   -0.551,   -0.94,+0.032, -0.0097,3, 1, 0, 0);
      ADDSOL(   -0.482,   -0.57,+0.005, -0.0045,3, 1, 0,-2);
      ADDSOL(    0.681,    0.96,-0.026,  0.0115,3,-1, 0, 0);
      ADDSOL(   -0.297,   -0.27, 0.002, -0.0009,2, 2, 0,-2);
      ADDSOL(    0.254,   +0.21,-0.003,  0.0   ,2,-2, 0,-2);
      ADDSOL(   -0.250,   -0.22, 0.004,  0.0014,1, 3, 0,-2);
      ADDSOL(   -3.996,    0.0 , 0.0  , +0.0004,2, 0, 2, 0);
      ADDSOL(    0.557,   -0.75, 0.0  , -0.0090,2, 0, 2,-2);
      ADDSOL(   -0.459,   -0.38, 0.0  , -0.0053,2, 0,-2, 2);
      ADDSOL(   -1.298,    0.74, 0.0  , +0.0004,2, 0,-2, 0);
      ADDSOL(    0.538,    1.14, 0.0  , -0.0141,2, 0,-2,-2);
      ADDSOL(    0.263,    0.02, 0.0  ,  0.0   ,1, 1, 2, 0);
      ADDSOL(    0.426,   +0.07, 0.0  , -0.0006,1, 1,-2,-2);
      ADDSOL(   -0.304,   +0.03, 0.0  , +0.0003,1,-1, 2, 0);
      ADDSOL(   -0.372,   -0.19, 0.0  , -0.0027,1,-1,-2, 2);
      ADDSOL(   +0.418,    0.0 , 0.0  ,  0.0   ,0, 0, 4, 0);
      ADDSOL(   -0.330,   -0.04, 0.0  ,  0.0   ,3, 0, 2, 0);
}

//  (* part N of the perturbations of ecliptic latitude                 *)
//  PROCEDURE SOLARN(VAR N: REAL);
void Castro::SOLARN(double *N)
{
    //double X,Y;
    //PROCEDURE ADDN(:REAL;P,Q,R,S:INTEGER);
    //  BEGIN TERM(P,Q,R,S,X,Y); N:=N+COEFFN*Y END;
      NN = 0.0;//use class global, ADDN was sub of this.
      ADDN(-526.069, 0, 0,1,-2); 
	  ADDN(  -3.352, 0, 0,1,-4);
      ADDN( +44.297,+1, 0,1,-2); 
	  ADDN(  -6.000,+1, 0,1,-4);
      ADDN( +20.599,-1, 0,1, 0); 
	  ADDN( -30.598,-1, 0,1,-2);
      ADDN( -24.649,-2, 0,1, 0); 
	  ADDN(  -2.000,-2, 0,1,-2);
      ADDN( -22.571, 0,+1,1,-2); 
	  ADDN( +10.985, 0,-1,1,-2);
	  *N = NN;
}

//    PROCEDURE ADDN(COEFFN:REAL;P,Q,R,S:INTEGER);
//was sub of SOLARN, need NN as global
void Castro:: ADDN(double A_COEFFN,int P,int Q,int R,int S)
{
	double X,Y;
	TERM(P,Q,R,S,&X,&Y);
	

	NN=NN+A_COEFFN*Y ;
}

//  (* perturbations of ecliptic latitude by Venus and Jupiter          *)
//  PROCEDURE PLANETARY(VAR DLAM:REAL);
void Castro::PLANETARY(double *DLAM)
{

      *DLAM  = *DLAM
        +0.82*SINE(0.7736  -62.5512*T)+0.31*SINE(0.0466 -125.1025*T)
        +0.35*SINE(0.5785  -25.1042*T)+0.66*SINE(0.4591+1335.8075*T)
        +0.64*SINE(0.3130  -91.5680*T)+1.14*SINE(0.1480+1331.2898*T)
        +0.21*SINE(0.5918+1056.5859*T)+0.44*SINE(0.5784+1322.8595*T)
        +0.24*SINE(0.2275   -5.7374*T)+0.28*SINE(0.2965   +2.6929*T)
        +0.33*SINE(0.3132   +6.3368*T);
}

