//starcat, Star Catalog Visualization Tool
// (C) 2000  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


// astro.h: interface for the Castro class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ASTRO_H__5A2866C5_C9E4_11D3_AB4E_444553540000__INCLUDED_)
#define AFX_ASTRO_H__5A2866C5_C9E4_11D3_AB4E_444553540000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <math.h>

	//REAL33 type from Pascal
	typedef struct tagREAL33
	{
		double	A11;
		double	A12;
		double	A13;
		double	A21;
		double	A22;
		double	A23;
		double	A31;
		double	A32;
		double	A33;      
	} REAL33;

class Castro  
{
public:
	Castro();
	virtual ~Castro();

	double SN(double X);
	double CS(double X);
	double TN(double X);
	double ATN(double X);
	double ATN2(double Y, double X);
	void POLAR(double X, double Y, double Z,
			   double *R, double *THETA, double *PHI);
	void CART(double R, double THETA, double PHI,
				  double *X, double *Y, double *Z);
	void DMS(double DD, 
			 int *D, int *M, double *S);
	double MJD(int DAY, int MONTH, int YEAR, double HOUR);
	void ECLEQU(double T, 
				double *X, double *Y, double *Z);
	void PMATECL(double T1, double T2, 
						REAL33 *A);
	void PRECART(REAL33 A, double *X, double *Y,double *Z);

	void NUTEQU(double T,double *X, double *Y,double *Z);
	double NU_FRAC(double X);

	void GEOCEN(double T, double LP,double BP,double RP, 
				double LS,double BS,double RS,
				int IPLAN, int IMODE,
				double *XP,double *YP,double *ZP,
				double *XS,double *YS,double *ZS, 
				double *X,double *Y,double *Z,
				double  *DELTA0);
	double GC_FRAC(double X);
	void GC_POSVEL(double L,double B,double R,double DL,
					   double DB,double DR,
					   double *X,double *Y,double *Z,
					   double *VX,double *VY,double *VZ);

	//functions for planets
	double FRAC(double X);
	void ADDTHE(double C1,double S1,double C2,double S2,
				double *C,double *S);

	void SAT200(double TT,
				double *L,double *B, double *R);
	void S_TERM(int I6,int I,int IT,
					double DLC,double DLS,double DRC,double DRS,
					double DBC,double DBS);
	void S_PERTJUP();
	void S_PERTURA();
	void S_PERTNEP();
	void S_PERTJUR();

	void SUN200(double T, double *L, double *B,double *R);
	void SN_TERM(int I1,int I,int IT,
				double DLC,double DLS,double DRC,double DRS,
				double DBC,double DBS);
	void SN_PERTVEN();
	void SN_PERTMAR();
	void SN_PERTJUP();
	void SN_PERTSAT();
	void SN_PERTMOO();

	void MER200(double TT, double *L, double *B, double *R);
	void M_TERM(int I1, int I, int IT,
				double DLC,double DLS,double DRC,double DRS,
				double DBC,double DBS);
	void M_PERTVEN();
	void M_PERTEAR();
	void M_PERTJUP();
	void M_PERTSAT();

	void VEN200(double TT, double *L, double *B, double *R);
	void V_TERM(int I1, int I, int IT,
				double DLC,double DLS,double DRC,double DRS,
				double DBC,double DBS);
	void V_PERTMER();
	void V_PERTEAR();
	void V_PERTMAR();
	void V_PERTJUP();
	void V_PERTSAT();

	void MAR200(double TT, double *L, double *B, double *R);
	void MA_TERM(int I1, int I, int IT,
				double DLC,double DLS,double DRC,double DRS,
				double DBC,double DBS);
	void MA_PERTVEN();
	void MA_PERTEAR();
	void MA_PERTJUP();
	void MA_PERTSAT();

	void JUP200(double TT, double *L, double *B, double *R);
	void J_TERM(int I5, int I, int IT,
				double DLC,double DLS,double DRC,double DRS,
				double DBC,double DBS);
	void J_PERTSAT();
	void J_PERTURA();
	void J_PERTSUR();

	void URA200(double TT, double *L, double *B, double *R);
	void U_TERM(int I7, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS);
	void U_PERTJUP();
	void U_PERTSAT();
	void U_PERTNEP();
	void U_PERTJSU();

	void NEP200(double TT, double *L, double *B, double *R);
	void N_TERM(int I1, int I, int IT,
					   double DLC,double DLS,double DRC,double DRS,
						double DBC,double DBS);
	void N_PERTURA();
	void N_PERTSAT();
	void N_PERTJUP();

	void PLU200(double TT, double *L, double *B, double *R);
	void P_TERM(int I9,int I,double DLC,double DLS,
				double DRC,double DRS,double DBC,double DBS);
	void P_PERTJUP();
	void P_PERTSAT();
	void P_PERTJUS();
	void P_PREC(double T,double *L,double *B);

	void MINI_MOON (double T, double *RA, double *DEC);

	//Browns Moon Formula:
	void MOON(double T, double *LAMBDA, double *BETA,double *R);
	double SINE(double PHI);
	void LONG_PERIODIC(double T, double *DL0,
						double *DL,double *DLS,
						double *DF,double *DD,
						double *DGAM);
	void INIT();
	void TERM(int P,int Q,int R,int S,
			double *X,double *Y);
	void ADDSOL(double COEFFL,double COEFFS,
					double COEFFG,double COEFFP,
					int P,int Q,int R,int S);
	void SOLAR1();
	void SOLAR2();
	void SOLAR3();
	void SOLARN(double *N);
	void ADDN(double REAL,int P,int Q,int R,int S);

	void PLANETARY(double *DLAM);
	void MOONEQU(double T,double *RA,double *DEC,double *R);

private:


//planet eval globals for function in the pascal functions:
//jupiter
//CONST P2=6.283185307;
//  VAR C6,S6:         ARRAY [ 0..11] OF REAL;
//      C,S:           ARRAY [-6.. 1] OF REAL;
// Use 0 offset for C6,S6
// Use +6 offset for C,S

	double C6[12],S6[12];
	double C[12],S[12];
	double	M5,M6,M7,M8,M9;
	double	U,V, DL,DR,DB;
	double T;

	
	//  VAR C3,S3:          ARRAY [-1..7] OF REAL;
    //  C,S:            ARRAY [-8..0] OF REAL;
	//for sun use +1 offset for C3,S3, +8 offset for C,S
	double M1,M2,M3,M4;
	double D,A,UU;
	double C3[9],S3[9];//for sun

	//globals used by MOON
	double DGAM,FAC;
	double DLAM,/*N,*/GAM1C,SINPI;
	double NN;//for ADDN procedure,(was sub procedure)
	double L0, L, LS, F;//, D ,S ;
	double DL0,/*DL,*/DLS,DF,DD,DS;
	//CO,SI: ARRAY[-6..6,1..4] OF REAL;
	//use +6 for first, 0 for second
	double CO[13][5],SI[13][5];

	//mer
	//  VAR C1,S1:          ARRAY [-1..9] OF REAL;
    //  C,S:            ARRAY [-5..0] OF REAL;
	//use +1 for C1, +5 for C,S
	double C1[11],S1[11];

	//ven
	//  VAR C2,S2:             ARRAY [ 0..8] OF REAL;
	//C,S:               ARRAY [-8..0] OF REAL;
	//use 0 for C2, +8 for C
	double C2[9],S2[9];

	//mars
	//  VAR C4,S4:          ARRAY [-2..16] OF REAL;
    //  C,S:            ARRAY [-9.. 0] OF REAL;
	//use +2 for C4, +9 for C
	double C4[19],S4[19];

	//jupiter
	//  VAR C5,S5:        ARRAY [-1..5] OF REAL;
    //  C,S:          ARRAY [-10..0] OF REAL;
	//use +1 for C5, +10 for C
	double C5[7],S5[7];

	//ura
	//  VAR C7,S7:         ARRAY [-2..7] OF REAL;
    //  C,S:           ARRAY [-8..0] OF REAL;
	// use +2 for C7, +8 for C
	double C7[10],S7[10];

	//nep
	//  VAR C8,S8:         ARRAY [ 0..6] OF REAL;
     // C,S:           ARRAY [-6..0] OF REAL;
	//use 0 for C8, +6 for C
	double C8[7],S8[7];

	//plu
	// VAR C9,S9:    ARRAY [ 0..6] OF REAL;
    //  C,S:      ARRAY [-3..2] OF REAL;
	//use 0 for C8, +3 for C
	double C9[7],S9[7];


};

#endif // !defined(AFX_ASTRO_H__5A2866C5_C9E4_11D3_AB4E_444553540000__INCLUDED_)
